package org.apache.torque.gf.maven;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.Mojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.MavenProject;
import org.apache.torque.gf.configuration.UnitDescriptor;
import org.apache.torque.gf.configuration.UnitDescriptor.Packaging;
import org.apache.torque.gf.configuration.controller.Loglevel;
import org.apache.torque.gf.configuration.option.MapOptionsConfiguration;
import org.apache.torque.gf.configuration.option.OptionsConfiguration;
import org.apache.torque.gf.configuration.paths.CustomProjectPaths;
import org.apache.torque.gf.configuration.paths.DefaultTorqueGfPaths;
import org.apache.torque.gf.configuration.paths.Maven2DirectoryProjectPaths;
import org.apache.torque.gf.configuration.paths.Maven2JarProjectPaths;
import org.apache.torque.gf.configuration.paths.ProjectPaths;
import org.apache.torque.gf.control.Controller;

/**
 * Executes a unit of generation within torque-gf.
 *
 * $Id: $
 *
 * @goal generate
 */
public class TorqueGfMojo extends AbstractMojo implements Mojo
{
    /**
     * The packaging type of the generation unit, either "directory" , "jar"
     * or "classpath".
     *
     * @parameter expression="directory"
     * @required
     */
    private String packaging;

    /**
     * The root directory of the project.
     * Has no effect if packaging is "classpath".
     *
     * @parameter expression="${basedir}"
     * @required
     */
    private File projectRootDir;

    /**
     * The configuration directory of the generation unit.
     * Has no effect if packaging is "classpath".
     *
     * @parameter
     */
    private File configDir;

    /**
     * The configuration package of the generation unit.
     * Has only effect if packaging is "classpath".
     *
     * @parameter
     */
    private String configPackage;

    /**
     * The directory where the source files reside.
     *
     * @parameter
     */
    private File sourceDir;

    /**
     * The target directory for files which are generated each time anew.
     *
     * @parameter expression="${project.build.directory}/generated-sources"
     */
    private File newFileTargetDir;

    /**
     * The target directory for files which are not generated each time anew.
     *
     * @parameter expression="${basedir}/src/main/generated-sources"
     */
    private File modifiedFileTargetDir;

    /**
     * The filename of the jar file of the generation unit.
     * Has only effect if packaging is "jar".
     *
     * @parameter
     */
    private String jarFile;

    /**
     * Whether the generated files in newFileTargetDir should be treated
     * as compileable sources.
     *
     * @parameter expression="true"
     */
    private boolean compileNewFileTargetDir;

    /**
     * Whether the generated files in modifiedFileTargetDir should be treated
     * as compileable sources.
     *
     * @parameter expression="true"
     */
    private boolean compileModifiedFileTargetDir;

    /**
     * The config directory of the project overriding the settings.
     * Then, the settings of this directory are used as "child"
     * and the "normal" settings are used as "parent".
     *
     * @parameter
     */
    private File overrideConfigDir;

    /**
     * The Loglevel to use in the generation process. Must be one of
     * trace, debug, info, warn or error.
     *
     * @parameter
     */
    private String loglevel;

    /**
     * Additional options which can be added to the generation process.
     * This overrides any existing options.
     *
     * @parameter
     */
    private Map<String, String> options;

    /**
     * The Maven project this plugin runs in.
     *
     * @parameter expression="${project}"
     * @required
     * @readonly
     */
    private MavenProject project;

    public void execute() throws MojoExecutionException
    {
        Controller controller = new Controller();
        List<UnitDescriptor> unitDescriptors = new ArrayList<UnitDescriptor>();

        UnitDescriptor.Packaging packaging;
        if ("jar".equals(this.packaging))
        {
            packaging = UnitDescriptor.Packaging.JAR;
        }
        else if ("directory".equals(this.packaging))
        {
            packaging = UnitDescriptor.Packaging.DIRECTORY;
        }
        else if ("classpath".equals(this.packaging))
        {
            packaging = UnitDescriptor.Packaging.CLASSPATH;
        }
        else
        {
            throw new IllegalArgumentException(
                    "Unknown packaging " + this.packaging
                        + ", must be either jar or directory");
        }
        getLog().debug("Packaging is " + packaging);

        ProjectPaths defaultProjectPaths;
        if (UnitDescriptor.Packaging.JAR == packaging)
        {
            defaultProjectPaths
                    = new Maven2JarProjectPaths(projectRootDir, jarFile);
        }
        else if (UnitDescriptor.Packaging.DIRECTORY == packaging)
        {
            defaultProjectPaths
                    = new Maven2DirectoryProjectPaths(projectRootDir);
        }
        else if (UnitDescriptor.Packaging.CLASSPATH == packaging)
        {
            defaultProjectPaths
                    = new Maven2DirectoryProjectPaths(projectRootDir);
        }
        else
        {
             throw new IllegalStateException("Unknown packaging" + packaging);
        }

        CustomProjectPaths projectPaths
                = new CustomProjectPaths(defaultProjectPaths);

        if (UnitDescriptor.Packaging.CLASSPATH == packaging)
        {
            if (configPackage == null)
            {
                throw new MojoExecutionException(
                    "configPackage must be set for packaging =\"classpath\"");
            }
            projectPaths.setConfigurationPackage(configPackage);
            projectPaths.setConfigurationDir(null);
        }
        else
        {
            if (configDir != null)
            {
                projectPaths.setConfigurationDir(configDir);
                getLog().debug("Setting config dir to " + configDir.toString());
            }
        }

        if (sourceDir != null)
        {
            projectPaths.setSourceDir(sourceDir);
            getLog().debug("Setting source dir to " + sourceDir.toString());
        }

        if (newFileTargetDir != null)
        {
            projectPaths.setNewFileTargetDir(newFileTargetDir);
            getLog().debug("Setting newFileTargetDir to "
                    + newFileTargetDir.toString());
        }
        if (modifiedFileTargetDir != null)
        {
            projectPaths.setModifiedFileTargetDir(modifiedFileTargetDir);
            getLog().debug("Setting modifiedFileTargetDir to "
                    + modifiedFileTargetDir.toString());
        }
        getLog().debug("ProjectPaths = " + projectPaths);

        OptionsConfiguration optionConfiguration = null;
        if (options != null)
        {
            optionConfiguration = new MapOptionsConfiguration(options);
        }
        Loglevel convertedLoglevel = null;
        if (this.loglevel != null)
        {
            convertedLoglevel = Loglevel.getByKey(loglevel);
        }
        UnitDescriptor unitDescriptor = new UnitDescriptor(
                packaging,
                projectPaths,
                new DefaultTorqueGfPaths(),
                null,
                optionConfiguration,
                convertedLoglevel);
        getLog().debug("unit descriptor created");
        if (overrideConfigDir != null)
        {
            CustomProjectPaths childProjectPaths
                = new CustomProjectPaths(projectPaths);
            childProjectPaths.setConfigurationDir(overrideConfigDir);
            unitDescriptor = new UnitDescriptor(
                    Packaging.DIRECTORY,
                    childProjectPaths,
                    new DefaultTorqueGfPaths(),
                    unitDescriptor,
                    optionConfiguration,
                    convertedLoglevel);
            getLog().debug("child unit descriptor created");
        }
        unitDescriptors.add(unitDescriptor);
        try
        {
            getLog().debug("about to run controller");
            controller.run(unitDescriptors);
            getLog().info("Generation successful");
        }
        catch (Exception e)
        {
            getLog().error(e);
            throw new MojoExecutionException(e.getMessage());
        }

        if (compileNewFileTargetDir
                && projectPaths.getNewFileTargetPath().exists())
        {
            getLog().debug("Adding newFileTargetDir "
                    + projectPaths.getNewFileTargetPath()
                    + " as CompileSourceRoot");

            project.addCompileSourceRoot(
                    projectPaths.getNewFileTargetPath().toString());
        }

        if (compileModifiedFileTargetDir
                && projectPaths.getModifiedFileTargetPath().exists())
        {
            getLog().debug("Adding modifiedFileTargetDir "
                    + projectPaths.getModifiedFileTargetPath()
                    + " as CompileSourceRoot");

            project.addCompileSourceRoot(
                    projectPaths.getModifiedFileTargetPath().toString());
        }
    }

    /**
     * Sets the packaging.
     *
     * @param packaging the packaging, either "jar" or "directory"
     */
    public void setPackaging(String packaging)
    {
        this.packaging = packaging;
    }

    /**
     * Sets the root directory of the project.
     *
     * @param projectRootDir the project root Directory.
     */
    public void setProjectRootDir(File projectRootDir)
    {
        this.projectRootDir = projectRootDir;
    }

    public void setConfigDir(File configDir)
    {
        this.configDir = configDir;
    }

    public void setConfigPackage(String configPackage)
    {
        this.configPackage = configPackage;
    }

    /**
     * Sets the target directory for files which are generated each time anew.
     *
     * @param targetDir the target directory, or null to use the default.
     */
    public void setNewFileTargetDir(File targetDir)
    {
        this.newFileTargetDir = targetDir;
    }

    /**
     * Sets the target directory for files which are not generated
     * each time anew.
     *
     * @param targetDir the target directory, or null to use the default.
     */
    public void setModifiedFileTargetDir(File targetDir)
    {
        this.modifiedFileTargetDir = targetDir;
    }

    /**
     * The path to the jar file to use.
     *
     * @param jarFile the jar file, or null.
     */
    public void setJarFile(String jarFile)
    {
        this.jarFile = jarFile;
    }

    /**
     * Sets the maven project this mojo runs in.
     *
     * @param project the maven project this mojo runs in.
     */
    public void setProject(MavenProject project)
    {
        this.project = project;
    }
}
