package org.apache.torque.templates;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.apache.torque.gf.control.ControllerState;
import org.apache.torque.gf.option.OptionName;
import org.apache.torque.gf.source.transform.SourceTransformerException;

/**
 * The option names which are used in the java code of the templates.
 * Other options may exist which are not used in the java code of the
 * templates.
 *
 * $Id: $
 */
public enum TemplateOptionName implements OptionName
{
    /** The database vendor. See the schema for possible values.*/
    DATABASE("torque.database", true),

    /** The base package of the OM classes.*/
    OM_PACKAGE("torque.om.package", true),

    /**
     * Whether getters for boolean fields should use "is" instead of "get"
     * as prefix.
     */
    OM_USE_IS_FOR_BOOLEAN_GETTERS("torque.om.useIsForBooleanGetters"),

    /** The default type for a column if no type is given. */
    DEFAULT_TYPE("torque.om.column.defaultType"),

    /** Whether bean objects should be generated. */
    GENERATE_BEANS("torque.om.generateBeans"),

    /** Whether manager classes should be used. */
    OM_USE_MANAGERS("torque.om.useManagers"),

    /**
     * Whether the map init classes which initialize all database maps
     * at once should be generated.
     */
    OM_GENERATE_MAP_INIT("torque.om.generateMapInit"),

    /** The default base class for dbObjects. */
    OM_DB_OBJECT_DEFAULT_BASE_CLASS("torque.om.dbObjectDefaultBaseClass"),

    /** The prefix of the name of the mapInit class. */
    OM_MAP_INIT_CLASS_NAME_PREFIX("torque.om.mapInit.mapInitClassNamePrefix"),

    /** The prefix of the name of the mapInit class. */
    OM_MAP_INIT_CLASS_NAME_SUFFIX("torque.om.mapInit.mapInitClassNameSuffix"),

    OM_MANAGER_CLASS_NAME_PREFIX("torque.om.className.managerClassNamePrefix"),

    OM_MANAGER_CLASS_NAME_SUFFIX("torque.om.className.managerClassNameSuffix");

    /**
     * The fully qualified name of the option.
     */
    private String name;

    /**
     * Whether this option must be set or not.
     */
    private boolean required;

    /**
     * Constructor for an option which is not required..
     *
     * @param name the fully qualified name of the option, not null.
     */
    private TemplateOptionName(String name)
    {
        this(name, false);
    }

    /**
     * Constructor.
     *
     * @param name the fully qualified name of the option, not null.
     * @param required whether the option is required.
     */
    private TemplateOptionName(String name, boolean required)
    {
        this.name = name;
        this.required = required;
    }

    /**
     * Returns the name of the option.
     *
     * @return the fully qualified name of the option, not null.
     */
    public String getName()
    {
        return name;
    }

    /**
     * Returns whether this option must be set.
     *
     * @return true if the option must be set, false if it may be set.
     */
    public boolean isRequired()
    {
        return required;
    }

    @Override
    public String toString()
    {
        return name;
    }

    /**
     * Checks whether all required options are set.
     *
     * @param controllerState the current controller state, not null.
     *
     * @throws SourceTransformerException if a required option is not set.
     */
    public static void checkRequiredOptions(ControllerState controllerState)
        throws SourceTransformerException
    {
        for (TemplateOptionName templateOption : values())
        {
            if (templateOption.isRequired())
            {
                Object optionValue
                    = controllerState.getOption(templateOption.getName());
                if (optionValue == null)
                {
                    throw new SourceTransformerException(
                            "Option " + optionValue + " must be set");
                }
            }
        }
    }
}
