package org.apache.torque.templates.transformer;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.apache.torque.gf.control.ControllerState;
import org.apache.torque.gf.source.SourceElement;
import org.apache.torque.gf.source.transform.SourceTransformerException;
import org.apache.torque.templates.TemplateOptionName;
import org.apache.torque.templates.TorqueSchemaAttributeName;
import org.apache.torque.templates.platform.Platform;
import org.apache.torque.templates.platform.PlatformFactory;
import org.apache.torque.templates.typemapping.SchemaType;
import org.apache.torque.templates.typemapping.SqlType;

/**
 * Helper class for retrieving the schema type of a column.
 *
 * $Id: $
 */
public class SchemaTypeHelper
{
    /**
     * Determines the schema type of a column.
     *
     * @param columnElement the source element which defines the column
     *        for which the schema type should be determined; not null.
     * @param controllerState the controller state, not null.
     *
     * @return the schema type of the column, not null.
     *
     * @throws SourceTransformerException if the name attribute is not set
     *         in the column or if the type refers to an unknown type.
     */
    public static SchemaType getSchemaType(
            SourceElement columnElement,
            ControllerState controllerState)
        throws SourceTransformerException
    {
        String columnNameFromSchema
                = (String) columnElement.getAttribute(
                        TorqueSchemaAttributeName.NAME);
        if (columnNameFromSchema == null)
        {
            throw new SourceTransformerException("The attribute "
                    + TorqueSchemaAttributeName.NAME.getName()
                    + " on element "
                    + columnElement.getName()
                    + " is null");
        }
        SchemaType schemaType = null;
        String schemaTypeString = (String) columnElement.getAttribute(
                TorqueSchemaAttributeName.TYPE.getName());
        // TODO: should there be a default type ? If not, throw an exception.
        if (schemaTypeString == null)
        {
            schemaTypeString = controllerState.getStringOption(
                    TemplateOptionName.DEFAULT_TYPE);
        }
        schemaType = SchemaType.valueOf(schemaTypeString);
        if (schemaType == null)
        {
            throw new SourceTransformerException("Unknown type "
                    + schemaTypeString
                    + " in Column "
                    + columnNameFromSchema);
        }
        return schemaType;
    }

    /**
     * Returns the SQL type for a schema type and the specified target database.
     *
     * @param schemaType the schema type for which the domain should be
     *        determined, not null.
     * @param controllerState the controller state, not null.
     *
     * @return the domain for the schema type, or null if no domain
     *         exists for the schema type.
     */
    public static SqlType getSqlType(
            SchemaType schemaType,
            ControllerState controllerState,
            String size,
            String scale,
            String defaultValue)
    {
        Platform platform = PlatformFactory.getPlatformFor(
                controllerState.getStringOption(TemplateOptionName.DATABASE));
        SqlType platformSqlType = platform.getSqlTypeForSchemaType(schemaType);
        SqlType result
                = new SqlType(platformSqlType, size, scale, defaultValue);
        return result;
    }
}
