package org.apache.torque.templates.transformer.om;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.apache.commons.lang.StringUtils;
import org.apache.torque.gf.control.ControllerState;
import org.apache.torque.templates.TemplateOptionName;
import org.apache.torque.templates.typemapping.JavaType;

/**
 * Generates method names from field names.
 *
 * $Id: $
 */
final class FieldHelper
{
    /**
     * Private constructor for utility class.
     */
    private FieldHelper()
    {
    }

    /**
     * Returns the name of the getter method for a field.
     * "get" is used as prefix, except if the option
     * <code>USE_IS_FOR_GETTERS_OPTION_NAME</code> is set to true and
     * the field type is "boolean", in which case "is" is used as prefix.
     *
     * @param fieldName the name of the field, not null.
     * @param fieldType the type of the field, not null.
     * @param controllerState the current controller state, not null.
     *
     * @return the getter method name, not null.
     */
    public static String getGetterName(
            String fieldName,
            String fieldType,
            ControllerState controllerState)
    {
        String getterName;
        if (controllerState.getBooleanOption(
                TemplateOptionName.OM_USE_IS_FOR_BOOLEAN_GETTERS)
            && JavaType.BOOLEAN_PRIMITIVE.getClassName().equals(fieldType))
        {
            getterName = "is" + StringUtils.capitalize(fieldName);
        }
        else
        {
            getterName = "get" + StringUtils.capitalize(fieldName);
        }
        return getterName;
    }

    /**
     * Returns the name of the setter method for a field.
     * Uses "set" as prefix.
     *
     * @param fieldName the name of the field, not null.
     *
     * @return the setter method name, not null.
     */
    public static String getSetterName(String fieldName)
    {
        String setterName = "set" + StringUtils.capitalize(fieldName);
        return setterName;
    }

    /**
     * Returns the name of the adder method for a field.
     * Uses "add" as prefix.
     *
     * @param fieldName the name of the field, not null.
     *
     * @return the adder method name, not null.
     */
    public static String getAdderName(String fieldName)
    {
        String setterName = "add" + StringUtils.capitalize(fieldName);
        return setterName;
    }

    /**
     * Returns the name of the initializer method for a field.
     * Uses "init" as prefix and "s" as suffix.
     *
     * @param fieldName the name of the field, not null.
     *
     * @return the initializer method name, not null.
     */
    public static String getInitializerName(String fieldName)
    {
        String setterName = "init" + StringUtils.capitalize(fieldName) + "s";
        return setterName;
    }
}
