package org.apache.torque.templates.transformer.om;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.apache.commons.lang.StringUtils;
import org.apache.torque.gf.control.ControllerState;
import org.apache.torque.gf.processor.string.Camelbacker;
import org.apache.torque.gf.source.SourceElement;
import org.apache.torque.gf.source.transform.SourceTransformerException;
import org.apache.torque.templates.TorqueSchemaAttributeName;
import org.apache.torque.templates.TorqueSchemaElementName;
import org.apache.torque.templates.TorqueSchemaJavaType;
import org.apache.torque.templates.transformer.SchemaTypeHelper;
import org.apache.torque.templates.typemapping.JavaType;
import org.apache.torque.templates.typemapping.SchemaType;
import org.apache.torque.templates.typemapping.TypeMap;
import org.apache.torque.templates.typemapping.VillageMethod;

/**
 * Sets the class names and packages for the OM model.
 */
public class OMColumnTransformer
{
    private static Camelbacker camelbacker = new Camelbacker();

    private static OMInheritanceTransformer inheritanceTransformer
            = new OMInheritanceTransformer();

    public void transform(
            SourceElement columnElement,
            ControllerState controllerState,
            int columnPosition)
        throws SourceTransformerException
    {
        checkElementName(columnElement);
        checkColumnNameExists(columnElement);

        columnElement.setAttribute(
                ColumnAttributeName.POSITION,
                columnPosition);
        setAttributeDefaultValues(columnElement);

        SchemaType schemaType = SchemaTypeHelper.getSchemaType(
                columnElement,
                controllerState);

        JavaType fieldJavaType = getFieldJavaType(columnElement, schemaType);
        columnElement.setAttribute(
                JavaFieldAttributeName.FIELD_TYPE,
                fieldJavaType.getFullClassName());


        setPrimitiveTypeAttribute(columnElement, fieldJavaType);
        setNumberTypeAttribute(columnElement, fieldJavaType);
        setJavaNameAttribute(columnElement);
        setFieldNameAttribute(columnElement);
        setPeerColumnNameAttribute(columnElement);
        setQualifiedColumnNameAttribute(columnElement);
        setGetterNameAttribute(columnElement, fieldJavaType, controllerState);
        setSetterNameAttribute(columnElement);
        setAccessModifierAttributes(columnElement);
        setDefaultValueAttribute(columnElement, fieldJavaType);
        setVillageMethodAttribute(columnElement, fieldJavaType, schemaType);
        setSampleObjectAttribute(columnElement, schemaType);

        for (SourceElement inheritanceElement : columnElement.getChildren(
                TorqueSchemaElementName.INHERITANCE.getName()))
        {
            inheritanceTransformer.transform(
                    inheritanceElement,
                    controllerState);
        }
    }

    /**
     * Sets default values for attributes of the column element
     * if the attribute is not set.
     * The following attributes are checked:
     * autoIncrement, protected, inheritance, required, primaryKey
     *
     * @param columnElement the column element, not null.
     */
    private void setAttributeDefaultValues(SourceElement columnElement)
    {
        // set autoincrement attribute
        // TODO: remove once the autoincrement attribute is removed
        // from the Torque schema.
        if (columnElement.getAttribute(
                    TorqueSchemaAttributeName.AUTO_INCREMENT)
                == null)
        {
            columnElement.setAttribute(
                TorqueSchemaAttributeName.AUTO_INCREMENT,
                Boolean.FALSE.toString());
        }

        // set autoincrement attribute
        if (columnElement.getAttribute(
                    TorqueSchemaAttributeName.PROTECTED)
                == null)
        {
            columnElement.setAttribute(
                TorqueSchemaAttributeName.PROTECTED,
                Boolean.FALSE.toString());
        }

        // set inheritance attribute
        if (columnElement.getAttribute(
                    TorqueSchemaAttributeName.INHERITANCE)
                == null)
        {
            columnElement.setAttribute(
                TorqueSchemaAttributeName.INHERITANCE,
                Boolean.FALSE.toString());
        }

        Object required = columnElement.getAttribute(
                TorqueSchemaAttributeName.REQUIRED);
        if (required == null)
        {
            columnElement.setAttribute(
                    TorqueSchemaAttributeName.REQUIRED,
                    Boolean.FALSE.toString());
        }

        Object primaryKey = columnElement.getAttribute(
                TorqueSchemaAttributeName.PRIMARY_KEY);
        if (primaryKey == null)
        {
            columnElement.setAttribute(
                    TorqueSchemaAttributeName.PRIMARY_KEY,
                    Boolean.FALSE.toString());
        }
    }

    /**
     * Sets the attributes getterAccessModifer and setterAccessModifer
     * on the column element.
     *
     * @param columnElement the column element, not null.
     */
    private void setAccessModifierAttributes(SourceElement columnElement)
    {
        boolean isProtected = "true".equals(
                 columnElement.getAttribute(
                         TorqueSchemaAttributeName.PROTECTED));

        String accessModifier;
        if (isProtected)
        {
            accessModifier = "protected";
        }
        else
        {
            accessModifier = "public";
        }
        columnElement.setAttribute(
                JavaFieldAttributeName.GETTER_ACCESS_MODIFIER,
                accessModifier);
        columnElement.setAttribute(
                JavaFieldAttributeName.SETTER_ACCESS_MODIFIER,
                accessModifier);
    }


    /**
     * Returns the java type of the field representing a database column.
     *
     * @param columnElement the column element, not null.
     * @param schemaType the schema type, not null.
     *
     * @return the java type of the column
     */
    protected JavaType getFieldJavaType(
                SourceElement columnElement,
                SchemaType schemaType)
            throws SourceTransformerException
    {
        JavaType result;
        String javaType = (String) columnElement.getAttribute(
                TorqueSchemaAttributeName.JAVA_TYPE);
        if (TorqueSchemaJavaType.OBJECT.getValue().equals(javaType))
        {
            result = TypeMap.getJavaObjectType(schemaType);
        }
        else if (TorqueSchemaJavaType.PRIMITIVE.getValue().equals(javaType)
                 || javaType == null)
        {
            result = TypeMap.getJavaPrimitiveType(schemaType);
        }
        else
        {
            String columnName = (String) columnElement.getAttribute(
                    TorqueSchemaAttributeName.NAME);
            throw new SourceTransformerException("Unknown javaType "
                    + javaType
                    + " in column "
                    + columnName);
        }
        return result;
    }

    /**
     * Checks that the name of the column element is correct.
     *
     * @param columnElement the column element, not null.
     *
     * @throws IllegalArgumentException if the element name is wrong.
     */
    protected void checkElementName(SourceElement columnElement)
    {
        if (!TorqueSchemaElementName.COLUMN.getName().equals(
                columnElement.getName()))
        {
            throw new IllegalArgumentException("Illegal element Name "
                    + columnElement.getName());
        }
    }

    /**
     * Checks that the name attribute exists on the column element.
     *
     * @param columnElement the column element, not null.
     *
     * @throws SourceTransformerException if the name attribute does not exist.
     */
    protected void checkColumnNameExists(SourceElement columnElement)
            throws SourceTransformerException
    {
        String columnName
                = (String) columnElement.getAttribute(
                        TorqueSchemaAttributeName.NAME);
        if (columnName == null)
        {
            throw new SourceTransformerException("The attribute "
                    + TorqueSchemaAttributeName.NAME.getName()
                    + " on element "
                    + columnElement.getName()
                    + " is null");
        }
    }

    /**
     * Sets the javaName attribute of the column element if it is not
     * already set.
     *
     * @param columnElement the column element, not null.
     */
    protected void setJavaNameAttribute(SourceElement columnElement)
    {
        if (columnElement.getAttribute(TorqueSchemaAttributeName.JAVA_NAME)
                != null)
        {
            return;
        }
        String columnName = (String) columnElement.getAttribute(
                TorqueSchemaAttributeName.NAME);
        String javaName = camelbacker.process(columnName);
        columnElement.setAttribute(
                TorqueSchemaAttributeName.JAVA_NAME,
                javaName);
    }

    /**
     * Sets the fieldName attribute of the column element if it is not
     * already set.
     * The javaName attribute of the column must be set.
     *
     * @param columnElement the column element, not null.
     */
    protected void setFieldNameAttribute(SourceElement columnElement)
    {
        if (columnElement.getAttribute(JavaFieldAttributeName.FIELD_NAME)
                != null)
        {
            return;
        }
        String javaName = (String) columnElement.getAttribute(
                TorqueSchemaAttributeName.JAVA_NAME);
        String fieldName = StringUtils.uncapitalize(javaName);
        columnElement.setAttribute(
                JavaFieldAttributeName.FIELD_NAME,
                fieldName);
    }

    /**
     * Sets the peerColumnName attribute of the column element if it is not
     * already set.
     *
     * @param columnElement the column element, not null.
     */
    protected void setPeerColumnNameAttribute(SourceElement columnElement)
    {
        if (columnElement.getAttribute(ColumnAttributeName.PEER_COLUMN_NAME)
                != null)
        {
            return;
        }
        String columnName = (String) columnElement.getAttribute(
                TorqueSchemaAttributeName.NAME);
        String peerColumnName = columnName.toUpperCase();
        columnElement.setAttribute(
                ColumnAttributeName.PEER_COLUMN_NAME,
                peerColumnName);
    }

    /**
     * Sets the qualifiedColumnName attribute of the column element
     * if it is not already set.
     *
     * @param columnElement the column element, not null.
     */
    protected void setQualifiedColumnNameAttribute(SourceElement columnElement)
    {
        if (columnElement.getAttribute(ColumnAttributeName.QUALIFIED_COLUMN_NAME)
                != null)
        {
            return;
        }
        String tableName = (String) columnElement.getParent().getAttribute(
                TorqueSchemaAttributeName.NAME);
        String columnName = (String) columnElement.getAttribute(
                TorqueSchemaAttributeName.NAME);
        String qualifiedColumnName
                = tableName + "." + columnName;
        columnElement.setAttribute(
                ColumnAttributeName.QUALIFIED_COLUMN_NAME,
                qualifiedColumnName);
    }

    /**
     * Sets the getterName attribute of the column element
     * if it is not already set.
     * The fieldName attribute of the column element must already be set.
     *
     * @param columnElement the column element, not null.
     * @param javaType the java type of the column, not null.
     * @param controllerState the controller state, not null.
     */
    protected void setGetterNameAttribute(
            SourceElement columnElement,
            JavaType javaType,
            ControllerState controllerState)
    {
        if (columnElement.getAttribute(JavaFieldAttributeName.GETTER_NAME)
                != null)
        {
            return;
        }
        String fieldName = (String) columnElement.getAttribute(
                JavaFieldAttributeName.FIELD_NAME);
        String getterName = FieldHelper.getGetterName(
                fieldName,
                javaType.getFullClassName(),
                controllerState);
        columnElement.setAttribute(
                JavaFieldAttributeName.GETTER_NAME,
                getterName);
    }

    /**
     * Sets the setterName attribute of the column element
     * if it is not already set.
     * The fieldName attribute of the column element must already be set.
     *
     * @param columnElement the column element, not null.
     */
    protected void setSetterNameAttribute(SourceElement columnElement)
    {
        if (columnElement.getAttribute(JavaFieldAttributeName.SETTER_NAME)
                != null)
        {
            return;
        }
        String fieldName = (String) columnElement.getAttribute(
                JavaFieldAttributeName.FIELD_NAME);
        String setterName = FieldHelper.getSetterName(fieldName);
        columnElement.setAttribute(
                JavaFieldAttributeName.SETTER_NAME,
                setterName);
    }

    /**
     * Sets the primitiveType attribute of the column element
     * if it is not already set.
     *
     * @param columnElement the column element, not null.
     * @param javaType the type of the java field corresponding to the
     *        column, not null.
     */
    protected void setPrimitiveTypeAttribute(
            SourceElement columnElement,
            JavaType javaType)
    {
        if (columnElement.getAttribute(ColumnAttributeName.PRIMITIVE_TYPE)
                != null)
        {
            return;
        }
        boolean primitiveFieldType = javaType.isPrimitive();
        columnElement.setAttribute(
                ColumnAttributeName.PRIMITIVE_TYPE,
                Boolean.toString(primitiveFieldType));
    }

    /**
     * Sets the numberType attribute of the column element
     * if it is not already set.
     *
     * @param columnElement the column element, not null.
     * @param javaType the type of the java field corresponding to the
     *        column, not null.
     */
    protected void setNumberTypeAttribute(
            SourceElement columnElement,
            JavaType javaType)
    {
        if (columnElement.getAttribute(ColumnAttributeName.NUMBER_TYPE)
                != null)
        {
            return;
        }
        boolean numberFieldType = javaType.isNumber();
        columnElement.setAttribute(
                ColumnAttributeName.NUMBER_TYPE,
                Boolean.toString(numberFieldType));
    }


    /**
     * Sets the defaultValue attribute of the column element
     * if it is not already set.
     *
     * @param columnElement the column element, not null.
     * @param javaType the type of the java field corresponding to the
     *        column, not null.
     *
     * @throws SourceTransformerException if an unknown primitive type
     *         is encountered
     */
    protected void setDefaultValueAttribute(
                SourceElement columnElement,
                JavaType javaType)
            throws SourceTransformerException
    {
        if (columnElement.getAttribute(JavaFieldAttributeName.DEFAULT_VALUE)
                != null)
        {
            return;
        }
        String defaultValue = (String) columnElement.getAttribute(
                TorqueSchemaAttributeName.DEFAULT.getName());
        boolean primitiveFieldType = javaType.isPrimitive();

        String fieldDefaultValue;
        if (defaultValue != null)
        {
            if (JavaType.BOOLEAN_PRIMITIVE == javaType)
            {
                if ("Y".equals(defaultValue)
                    || "1".equals(defaultValue)
                    || "true".equalsIgnoreCase(defaultValue))
                {
                    fieldDefaultValue = "true";
                }
                else
                {
                    fieldDefaultValue = "false";
                }
            }
            else if (JavaType.BOOLEAN_OBJECT == javaType)
            {
                if ("Y".equals(defaultValue)
                    || "1".equals(defaultValue)
                    || "true".equalsIgnoreCase(defaultValue))
                {
                    fieldDefaultValue = "Boolean.TRUE";
                }
                else
                {
                    fieldDefaultValue = "Boolean.FALSE";
                }
            }
            else if (JavaType.STRING == javaType)
            {
                fieldDefaultValue = "\"" + defaultValue + "\"";
            }
            else if (JavaType.SHORT_OBJECT == javaType)
            {
                // The following is better than casting with (short)
                // because a range check is performed,
                fieldDefaultValue = "new Short(\"" + defaultValue + "\")";
            }
            else if (JavaType.BYTE_OBJECT == javaType)
            {
                // The following is better than casting with (byte)
                // because a range check is performed,
                fieldDefaultValue = "new Byte(\"" + defaultValue + "\")";
            }
            else if (primitiveFieldType)
            {
                fieldDefaultValue = defaultValue;
            }
            else
            {
                fieldDefaultValue
                        = "new " + javaType.getFullClassName()
                            + "(" + defaultValue + ")";
            }
        }
        else
        {
            if (primitiveFieldType)
            {
                if (JavaType.BOOLEAN_PRIMITIVE == javaType)
                {
                    fieldDefaultValue = "false";
                }
                else if (JavaType.BYTE_PRIMITIVE == javaType)
                {
                    fieldDefaultValue = "(byte) 0";
                }
                else if (JavaType.SHORT_PRIMITIVE == javaType)
                {
                    fieldDefaultValue = "(short) 0";
                }
                else if (JavaType.INTEGER_PRIMITIVE == javaType)
                {
                    fieldDefaultValue = "0";
                }
                else if (JavaType.LONG_PRIMITIVE == javaType)
                {
                    fieldDefaultValue = "0L";
                }
                else if (JavaType.FLOAT_PRIMITIVE == javaType)
                {
                    fieldDefaultValue = "0";
                }
                else if (JavaType.DOUBLE_PRIMITIVE == javaType)
                {
                    fieldDefaultValue = "0";
                }
                else if (JavaType.CHAR_PRIMITIVE == javaType)
                {
                    fieldDefaultValue = "'\0'";
                }
                else
                {
                    throw new SourceTransformerException(
                            "unknown primitive type" + javaType);
                }
            }
            else
            {
                fieldDefaultValue = "null";
            }
        }
        columnElement.setAttribute(
                JavaFieldAttributeName.DEFAULT_VALUE,
                fieldDefaultValue);
    }

    /**
     * Sets the villageMethod attribute of the column element
     * if it is not already set.
     *
     * @param columnElement the column element, not null.
     * @param javaType the type of the java field corresponding to the
     *        column, not null.
     * @param schemaType the schema type of the column, not null.
     */
    protected void setVillageMethodAttribute(
            SourceElement columnElement,
            JavaType javaType,
            SchemaType schemaType)
    {
        if (columnElement.getAttribute(ColumnAttributeName.VILLAGE_METHOD)
                != null)
        {
            return;
        }
        VillageMethod villageMethod;
        if (javaType.isPrimitive())
        {
            villageMethod = TypeMap.getVillageMethod(schemaType);
        }
        else
        {
            villageMethod = TypeMap.getVillageObjectMethod(schemaType);
        }
        columnElement.setAttribute(
                ColumnAttributeName.VILLAGE_METHOD,
                villageMethod);
    }


    /**
     * Sets the sampleObject attribute of the column element
     * if it is not already set.
     *
     * @param columnElement the column element, not null.
     * @param schemaType the schema type of the column, not null.
     */
    protected void setSampleObjectAttribute(
            SourceElement columnElement,
            SchemaType schemaType)
    {
        if (columnElement.getAttribute(ColumnAttributeName.SAMPLE_OBJECT)
                != null)
        {
            return;
        }

        String sampleObject = TypeMap.getJavaObject(schemaType);
        columnElement.setAttribute(
                ColumnAttributeName.SAMPLE_OBJECT,
                sampleObject);
    }

}
