package org.apache.torque.templates.transformer.om;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.torque.gf.control.ControllerState;
import org.apache.torque.gf.source.RichSourceElementImpl;
import org.apache.torque.gf.source.SourceElement;
import org.apache.torque.gf.source.transform.SourceTransformerException;
import org.apache.torque.templates.TorqueSchemaAttributeName;
import org.apache.torque.templates.TorqueSchemaElementName;

/**
 * Sets the foreign tables for each of the foreign keys,
 * and define variables, getters and setters for the complexObjectModel.
 *
 * So the source elements are (attributes not shown)
 * foreign-key
 *   reference
 *   reference
 *   ...
 *
 * and the result is
 * foreign-key
 *   reference
 *     local-column
 *       column
 *     foreign-column
 *       column
 *   reference
 *     local-column
 *       column
 *     foreign-column
 *       column
 *   ...
 *   local-field (properties for the field on the local table's database object
 *                referencing the foreign database object)
 *   foreign-field (properties for the field on the foreign table's
 *                  database object referencing the local database objects)
 *   table (the foreign referenced table)
 *
 * On running this transformer, the javaName Attribute on the columns
 * must be set properly.
 */
public class OMForeignKeyTransformer
{
    private static final String LOCAL_FIELD_NAME_PREFIX_OPTION
            = "torque.om.complexObjectModel.localFieldNamePrefix";
    private static final String LOCAL_FIELD_NAME_SUFFIX_OPTION
            = "torque.om.complexObjectModel.localFieldNameSuffix";
    private static final String LOCAL_FIELD_NAME_RELATED_BY_OPTION
            = "torque.om.complexObjectModel.localFieldNameRelatedBy";
    private static final String FOREIGN_FIELD_NAME_PREFIX_OPTION
        = "torque.om.complexObjectModel.foreignFieldNamePrefix";
    private static final String FOREIGN_FIELD_NAME_SUFFIX_OPTION
        = "torque.om.complexObjectModel.foreignFieldNameSuffix";
    private static final String FOREIGN_FIELD_NAME_RELATED_BY_OPTION
        = "torque.om.complexObjectModel.foreignFieldNameRelatedBy";
    private static final String FOREIGN_FIELD_TYPE_OPTION
        = "torque.om.complexObjectModel.foreignFieldType";
    private static final String FOREIGN_FIELD_INIT_TYPE_OPTION
        = "torque.om.complexObjectModel.foreignFieldInitType";
    private static final String JAVA5_OPTION
        = "torque.om.java5";

    public void transform(
            SourceElement foreignKey,
            ControllerState controllerState)
        throws SourceTransformerException
    {
        if (!TorqueSchemaElementName.FOREIGN_KEY.getName().equals(
                foreignKey.getName()))
        {
            throw new IllegalArgumentException("Illegal element Name "
                    + foreignKey.getName());
        }

        SourceElement localTable = foreignKey.getParent();
        String localTableName = (String) localTable.getAttribute(
                TorqueSchemaAttributeName.NAME);
        SourceElement database = localTable.getParent();

        String foreignTableName = (String) foreignKey.getAttribute(
                TorqueSchemaAttributeName.FOREIGN_TABLE);
        SourceElement foreignTable
                = FindHelper.findTable(database, foreignTableName);
        foreignKey.addChild(foreignTable);

        for (SourceElement reference : foreignKey.getChildren(
                TorqueSchemaElementName.REFERENCE))
        {
            // create child element "local" for reference
            // and add local column to it
            {
                String localColumnName = (String)
                        reference.getAttribute(
                                TorqueSchemaAttributeName.LOCAL);
                SourceElement localColumnElement
                        = new RichSourceElementImpl(
                                ReferenceChildElementName.LOCAL_COLUMN);
                SourceElement column
                        = FindHelper.findColumn(localTable, localColumnName);
                localColumnElement.addChild(column);
                reference.addChild(localColumnElement);
            }
            // create child element "foreign" for reference
            // and add foreign column
            {
                String foreignColumnName = (String)
                        reference.getAttribute(
                                TorqueSchemaAttributeName.FOREIGN);
                SourceElement foreignColumnElement
                        = new RichSourceElementImpl(
                                ReferenceChildElementName.FOREIGN_COLUMN);
                SourceElement column
                        = FindHelper.findColumn(foreignTable, foreignColumnName);
                foreignColumnElement.addChild(column);
                reference.addChild(foreignColumnElement);
            }
        }

        {
            // create attributes for a field in the local table referencing
            // the foreign table. This field will store one instance of the
            // foreign table's java object.
            StringBuilder referencedBySuffix = new StringBuilder();
            List<SourceElement> referencingSameTable
                    = FindHelper.findForeignKeyByReferencedTable(
                            localTable,
                            foreignTableName);
            if (referencingSameTable.size() > 1
                    || foreignKey.getAttribute(
                            TorqueSchemaAttributeName.FOREIGN_TABLE)
                        .equals(localTableName))
            {
                referencedBySuffix.append((String) controllerState.getOption(
                        LOCAL_FIELD_NAME_RELATED_BY_OPTION));
                for (SourceElement reference : foreignKey.getChildren(
                        TorqueSchemaElementName.REFERENCE))
                {
                    SourceElement localColumnElement
                            = reference.getChildren(
                                    ReferenceChildElementName.LOCAL_COLUMN)
                                .get(0);
                    SourceElement localColumn
                            = localColumnElement.getChildren(
                                    TorqueSchemaElementName.COLUMN)
                                .get(0);
                    String fieldName = (String) localColumn.getAttribute(
                            JavaFieldAttributeName.FIELD_NAME);
                    referencedBySuffix.append(
                            StringUtils.capitalize(fieldName));
                }
            }
            // the field name to create the bean getter and setter names
            String beanGetterSetterFieldName
                    = (String) foreignTable.getAttribute(
                            TableAttributeName.BEAN_CLASS_NAME)
                        + referencedBySuffix;
            {
                // the field name for the variable used
                String localFieldName = (String) controllerState.getOption(
                            LOCAL_FIELD_NAME_PREFIX_OPTION)
                        + foreignTable.getAttribute(
                                TableAttributeName.DB_OBJECT_CLASS_NAME)
                        + controllerState.getOption(
                                    LOCAL_FIELD_NAME_SUFFIX_OPTION)
                        + referencedBySuffix;
                // the field name to create the getter and setter names
                String getterSetterFieldName
                        = (String) foreignTable.getAttribute(
                                TableAttributeName.DB_OBJECT_CLASS_NAME)
                            + referencedBySuffix;

                SourceElement localFieldElement
                        = new RichSourceElementImpl(
                                ForeignKeyChildElementName.LOCAL_FIELD);

                localFieldElement.setAttribute(
                        JavaFieldAttributeName.FIELD_NAME,
                        localFieldName.toString());

                String fieldType = (String) foreignTable.getAttribute(
                        TableAttributeName.DB_OBJECT_CLASS_NAME);
                localFieldElement.setAttribute(
                        JavaFieldAttributeName.FIELD_TYPE,
                        fieldType);
                localFieldElement.setAttribute(
                        JavaFieldAttributeName.DEFAULT_VALUE,
                        "null");
                {
                    String getterName = FieldHelper.getGetterName(
                            getterSetterFieldName,
                            fieldType,
                            controllerState);
                    localFieldElement.setAttribute(
                            JavaFieldAttributeName.GETTER_NAME,
                            getterName);
                }
                {
                    String setterName = FieldHelper.getSetterName(
                            getterSetterFieldName);
                    localFieldElement.setAttribute(
                            JavaFieldAttributeName.SETTER_NAME,
                            setterName);
                    localFieldElement.setAttribute(
                            JavaFieldAttributeName.SETTER_THROWS,
                            "TorqueException");
                }
                {
                    String beanGetterName = FieldHelper.getGetterName(
                            beanGetterSetterFieldName,
                            fieldType,
                            controllerState);
                    localFieldElement.setAttribute(
                            ForeignKeyChildAttributeName.BEAN_GETTER,
                            beanGetterName);
                }
                {
                    String beanSetterName = FieldHelper.getSetterName(
                            beanGetterSetterFieldName);
                    localFieldElement.setAttribute(
                            ForeignKeyChildAttributeName.BEAN_SETTER,
                            beanSetterName);
                }

                foreignKey.addChild(localFieldElement);
            }
            {
                // the field name for the variable used
                String localBeanFieldName = (String) controllerState.getOption(
                            LOCAL_FIELD_NAME_PREFIX_OPTION)
                        + foreignTable.getAttribute(
                                TableAttributeName.BEAN_CLASS_NAME)
                        + controllerState.getOption(
                                    LOCAL_FIELD_NAME_SUFFIX_OPTION)
                        + referencedBySuffix;
                SourceElement localFieldInBeanElement
                        = new RichSourceElementImpl(
                                ForeignKeyChildElementName.LOCAL_FIELD_IN_BEAN);

                localFieldInBeanElement.setAttribute(
                        JavaFieldAttributeName.FIELD_NAME,
                        localBeanFieldName.toString());

                String fieldType = (String) foreignTable.getAttribute(
                        TableAttributeName.BEAN_CLASS_NAME);
                localFieldInBeanElement.setAttribute(
                        JavaFieldAttributeName.FIELD_TYPE,
                        fieldType);
                localFieldInBeanElement.setAttribute(
                        JavaFieldAttributeName.DEFAULT_VALUE,
                        "null");
                {
                    String getterName = FieldHelper.getGetterName(
                            beanGetterSetterFieldName,
                            fieldType,
                            controllerState);
                    localFieldInBeanElement.setAttribute(
                            JavaFieldAttributeName.GETTER_NAME,
                            getterName);
                }
                {
                    String setterName = FieldHelper.getSetterName(
                            beanGetterSetterFieldName);
                    localFieldInBeanElement.setAttribute(
                            JavaFieldAttributeName.SETTER_NAME,
                            setterName);
                }
                foreignKey.addChild(localFieldInBeanElement);

            }
        }

        {
            // create attributes for a field in the foreign table referencing
            // the local table. As more than one foreign object can point
            // to this object, it needs to be a collection.
            StringBuilder referencedBySuffix = new StringBuilder();
            List<SourceElement> referencingSameTable
                    = FindHelper.findForeignKeyByReferencedTable(
                            localTable,
                            foreignTableName);
            if (referencingSameTable.size() > 1)
            {
                referencedBySuffix.append((String) controllerState.getOption(
                        FOREIGN_FIELD_NAME_RELATED_BY_OPTION));
                for (SourceElement reference : foreignKey.getChildren(
                        TorqueSchemaElementName.REFERENCE))
                {
                    SourceElement localColumnElement
                            = reference.getChildren(
                                    ReferenceChildElementName.LOCAL_COLUMN)
                                .get(0);
                    SourceElement localColumn
                            = localColumnElement.getChildren(
                                    TorqueSchemaElementName.COLUMN)
                                .get(0);
                    String fieldName = (String) localColumn.getAttribute(
                            JavaFieldAttributeName.FIELD_NAME);
                    referencedBySuffix.append(
                            StringUtils.capitalize(fieldName));
                }
            }
            // the field name to create the name of the bean getter and setter
            String beanGetterSetterFieldName
                    = (String) localTable.getAttribute(
                            TableAttributeName.BEAN_CLASS_NAME)
                        + referencedBySuffix;
            {
                // the field name for the variable used
                String foreignFieldName = (String) controllerState.getOption(
                            FOREIGN_FIELD_NAME_PREFIX_OPTION)
                        + localTable.getAttribute(
                                TableAttributeName.DB_OBJECT_CLASS_NAME)
                        + controllerState.getOption(
                            FOREIGN_FIELD_NAME_SUFFIX_OPTION)
                        + referencedBySuffix;
                // the field name to create the name of the getter and setter
                String getterSetterFieldName
                        = (String) localTable.getAttribute(
                                TableAttributeName.DB_OBJECT_CLASS_NAME)
                            + referencedBySuffix;
                SourceElement foreignFieldElement
                        = new RichSourceElementImpl(
                                ForeignKeyChildElementName.FOREIGN_FIELD);

                foreignFieldElement.setAttribute(
                        JavaFieldAttributeName.FIELD_NAME,
                        foreignFieldName);
                foreignFieldElement.setAttribute(
                        JavaFieldAttributeName.FIELD_ACCESS_MODIFIER,
                        "protected");
                {
                    // the field name to cache the used Criteria
                    String criteriaFieldName = "last"
                            + StringUtils.capitalize(getterSetterFieldName)
                            + "Criteria";
                    foreignFieldElement.setAttribute(
                            ForeignKeyChildAttributeName
                                    .FOREIGN_COLUMN_CRITERIA_CACHE_FIELD,
                            criteriaFieldName);
                }


                String fieldContainedType = (String) localTable.getAttribute(
                        TorqueSchemaAttributeName.JAVA_NAME);
                foreignFieldElement.setAttribute(
                        JavaFieldAttributeName.FIELD_CONTAINED_TYPE,
                        fieldContainedType);

                String fieldType = (String) controllerState.getOption(
                        FOREIGN_FIELD_TYPE_OPTION);
                if ("true".equals(controllerState.getOption(JAVA5_OPTION)))
                {
                    fieldType = fieldType + "<" + fieldContainedType + ">";
                }
                foreignFieldElement.setAttribute(
                        JavaFieldAttributeName.FIELD_TYPE,
                        fieldType);

                foreignFieldElement.setAttribute(
                        JavaFieldAttributeName.DEFAULT_VALUE,
                        "null");
                {
                    String getterName = FieldHelper.getGetterName(
                                getterSetterFieldName,
                                fieldType,
                                controllerState)
                            + "s";
                    foreignFieldElement.setAttribute(
                            JavaFieldAttributeName.GETTER_NAME,
                            getterName);
                }
                {
                    String setterName = FieldHelper.getSetterName(
                                getterSetterFieldName)
                            + "s";
                    foreignFieldElement.setAttribute(
                            JavaFieldAttributeName.SETTER_NAME,
                                setterName);
                }
                {
                    String adderName = FieldHelper.getAdderName(
                            getterSetterFieldName);
                    foreignFieldElement.setAttribute(
                            JavaFieldAttributeName.ADDER_NAME,
                            adderName);
                }
                {
                    String initializerName = FieldHelper.getInitializerName(
                            getterSetterFieldName);
                    foreignFieldElement.setAttribute(
                            JavaFieldAttributeName.INITIALIZER_NAME,
                            initializerName);
                }
                {
                    String beanGetterName = FieldHelper.getGetterName(
                                beanGetterSetterFieldName,
                                fieldType,
                                controllerState)
                            + "s";
                    foreignFieldElement.setAttribute(
                            ForeignKeyChildAttributeName.BEAN_GETTER,
                            beanGetterName);
                }
                {
                    String beanSetterName = FieldHelper.getSetterName(
                                beanGetterSetterFieldName)
                            + "s";
                    foreignFieldElement.setAttribute(
                            ForeignKeyChildAttributeName.BEAN_SETTER,
                            beanSetterName);
                }
                {
                    String initType = (String) controllerState.getOption(
                            FOREIGN_FIELD_INIT_TYPE_OPTION);
                    if ("true".equals(controllerState.getOption(JAVA5_OPTION)))
                    {
                        initType = initType + "<" + fieldContainedType + ">";
                    }
                    foreignFieldElement.setAttribute(
                            JavaFieldAttributeName.INITIALIZER_TYPE,
                            initType);

                }
                {
                    // Name for a getter in the foreign table to
                    // retrieve entries in the foreign table plus the joined
                    // entries in the local table.
                    String joinGetterFieldName
                            = StringUtils.capitalize(getterSetterFieldName)
                            + "sJoin"
                            + foreignTable.getAttribute(
                                TorqueSchemaAttributeName.JAVA_NAME);
                    String joinGetterName = FieldHelper.getGetterName(
                            joinGetterFieldName,
                            fieldType,
                            controllerState);
                    foreignFieldElement.setAttribute(
                            ForeignKeyChildAttributeName
                                .FOREIGN_FIELD_JOIN_GETTER,
                            joinGetterName);
                }
                {
                    // Name for a getter in the foreign table to
                    // retrieve entries in the foreign table plus all joined
                    // entries, except the entries in the local table.
                    String joinAllExceptGetterFieldName
                            = StringUtils.capitalize(getterSetterFieldName)
                            + "sJoinAllExcept"
                            + foreignTable.getAttribute(
                                    TorqueSchemaAttributeName.JAVA_NAME);
                    String joinAllExceptGetterName = FieldHelper.getGetterName(
                            joinAllExceptGetterFieldName,
                            fieldType,
                            controllerState);
                    foreignFieldElement.setAttribute(
                            ForeignKeyChildAttributeName
                                .FOREIGN_FIELD_JOIN_GETTER,
                            joinAllExceptGetterName);
                }
                {
                    // Name for the doSelectJoinXXX method in the Peer Class
                    // of the foreign table.
                    String peerJoinSelectMethodName
                            = "doSelectJoin"
                                + foreignTable.getAttribute(
                                        TorqueSchemaAttributeName.JAVA_NAME)
                                + referencedBySuffix;
                    foreignFieldElement.setAttribute(
                            ForeignKeyChildAttributeName
                                .PEER_JOIN_SELECT_METHOD,
                            peerJoinSelectMethodName);
                }
                {
                    // Name for the doSelectJoinAllExceptXXX method
                    // in the Peer Class of the foreign table.
                    String peerJoinAllExceptSelectMethodName
                            = "doSelectJoinAllExcept"
                                + foreignTable.getAttribute(
                                        TorqueSchemaAttributeName.JAVA_NAME)
                                + referencedBySuffix;
                    foreignFieldElement.setAttribute(
                            ForeignKeyChildAttributeName
                                .PEER_JOIN_ALL_EXCEPT_SELECT_METHOD,
                            peerJoinAllExceptSelectMethodName);
                }
                foreignKey.addChild(foreignFieldElement);
            }
            {
                // the field name for the variable used
                String foreignFieldInBeanName
                        = (String) controllerState.getOption(
                                FOREIGN_FIELD_NAME_PREFIX_OPTION)
                            + localTable.getAttribute(
                                    TableAttributeName.BEAN_CLASS_NAME)
                            + controllerState.getOption(
                                FOREIGN_FIELD_NAME_SUFFIX_OPTION)
                            + referencedBySuffix;
                SourceElement foreignFieldInBeanElement
                        = new RichSourceElementImpl(
                            ForeignKeyChildElementName.FOREIGN_FIELD_IN_BEAN);

                foreignFieldInBeanElement.setAttribute(
                        JavaFieldAttributeName.FIELD_NAME,
                        foreignFieldInBeanName);
                foreignFieldInBeanElement.setAttribute(
                        JavaFieldAttributeName.FIELD_ACCESS_MODIFIER,
                        "protected");

                String fieldContainedType = (String) localTable.getAttribute(
                        TableAttributeName.BEAN_CLASS_NAME);
                foreignFieldInBeanElement.setAttribute(
                        JavaFieldAttributeName.FIELD_CONTAINED_TYPE,
                        fieldContainedType);

                String fieldType = (String) controllerState.getOption(
                        FOREIGN_FIELD_TYPE_OPTION);
                if ("true".equals(controllerState.getOption(JAVA5_OPTION)))
                {
                    fieldType = fieldType + "<" + fieldContainedType + ">";
                }
                foreignFieldInBeanElement.setAttribute(
                        JavaFieldAttributeName.FIELD_TYPE,
                        fieldType);

                {
                    String initType = (String) controllerState.getOption(
                            FOREIGN_FIELD_INIT_TYPE_OPTION);
                    if ("true".equals(controllerState.getOption(JAVA5_OPTION)))
                    {
                        initType = initType + "<" + fieldContainedType + ">";
                    }
                    foreignFieldInBeanElement.setAttribute(
                            JavaFieldAttributeName.INITIALIZER_TYPE,
                            initType);

                }

                foreignFieldInBeanElement.setAttribute(
                        JavaFieldAttributeName.DEFAULT_VALUE,
                        "null");
                {
                    String getterName = FieldHelper.getGetterName(
                                beanGetterSetterFieldName,
                                fieldType,
                                controllerState)
                            + "s";
                    foreignFieldInBeanElement.setAttribute(
                            JavaFieldAttributeName.GETTER_NAME,
                            getterName);
                }
                {
                    String setterName = FieldHelper.getSetterName(
                                beanGetterSetterFieldName)
                            + "s";
                    foreignFieldInBeanElement.setAttribute(
                            JavaFieldAttributeName.SETTER_NAME,
                                setterName);
                }
                foreignKey.addChild(foreignFieldInBeanElement);
            }
        }
    }
}
