package org.apache.torque.templates.transformer.om;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.InputStreamReader;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.torque.gf.control.ControllerState;
import org.apache.torque.gf.processor.string.Camelbacker;
import org.apache.torque.gf.source.RichSourceElementImpl;
import org.apache.torque.gf.source.SourceElement;
import org.apache.torque.gf.source.transform.AttributeTransformer;
import org.apache.torque.gf.source.transform.SourceTransformerException;
import org.apache.torque.templates.TemplateOptionName;
import org.apache.torque.templates.TorqueSchemaAttributeName;
import org.apache.torque.templates.TorqueSchemaElementName;
import org.apache.torque.templates.TorqueSchemaIdMethod;
import org.apache.torque.templates.TorqueSchemaInheritance;
import org.apache.torque.templates.transformer.CollectAttributeSetTrueTransformer;

/**
 * Transforms the tables in the OM model.
 */
public class OMTableTransformer extends AttributeTransformer
{
    private static OMColumnTransformer columnTransformer
            = new OMColumnTransformer();;

    private static OMReferencingForeignKeyTableTransformer
            referencingForeignKeyTableTransformer
                = new OMReferencingForeignKeyTableTransformer();

    private static CollectAttributeSetTrueTransformer primaryKeyTransformer
            = new CollectAttributeSetTrueTransformer();

    private static Camelbacker camelbacker = new Camelbacker();

    public OMTableTransformer() throws SourceTransformerException
    {
        super(new InputStreamReader(
                OMTableTransformer.class.getResourceAsStream(
                        "OMTableTransformer.properties")));
    }


    public SourceElement transform(
            SourceElement tableElement,
            ControllerState controllerState)
        throws SourceTransformerException
    {
        checkElementName(tableElement);
        setJavaNameAttribute(tableElement);
        setIdMethodAttribute(tableElement);
        setSequenceAttributeIfNecessary(tableElement);

        // sets all the attributes defined in OMTableTransformer.properties
        super.transform(tableElement, controllerState);

        setManagerClassNameAttributes(tableElement, controllerState);

        createInheritanceColumnChildIfNecessary(tableElement);
        primaryKeyTransformer.transform(tableElement, controllerState,
                TorqueSchemaElementName.COLUMN,
                TorqueSchemaAttributeName.PRIMARY_KEY,
                TableChildElementName.PRIMARY_KEYS);

        // position is one based
        int columnPosition = 1;
        for (SourceElement element : tableElement.getChildren(
                TorqueSchemaElementName.COLUMN))
        {
            columnTransformer.transform(
                    element,
                    controllerState,
                    columnPosition);
            columnPosition++;
        }

        referencingForeignKeyTableTransformer.transform(
                tableElement,
                controllerState);
        return tableElement;
    }

    /**
     * Checks that the table element has the correct element name.
     *
     * @param tableElement the table element to check, not null.
     *
     * @throws IllegalArgumentException if the name of the element is wrong.
     */
    private void checkElementName(SourceElement tableElement)
    {
        if (!TorqueSchemaElementName.TABLE.getName().equals(
                tableElement.getName()))
        {
            throw new IllegalArgumentException("Illegal element Name "
                    + tableElement.getName());
        }
    }

    /**
     * Sets the javaName attribute of the table element, if not
     * already set.
     *
     * @param tableElement the table element, not null.
     *
     * @throws SourceTransformerException if both attributes javaName and name
     *         are not set on the tableElement.
     */
    protected void setJavaNameAttribute(SourceElement tableElement)
            throws SourceTransformerException
    {
        // set javaName attribute
        Object javaName = tableElement.getAttribute(
                TorqueSchemaAttributeName.JAVA_NAME);
        if (javaName == null)
        {
            Object inputName = tableElement.getAttribute(
                    TorqueSchemaAttributeName.NAME);
            if (inputName == null)
            {
                throw new SourceTransformerException("The required attribute "
                        + TorqueSchemaAttributeName.NAME
                        + " on element "
                        + tableElement.getName()
                        + " is null");
            }
            javaName = camelbacker.process(inputName.toString());
            tableElement.setAttribute(
                    TorqueSchemaAttributeName.JAVA_NAME,
                    javaName);
        }
    }

    protected void setIdMethodAttribute(SourceElement tableElement)
            throws SourceTransformerException
    {
        Object idMethod = tableElement.getAttribute(
                TorqueSchemaAttributeName.ID_METHOD);
        // TODO: in the 3.3 schema, "null" is the default value for idMethod.
        // Therefore we must test for this explicit string.
        // Can be removed once the dtd/schema is updated.
        if (idMethod == null
            || TorqueSchemaIdMethod.NULL.getName().equals(idMethod))
        {
            idMethod = tableElement.getParent().getAttribute(
                    TorqueSchemaAttributeName.DEFAULT_ID_METHOD);
            if (idMethod == null)
            {
                throw new SourceTransformerException("The attribute "
                        + TorqueSchemaAttributeName.DEFAULT_ID_METHOD
                        + " is not set on the database element "
                        + " and no id method is given on table "
                        + tableElement.getName());
            }
            tableElement.setAttribute(
                    TorqueSchemaAttributeName.ID_METHOD,
                    idMethod);
        }
    }

    protected void setSequenceAttributeIfNecessary(SourceElement tableElement)
    {
        // set sequence attribute
        if (!TorqueSchemaIdMethod.NATIVE.getName().equals(
                tableElement.getAttribute(TorqueSchemaAttributeName.ID_METHOD)))
        {
            return;
        }

        List<SourceElement> idMethodParams = tableElement.getChildren(
                TorqueSchemaElementName.ID_METHOD_PARAMETER);
        if (idMethodParams.isEmpty()
            && tableElement.getAttribute(TableAttributeName.SEQUENCE) == null)
        {
            tableElement.setAttribute(
                    TableAttributeName.SEQUENCE,
                    (String) tableElement.getAttribute(
                          TorqueSchemaAttributeName.NAME)
                        + "_SEQ");
        }
        else
        {
            tableElement.setAttribute(
                    TableAttributeName.SEQUENCE,
                        idMethodParams.get(0).getAttribute(
                            TorqueSchemaAttributeName.VALUE.getName()));
        }
    }


    protected void setManagerClassNameAttributes(
                SourceElement tableElement,
                ControllerState controllerState)
            throws SourceTransformerException
    {
        // Managers must be named after an interface if there is one.
        String interfaceName = (String) tableElement.getAttribute(
                TorqueSchemaAttributeName.INTERFACE);
        if (interfaceName == null)
        {
            String managerClassName = controllerState.getStringOption(
                    TemplateOptionName.OM_MANAGER_CLASS_NAME_PREFIX)
                + tableElement.getAttribute(TorqueSchemaAttributeName.JAVA_NAME)
                + controllerState.getStringOption(
                    TemplateOptionName.OM_MANAGER_CLASS_NAME_SUFFIX);
            tableElement.setAttribute(
                    TableAttributeName.MANAGER_CLASS_NAME,
                    managerClassName);
            String baseManagerClassName = controllerState.getStringOption(
                    "torque.om.className.baseManagerClassNamePrefix")
                + tableElement.getAttribute(TorqueSchemaAttributeName.JAVA_NAME)
                + controllerState.getStringOption(
                    "torque.om.className.baseManagerClassNameSuffix");
            tableElement.setAttribute(
                    TableAttributeName.BASE_MANAGER_CLASS_NAME,
                    baseManagerClassName);
        }
        else
        {
            if (StringUtils.isBlank(interfaceName))
            {
                throw new SourceTransformerException("The attribute "
                        + TorqueSchemaAttributeName.INTERFACE
                        + " is blank on table "
                        + tableElement.getName());
            }
            int dotPosition = interfaceName.lastIndexOf(".");
            if (dotPosition != -1)
            {
                interfaceName = interfaceName.substring(dotPosition + 1);
            }
            if (StringUtils.isBlank(interfaceName))
            {
                throw new SourceTransformerException("The attribute "
                        + TorqueSchemaAttributeName.INTERFACE
                        + " ends with a dot on table "
                        + tableElement.getName());
            }

            // first character upper case
            if (interfaceName.length() == 1)
            {
                interfaceName = interfaceName.toUpperCase();
            }
            else
            {
                interfaceName = interfaceName.substring(0, 1).toUpperCase()
                    + interfaceName.substring(1);
            }

            String managerClassName = controllerState.getOption(
                    TemplateOptionName.OM_MANAGER_CLASS_NAME_PREFIX)
                + interfaceName
                + controllerState.getOption(
                        TemplateOptionName.OM_MANAGER_CLASS_NAME_SUFFIX);
            tableElement.setAttribute(
                    TableAttributeName.MANAGER_CLASS_NAME,
                    managerClassName);
            String baseManagerClassName = controllerState.getOption(
                    "torque.om.className.baseManagerClassNamePrefix")
                + interfaceName
                + controllerState.getOption(
                    "torque.om.className.baseManagerClassNameSuffix");
            tableElement.setAttribute(
                    TableAttributeName.BASE_MANAGER_CLASS_NAME,
                    baseManagerClassName);
        }
    }

    public void createInheritanceColumnChildIfNecessary(
                SourceElement tableElement)
            throws SourceTransformerException
    {
        boolean inheritanceFound = false;
        for (SourceElement columnElement : tableElement.getChildren(
                TorqueSchemaElementName.COLUMN))
        {
            if (TorqueSchemaInheritance.SINGLE.getValue().equals(
                    columnElement.getAttribute(
                        TorqueSchemaAttributeName.INHERITANCE)))
            {
                if (inheritanceFound)
                {
                    throw new SourceTransformerException(
                            "more than one column with "
                                + TorqueSchemaAttributeName.INHERITANCE
                                + " set to \"single\" found in table "
                                + tableElement.getAttribute(
                                        tableElement.getName()));
                }
                SourceElement inheritanceColumnElement
                        = new RichSourceElementImpl(
                                TableChildElementName.INHERITANCE_COLUMN);
                inheritanceColumnElement.addChild(columnElement);
                tableElement.addChild(inheritanceColumnElement);
                inheritanceFound = true;
            }
        }
    }
}
