package org.apache.torque.templates.transformer.om;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.apache.torque.gf.control.ControllerState;
import org.apache.torque.gf.source.SourceElement;
import org.apache.torque.gf.source.transform.SourceTransformer;
import org.apache.torque.gf.source.transform.SourceTransformerException;
import org.apache.torque.templates.TemplateOptionName;
import org.apache.torque.templates.TorqueSchemaElementName;

/**
 * Performs the transformations which are necessary to apply the templates
 * to the source tree.
 * This transformer performs the following actions:
 * <ul>
 *   <li>adds attributes to source elements</li>
 *   <li>links elements to other source elements</li>
 * </ul>
 * No elements or attributes are deleted.
 *
 * TODO: add links to other transformers which are applied to certain elements
 *       of the source tree
 *
 * $Id: $
 */
public class OMTransformer implements SourceTransformer
{
    /**
     * The transformer which adds references to the foreign keys referenced by
     * this column or referencing this column.
     *
     * @see OMForeignKeyColumnTransformer
     */
    private static final OMForeignKeyColumnTransformer foreignKeyColumnTransformer
        = new OMForeignKeyColumnTransformer();

    /**
     * The transformer which adds references to the referenced and referencing
     * columns, tables and fields to the foreign key.
     *
     * @see OMForeignKeyTransformer
     */
    private static final OMForeignKeyTransformer foreignKeyTransformer
        = new OMForeignKeyTransformer();

    /**
     * The transformer which adds transforms the table and column elements
     * of the schema.
     *
     * @see OMTableTransformer
     */
    private static final OMTableTransformer tableTransformer;

    static
    {
        try
        {
            tableTransformer = new OMTableTransformer();
        }
        catch (SourceTransformerException e)
        {
            throw new RuntimeException(e);
        }
    }

    /**
     * Transforms the source tree so it can be used by the om templates.
     *
     * @param root the database root element of the source tree, not null.
     * @param controllerState the controller state, not null.
     *
     * @throws SourceTransformerException if the transformation fails.
     */
    public SourceElement transform(
            SourceElement root,
            ControllerState controllerState)
        throws SourceTransformerException
    {
        TemplateOptionName.checkRequiredOptions(controllerState);

        for (SourceElement tableElement : root.getChildren(
                TorqueSchemaElementName.TABLE.getName()))
        {
            tableTransformer.transform(
                    tableElement,
                    controllerState);
        }

        for (SourceElement tableElement : root.getChildren(
                TorqueSchemaElementName.TABLE.getName()))
        {
            for (SourceElement columnElement : tableElement.getChildren(
                    TorqueSchemaElementName.COLUMN.getName()))
            {
                foreignKeyColumnTransformer.transform(
                        columnElement,
                        controllerState);
            }
            for (SourceElement foreignKeyElement : tableElement.getChildren(
                    TorqueSchemaElementName.FOREIGN_KEY.getName()))
            {
                foreignKeyTransformer.transform(
                        foreignKeyElement,
                        controllerState);
            }
        }
        return root;
    }
}
