package org.apache.torque.templates.transformer.sql;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.ObjectUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.torque.gf.control.ControllerState;
import org.apache.torque.gf.source.SourceElement;
import org.apache.torque.gf.source.SourcePath;
import org.apache.torque.gf.source.transform.SourceTransformer;
import org.apache.torque.gf.source.transform.SourceTransformerException;
import org.apache.torque.templates.TemplateOptionName;
import org.apache.torque.templates.TorqueSchemaAttributeName;
import org.apache.torque.templates.TorqueSchemaElementName;
import org.apache.torque.templates.TorqueSchemaIdMethod;
import org.apache.torque.templates.platform.Platform;
import org.apache.torque.templates.platform.PlatformFactory;
import org.apache.torque.templates.transformer.CollectAttributeSetTrueTransformer;
import org.apache.torque.templates.transformer.SchemaTypeHelper;
import org.apache.torque.templates.transformer.om.TableChildElementName;
import org.apache.torque.templates.typemapping.SqlType;
import org.apache.torque.templates.typemapping.SchemaType;
import org.apache.torque.templates.typemapping.TypeMap;

/**
 * Transforms the tables in the OM model.
 */
public class SQLTransformer implements SourceTransformer
{
    private static final String PRIMARY_KEY_COLUMN_NAMES_ATTRIBUTE
            = "primaryKeyColumnNames";

    private static final String UNIQUE_COLUMN_NAMES_ATTRIBUTE
            = "uniqueColumnNames";

    private static final String INDEX_COLUMN_NAMES_ATTRIBUTE
            = "indexColumnNames";

    private static final String LOCAL_COLUMN_NAMES_ATTRIBUTE
            = "localColumnNames";

    private static final String FOREIGN_COLUMN_NAMES_ATTRIBUTE
            = "foreignColumnNames";

    private static final String PRIMARY_KEY_CONSTRAINT_NAME_ATTRIBUTE
            = "primaryKeyConstraintName";

    private static final String SEQUENCE_NAME_ATTRIBUTE
            = "sequenceName";

    private static final String DDL_SQL_ATTRIBUTE
            = "ddlSql";

    private CollectAttributeSetTrueTransformer collectAttributeSetTrueTransformer
            = new CollectAttributeSetTrueTransformer();

    public SourceElement transform(
            SourceElement databaseElement,
            ControllerState controllerState)
        throws SourceTransformerException
    {
        // TODO: move check to better place
        Object databaseOption = controllerState.getOption("torque.database");
        if (databaseOption == null)
        {
            throw new SourceTransformerException(
                    "Option torque:database must be set");
        }

        List<SourceElement> tableElements = databaseElement.getChildren(
                TorqueSchemaElementName.TABLE.getName());
        for (SourceElement tableElement : tableElements)
        {
            transformTable(tableElement, controllerState);
        }
        return databaseElement;
    }

    public void transformTable(
            SourceElement tableElement,
            ControllerState controllerState)
        throws SourceTransformerException
    {
        Object idMethod = tableElement.getAttribute(
                TorqueSchemaAttributeName.ID_METHOD.getName());
        // TODO: in the 3.3 schema, "null" is the default value for idMethod.
        // Therefore we must test for this explicit string.
        // Can be removed once the dtd/schema is updated.
        if (idMethod == null
            || TorqueSchemaIdMethod.NULL.getName().equals(idMethod))
        {
            Object defaultIdMethod = tableElement.getParent().getAttribute(
                    TorqueSchemaAttributeName.DEFAULT_ID_METHOD.getName());
            if (defaultIdMethod == null)
            {
                throw new SourceTransformerException("id Method is not set"
                        + " on table "
                        + tableElement.getAttribute(
                                TorqueSchemaAttributeName.NAME.getName())
                        + " and defaultIdMethod is not set on database");
            }
            tableElement.setAttribute(
                    TorqueSchemaAttributeName.ID_METHOD.getName(),
                    defaultIdMethod);
        }
        String tableName = (String) tableElement.getAttribute(
                TorqueSchemaAttributeName.NAME.getName());
        if (tableElement.getAttribute(PRIMARY_KEY_CONSTRAINT_NAME_ATTRIBUTE)
                == null)
        {
            // TODO: Column length from database descriptor
            String primaryKeyConstraintName =
                (tableName.length() > 27 ? tableName.substring(0, 26) : tableName)
                  + "_PK";
            tableElement.setAttribute(
                    PRIMARY_KEY_CONSTRAINT_NAME_ATTRIBUTE,
                    primaryKeyConstraintName);
        }
        if (tableElement.getAttribute(SEQUENCE_NAME_ATTRIBUTE)
                == null)
        {
            // TODO: Sequence name length from database descriptor
            String sequenceName =
                (tableName.length() > 27 ? tableName.substring(0, 26) : tableName)
                  + "_SEQ";
            tableElement.setAttribute(
                    SEQUENCE_NAME_ATTRIBUTE,
                    sequenceName);
        }

        // primary keys
        collectAttributeSetTrueTransformer.transform(
                tableElement,
                controllerState,
                TorqueSchemaElementName.COLUMN,
                TorqueSchemaAttributeName.PRIMARY_KEY,
                TableChildElementName.PRIMARY_KEYS);

        StringBuilder primaryKeyColumnNames = new StringBuilder();
        SourceElement primaryKeysElement = tableElement.getChild(
                TableChildElementName.PRIMARY_KEYS);
        boolean firstPk = true;
        for (SourceElement primaryKeyColumn : primaryKeysElement.getChildren())
        {
            if (!firstPk)
            {
                primaryKeyColumnNames.append(", ");
            }
            primaryKeyColumnNames.append(primaryKeyColumn.getAttribute(
                            TorqueSchemaAttributeName.NAME.getName()));
            firstPk = false;
        }
        tableElement.setAttribute(
                PRIMARY_KEY_COLUMN_NAMES_ATTRIBUTE,
                primaryKeyColumnNames.toString());

        // unique
        int uniqueIndex = 1;
        for (SourceElement uniqueElement : tableElement.getChildren(
                TorqueSchemaElementName.UNIQUE.getName()))
        {
            if (uniqueElement.getAttribute(
                        TorqueSchemaAttributeName.NAME.getName())
                    == null)
            {
                uniqueElement.setAttribute(
                        TorqueSchemaAttributeName.NAME.getName(),
                        uniqueElement.getParent().getAttribute(
                               TorqueSchemaAttributeName.NAME.getName())
                            + "_UQ_" + uniqueIndex);
            }
            String uniqueColumnNames = collectAttributes(
                    uniqueElement,
                    TorqueSchemaElementName.UNIQUE_COLUMN.getName(),
                    TorqueSchemaAttributeName.NAME.getName());
            uniqueElement.setAttribute(
                    UNIQUE_COLUMN_NAMES_ATTRIBUTE,
                    uniqueColumnNames);
            ++uniqueIndex;
        }

        // index
        for (SourceElement indexElement : tableElement.getChildren(
                TorqueSchemaElementName.INDEX.getName()))
        {
            String indexColumnNames = collectAttributes(
                    indexElement,
                    TorqueSchemaElementName.INDEX_COLUMN.getName(),
                    TorqueSchemaAttributeName.NAME.getName());
            indexElement.setAttribute(
                    INDEX_COLUMN_NAMES_ATTRIBUTE,
                    indexColumnNames);
        }

        List<SourceElement> columnElements = tableElement.getChildren(
                TorqueSchemaElementName.COLUMN.getName());
        for (SourceElement columnElement : columnElements)
        {
            transformColumn(columnElement, controllerState);
        }
        List<SourceElement> foreignKeyElements = tableElement.getChildren(
                TorqueSchemaElementName.FOREIGN_KEY.getName());

        int fkIndex = 1;
        for (SourceElement foreignKeyElemenElement : foreignKeyElements)
        {
            transformForeignKey(
                    foreignKeyElemenElement,
                    controllerState,
                    fkIndex);
            ++fkIndex;
        }
    }

    private void transformColumn(
            SourceElement columnElement,
            ControllerState controllerState)
        throws SourceTransformerException
    {
        String sql = (String) columnElement.getAttribute(
                DDL_SQL_ATTRIBUTE);
        if (sql == null)
        {
            sql = getSql(columnElement, controllerState);
            columnElement.setAttribute(DDL_SQL_ATTRIBUTE, sql);
        }
    }

    private String getSql(
            SourceElement columnElement,
            ControllerState controllerState)
        throws SourceTransformerException
    {
        SchemaType schemaType = SchemaTypeHelper.getSchemaType(
                columnElement,
                controllerState);
        Object size = columnElement.getAttribute(
                TorqueSchemaAttributeName.SIZE);
        Object scale = columnElement.getAttribute(
                TorqueSchemaAttributeName.SCALE);
        Object defaultValue = columnElement.getAttribute(
                TorqueSchemaAttributeName.DEFAULT);
        SqlType sqlType = SchemaTypeHelper.getSqlType(
                schemaType,
                controllerState,
                ObjectUtils.toString(size, null),
                ObjectUtils.toString(scale, null),
                ObjectUtils.toString(defaultValue, null));
        Platform platform = PlatformFactory.getPlatformFor(
                controllerState.getStringOption(
                        TemplateOptionName.DATABASE));

        List<String> resultList = new ArrayList<String>();

        String sqlTypeName = sqlType.getSqlTypeName();

        if (platform.hasSize(sqlType.getSqlTypeName()))
        {
            sqlTypeName += sqlType.printSize();
        }

        resultList.add(sqlTypeName);

        if (StringUtils.isNotEmpty(sqlType.getDefaultValue()))
        {
            resultList.add("default");

            if (TypeMap.isTextType(schemaType))
            {
                // TODO: Properly SQL-escape the text.
                resultList.add(
                        new StringBuffer()
                        .append('\'')
                        .append(sqlType.getDefaultValue())
                        .append('\'')
                        .toString());
            }
            else
            {
                resultList.add(sqlType.getDefaultValue());
            }
        }

        boolean primaryKey;
        {
            String primaryKeyString = (String) columnElement.getAttribute(
                    TorqueSchemaAttributeName.PRIMARY_KEY);
            primaryKey = Boolean.parseBoolean(primaryKeyString);
        }
        boolean required;
        {
            String requiredString = (String) columnElement.getAttribute(
                    TorqueSchemaAttributeName.REQUIRED);
            required = Boolean.parseBoolean(requiredString);
        }
        boolean isNotNull = primaryKey || required;
        String isNotNullString = platform.getNullString(isNotNull);

        if (platform.createNotNullBeforeAutoincrement())
        {
            if (StringUtils.isNotEmpty(isNotNullString))
            {
                resultList.add(isNotNullString);
            }
        }
        // if idMethod was not set explicitly by the user,
        // the transformTable() method sets the attribute from the
        // defaultIdMethod of the database, so this always returns
        // the id method
        Object idMethod = columnElement.getParent().getAttribute(
                TorqueSchemaAttributeName.ID_METHOD);
        if (primaryKey
                && TorqueSchemaIdMethod.NATIVE.getName().equals(idMethod))
        {
            String autoIncrement = platform.getAutoIncrement();
            if (StringUtils.isNotEmpty(autoIncrement))
            {
                resultList.add(autoIncrement);
            }
        }
        if (!platform.createNotNullBeforeAutoincrement())
        {
            if (StringUtils.isNotEmpty(isNotNullString))
            {
                resultList.add(isNotNullString);
            }
        }
        return StringUtils.join(resultList.iterator(), ' ');
    }

    private void transformForeignKey(
            SourceElement foreignKeyElement,
            ControllerState controllerState,
            int fkIndex)
        throws SourceTransformerException
    {
        // TODO: the 3.3 schema specifies "none" as default value
        // for the attributes onUpdate and onDelete.
        // This is equivalent to null. This code can be removed
        // once the schema/dtd is updated
        if ("none".equals(foreignKeyElement.getAttribute(
                TorqueSchemaAttributeName.ON_UPDATE.getName())))
        {
            foreignKeyElement.setAttribute(
                    TorqueSchemaAttributeName.ON_UPDATE.getName(),
                    null);
        }
        if ("none".equals(foreignKeyElement.getAttribute(
                TorqueSchemaAttributeName.ON_DELETE.getName())))
        {
            foreignKeyElement.setAttribute(
                    TorqueSchemaAttributeName.ON_DELETE.getName(),
                    null);
        }

        if (foreignKeyElement.getAttribute(
                    TorqueSchemaAttributeName.NAME.getName())
                == null)
        {
            foreignKeyElement.setAttribute(
                    TorqueSchemaAttributeName.NAME.getName(),
                    foreignKeyElement.getParent().getAttribute(
                           TorqueSchemaAttributeName.NAME.getName())
                        + "_FK_" + fkIndex);
        }
        String localColumnsNames = collectAttributes(
                foreignKeyElement,
                TorqueSchemaElementName.REFERENCE.getName(),
                TorqueSchemaAttributeName.LOCAL.getName());
        foreignKeyElement.setAttribute(
                LOCAL_COLUMN_NAMES_ATTRIBUTE,
                localColumnsNames);
        String foreignColumnsNames = collectAttributes(
                foreignKeyElement,
                TorqueSchemaElementName.REFERENCE.getName(),
                TorqueSchemaAttributeName.FOREIGN.getName());
        foreignKeyElement.setAttribute(
                FOREIGN_COLUMN_NAMES_ATTRIBUTE,
                foreignColumnsNames);
    }

    private String collectAttributes(
            SourceElement rootElement,
            String sourcePath,
            String attributeName)
    {
        StringBuilder result = new StringBuilder();
        List<SourceElement> toCollectFrom = SourcePath.getElements(
                rootElement, sourcePath);
        boolean firstElement = true;
        for (SourceElement sourceElement : toCollectFrom)
        {
            if (!firstElement)
            {
                result.append(", ");
            }
            result.append(sourceElement.getAttribute(attributeName));
            firstElement = false;
        }
        return result.toString();
    }
}
