package org.apache.torque.gf.configuration;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.BufferedInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.jar.JarFile;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.torque.gf.configuration.paths.TorqueGfPaths;
import org.apache.torque.gf.configuration.paths.ProjectPaths;

/**
 * Provides InputStreams to read a configuration of a unit of generation from a
 * jar file.
 */
public class ClasspathConfigurationProvider implements ConfigurationProvider
{
    /** The logger. */
    private static Log log
            = LogFactory.getLog(ClasspathConfigurationProvider.class);

    /**
     * The paths needed to interact with the enclosing project, not null.
     */
    private ProjectPaths projectPaths;

    /**
     * The internal directory structure of the generator configuration files,
     * not null.
     */
    private TorqueGfPaths configurationPaths;

    /** The prefix if an url points to a file. */
    private static final String FILE_URL_PREFIX = "file:";

    /**
     * Constructor.
     * @param projectPaths the paths needed to interact with the enclosing
     *         project, not null.
     * @param configurationPaths The internal directory structure of the
     *         generator configuration files, not null.
     *
     * @throws NullPointerException if projectPaths or configurationPaths
     *          are null.
     */
    public ClasspathConfigurationProvider(
            ProjectPaths projectPaths,
            TorqueGfPaths configurationPaths)
    {
        if (projectPaths == null)
        {
            throw new NullPointerException("projectPaths is null");
        }
        if (configurationPaths == null)
        {
            throw new NullPointerException("configurationPaths is null");
        }
        this.projectPaths = projectPaths;
        this.configurationPaths = configurationPaths;
    }

    public InputStream getControlConfigurationInputStream()
        throws ConfigurationException
    {
        String configFileName
                = getConfigResourceBase()
                    + "/"
                    + configurationPaths.getConfigurationDirectory()
                    + "/"
                    + configurationPaths.getControlConfigurationFile();

        InputStream inputStream
                = getClass().getClassLoader().getResourceAsStream(
                        configFileName);
        if (inputStream == null)
        {
            throw new ConfigurationException("Could not read entry "
                        + configFileName
                        + " in classpath");
        }
        BufferedInputStream bis = new BufferedInputStream(inputStream);
        if (log.isInfoEnabled())
        {
            log.info("Using control file: '"
                    + configFileName
                    + "' from classpath");
        }
        return bis;
    }

    public InputStream getTemplateInputStream(String name)
        throws ConfigurationException
    {
        String templateFileName
                = getConfigResourceBase()
                    + "/"
                    + configurationPaths.getTemplateSubdirectory()
                    + "/"
                    + name;

        InputStream inputStream
                = getClass().getClassLoader().getResourceAsStream(
                        templateFileName);

        if (inputStream == null)
        {
            throw new ConfigurationException("Could not read template file "
                        + templateFileName
                        + " in classpath");
        }
        BufferedInputStream bis = new BufferedInputStream(inputStream);
        if (log.isInfoEnabled())
        {
            log.info("Reading template: '"
                    + templateFileName
                    + "' in class path");
        }
        return bis;
    }

    public InputStream getGeneratorConfigurationInputStream(String name)
        throws ConfigurationException
    {
        String generatorConfigFileName
                = getConfigResourceBase()
                    + "/"
                    + configurationPaths.getGeneratorDefDirectory()
                    + "/"
                    + name;

        InputStream inputStream
                = getClass().getClassLoader().getResourceAsStream(
                        generatorConfigFileName);

        if (inputStream == null)
        {
            throw new ConfigurationException(
                    "Could not read generator configuration file "
                        + generatorConfigFileName
                        + " in classpath");
        }
        BufferedInputStream bis = new BufferedInputStream(inputStream);
        if (log.isInfoEnabled())
        {
            log.info("Reading generator configuration file: '"
                    + generatorConfigFileName
                    + "' in classpath");
        }
        return bis;
   }

    public Collection<String> getGeneratorConfigurationNames()
        throws ConfigurationException
    {
        String generatorConfigurationSubdir
                = getConfigResourceBase()
                    + "/"
                    + configurationPaths.getGeneratorDefDirectory();
        generatorConfigurationSubdir
                = generatorConfigurationSubdir.replace('\\', '/');


        URL dirUrl = getClass().getClassLoader().getResource(
                    generatorConfigurationSubdir);
        String dirUrlString = dirUrl.toExternalForm();
        if (dirUrlString.startsWith("jar"))
        {
            String jarFilePath = dirUrl.getFile();
            if (jarFilePath.startsWith(FILE_URL_PREFIX))
            {
                jarFilePath = jarFilePath.substring(FILE_URL_PREFIX.length());
            }
            jarFilePath = jarFilePath.substring(0, jarFilePath.indexOf("!"));
            if (log.isDebugEnabled())
            {
                log.debug("generator config located in jar file"
                        + jarFilePath);
            }
            JarFile jarFile;
            try
            {
                jarFile = new JarFile(jarFilePath);
            }
            catch (IOException e)
            {
                log.error("Could not open jar File "
                        + jarFilePath);
                throw new ConfigurationException(e);
            }

            String generatorConfigurationDirectory
                    = dirUrlString.substring(dirUrlString.lastIndexOf("!") + 1);
            if (generatorConfigurationDirectory.startsWith("/"))
            {
                generatorConfigurationDirectory
                        = generatorConfigurationDirectory.substring(1);
            }
            return JarConfigurationProvider.getGeneratorConfigurationNames(
                    jarFile,
                    generatorConfigurationDirectory);
        }
        File directory = new File(dirUrl.getFile());
        if (!directory.exists())
        {
            throw new ConfigurationException(
                    "Could not read generator directory "
                        + generatorConfigurationSubdir
                        + " in classpath; directory URL is "
                        + dirUrl
                        + " file is "
                        + dirUrl.getFile());
        }
        String[] filenames = directory.list();

        List<String> result = new ArrayList<String>();
        for (String filename : filenames)
        {
            File file = new File(filename);
            if (file.isDirectory())
            {
                continue;
            }
            String rawName = file.getName();
            if (!rawName.endsWith(".xml"))
            {
                continue;
            }
            result.add(filename);
        }
        return result;
    }

    public InputStream getOptionsInputStream(String name)
        throws ConfigurationException
    {
        String optionsFileName
                = getConfigResourceBase()
                    + "/"
                    + configurationPaths.getConfigurationDirectory()
                    + "/"
                    + name;

        InputStream inputStream
                = getClass().getClassLoader().getResourceAsStream(
                        optionsFileName);

        if (inputStream == null)
        {
            throw new ConfigurationException("Could not read options file "
                        + optionsFileName
                        + " in classpath");
        }
        BufferedInputStream bis = new BufferedInputStream(inputStream);
        if (log.isInfoEnabled())
        {
            log.info("Reading options file: '"
                    +  optionsFileName
                     + "' in classpath");
        }
        return bis;
    }

    /**
     * Gets the resource name for the configuration base directory from the
     * projectPaths.
     *
     * @return the resource name for the configuration base directory,
     *         not null.
     */
    private String getConfigResourceBase()
    {
        String configResourceBase
            = projectPaths.getConfigurationPackage().replace('.', '/');
        return configResourceBase;
    }
}
