package org.apache.torque.gf.configuration;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.torque.gf.configuration.paths.TorqueGfPaths;
import org.apache.torque.gf.configuration.paths.ProjectPaths;

/**
 * Provides InputStreams to read the configuration from a directory.
 */
public class DirectoryConfigurationProvider implements ConfigurationProvider
{
    /** The logger. */
    private static Log log
            = LogFactory.getLog(DirectoryConfigurationProvider.class);

    /**
     * The paths needed to interact with the enclosing project, not null.
     */
    private ProjectPaths projectPaths;

    /**
     * The internal directory structure of the generator configuration files,
     * not null.
     */
    private TorqueGfPaths configurationPaths;

    /**
     * Constructor.
     *
     * @param projectPaths the paths needed to interact with the enclosing
     *         project, not null.
     * @param configurationPaths The internal directory structure of the
     *         generator configuration files, not null.
     *
     * @throws NullPointerException if projectPaths or configurationPaths
     *          are null.
     */
    public DirectoryConfigurationProvider(
            ProjectPaths projectPaths,
            TorqueGfPaths configurationPaths)
    {
        if (projectPaths == null)
        {
            throw new NullPointerException("projectPaths is null");
        }
        if (configurationPaths == null)
        {
            throw new NullPointerException("configurationPaths is null");
        }
        this.projectPaths = projectPaths;
        this.configurationPaths = configurationPaths;
    }

    public InputStream getControlConfigurationInputStream()
        throws ConfigurationException
    {
        File configFile;
        {
            File configDirectory = new File(
                    projectPaths.getConfigurationPath(),
                    configurationPaths.getConfigurationDirectory());
            configFile = new File(
                    configDirectory,
                    configurationPaths.getControlConfigurationFile());
        }

        InputStream inputStream = null;
        try
        {
            inputStream = new FileInputStream(configFile);
        }
        catch (FileNotFoundException e)
        {
            throw new ConfigurationException("Controller configuration file "
                    + configFile.getAbsolutePath()
                    + " not found");
        }
        BufferedInputStream bis = new BufferedInputStream(inputStream);
        if (log.isInfoEnabled())
        {
            log.info("Using control file: '"
                    + configFile.getAbsolutePath() + "'");
        }
        return bis;
    }

    public InputStream getTemplateInputStream(String name)
        throws ConfigurationException
    {
        File templateFile;
        {
            File templateDir =  new File(
                    projectPaths.getConfigurationPath(),
                    configurationPaths.getTemplateSubdirectory());
            templateFile = new File(
                    templateDir,
                    name);
        }

        InputStream inputStream = null;
        try
        {
            inputStream = new FileInputStream(templateFile);
        }
        catch (FileNotFoundException e)
        {
            throw new ConfigurationException("Template file "
                    + templateFile.getAbsolutePath()
                    + " not found");
        }
        BufferedInputStream bis = new BufferedInputStream(inputStream);
        if (log.isInfoEnabled())
        {
            log.info("Reading Template: '"
                    + templateFile.getAbsolutePath() + "'");
        }
        return bis;
    }

    public InputStream getGeneratorConfigurationInputStream(String name)
        throws ConfigurationException
    {
        File generatorConfigFile;
        {
            File generatorConfigDir =  new File(
                    projectPaths.getConfigurationPath(),
                    configurationPaths.getGeneratorDefDirectory());

            generatorConfigFile = new File(generatorConfigDir, name);
        }

        InputStream inputStream = null;
        try
        {
            inputStream = new FileInputStream(generatorConfigFile);
        }
        catch (FileNotFoundException e)
        {
            throw new ConfigurationException("Generator configuration file "
                    + generatorConfigFile.getAbsolutePath()
                    + " not found");
        }
        BufferedInputStream bis = new BufferedInputStream(inputStream);
        if (log.isInfoEnabled())
        {
            log.info("Reading Generator configuration file: '"
                    + generatorConfigFile.getAbsolutePath() + "'");
        }
        return bis;

    }

    public Collection<String> getGeneratorConfigurationNames()
        throws ConfigurationException
    {
        File generatorConfigDir =  new File(
                projectPaths.getConfigurationPath(),
                configurationPaths.getGeneratorDefDirectory());

        List<String> result = new ArrayList<String>();
        if (!generatorConfigDir.isDirectory())
        {
            throw new ConfigurationException(
                    "GeneratorsConfigDirectory "
                        + generatorConfigDir.getAbsolutePath()
                        + "must be a directory");
        }

        File[] sourceFiles = generatorConfigDir.listFiles();
        for (int fileNr = 0; fileNr < sourceFiles.length; ++fileNr)
        {
            if (!sourceFiles[fileNr].isDirectory()
                    && sourceFiles[fileNr].getPath().endsWith("xml"))
            {
                String name = sourceFiles[fileNr].getName();
                result.add(name);
            }
        }
        return result;
    }

    public InputStream getOptionsInputStream(String name)
        throws ConfigurationException
    {
        File optionsFile;
        {
            File configDirectory = new File(
                    projectPaths.getConfigurationPath(),
                    configurationPaths.getConfigurationDirectory());
            optionsFile = new File(configDirectory, name);
        }

        InputStream inputStream = null;
        try
        {
            inputStream = new FileInputStream(optionsFile);
        }
        catch (FileNotFoundException e)
        {
            throw new ConfigurationException("Options file "
                    + optionsFile.getAbsolutePath()
                    + " not found");
        }
        BufferedInputStream bis = new BufferedInputStream(inputStream);
        if (log.isInfoEnabled())
        {
            log.info("Reading Options file: '"
                    + optionsFile.getAbsolutePath() + "'");
        }
        return bis;
    }
}
