package org.apache.torque.gf.configuration;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.apache.torque.gf.configuration.generator.GeneratorSaxHandlerFactory;
import org.apache.torque.gf.configuration.generator.JavaGeneratorSaxHandlerFactory;
import org.apache.torque.gf.configuration.generator.VelocityGeneratorSaxHandlerFactory;

/**
 * Manages the types of known Generators. This include information
 * about how to read the configuration of each generator type.
 */
public class GeneratorTypes
{
    /**
     * A map containing all known GeneratorSaxHandlerFactories,
     * keyed by the type of the generator.
     */
    private Map<String, GeneratorSaxHandlerFactory>
        generatorHandlerFactories
            = new HashMap<String, GeneratorSaxHandlerFactory>();

    /**
     * Constructor. Creates a new instance containing the mappings to the
     * default generator types.
     */
    public GeneratorTypes()
    {
        // register default generator types
        try
        {
            registerGeneratorHandlerFactory(
                    new VelocityGeneratorSaxHandlerFactory());
            registerGeneratorHandlerFactory(
                    new JavaGeneratorSaxHandlerFactory());
        }
        catch (ConfigurationException e)
        {
            // should not happen
            throw new RuntimeException(e);
        }
    }

    /**
     * Registers a handler for a new type of generators.
     *
     * @param factory the factory which handles the generators of the
     *         given type.
     *
     * @throws NullPointerException if factory is null.
     * @throws ConfigurationException if a factory already exists
     *          for the type of the generator.
     */
    public void registerGeneratorHandlerFactory(
            GeneratorSaxHandlerFactory factory)
        throws ConfigurationException
    {
        if (factory == null)
        {
            throw new NullPointerException("factory must not be null");
        }
        GeneratorSaxHandlerFactory oldFactory
            = generatorHandlerFactories.get(factory.getType());
        if (oldFactory != null)
        {
            throw new ConfigurationException(
                    "Attempted to register a GeneratorSaxHandlerFactory "
                        + "of type "
                        + factory.getType()
                        + " and class "
                        + factory.getClass().getName()
                        + " : A factory with this type already exists, "
                        + " it has the class "
                        + oldFactory.getClass().getName());
        }
        generatorHandlerFactories.put(factory.getType(), factory);
    }

    /**
     * Returns an unmodifiable  map containing all the generator handler
     * factories, keyed by their type.
     *
     * @return all generator handler factories, not null.
     */
    public Map<String, GeneratorSaxHandlerFactory>
        getGeneratorHandlerFactories()
    {
        return Collections.unmodifiableMap(generatorHandlerFactories);
    }
}
