package org.apache.torque.gf.configuration;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.torque.gf.configuration.paths.TorqueGfPaths;
import org.apache.torque.gf.configuration.paths.ProjectPaths;

/**
 * Provides InputStreams to read a configuration of a unit of generation from a
 * jar file.
 */
public class JarConfigurationProvider implements ConfigurationProvider
{
    /** The logger. */
    private static Log log
            = LogFactory.getLog(JarConfigurationProvider.class);

    /**
     * The paths needed to interact with the enclosing project, not null.
     */
    private ProjectPaths projectPaths;

    /**
     * The internal directory structure of the generator configuration files,
     * not null.
     */
    private TorqueGfPaths configurationPaths;

    /**
     * The jar file from which the configuration should be read, not null.
     */
    private JarFile jarFile;

    /**
     * Constructor.
     * @param projectPaths the paths needed to interact with the enclosing
     *         project, not null.
     * @param configurationPaths The internal directory structure of the
     *         generator configuration files, not null.
     *
     * @throws NullPointerException if projectPaths or configurationPaths
     *          are null.
     * @throws ConfigurationException if the jar file can not be accessed.
     */
    public JarConfigurationProvider(
            ProjectPaths projectPaths,
            TorqueGfPaths configurationPaths)
        throws ConfigurationException
    {
        if (projectPaths == null)
        {
            throw new NullPointerException("projectPaths is null");
        }
        if (configurationPaths == null)
        {
            throw new NullPointerException("configurationPaths is null");
        }
        this.projectPaths = projectPaths;
        this.configurationPaths = configurationPaths;

        try
        {
            jarFile = new JarFile(projectPaths.getConfigurationPath());
        }
        catch (IOException e)
        {
            log.error("Could not open jar File "
                    + projectPaths.getConfigurationPath()
                        .getAbsolutePath());
            throw new ConfigurationException(e);
        }

    }

    public InputStream getControlConfigurationInputStream()
        throws ConfigurationException
    {
        String configFileName
                = configurationPaths.getConfigurationDirectory()
                        + "/"
                        + configurationPaths.getControlConfigurationFile();


        InputStream inputStream = null;
        try
        {
            JarEntry jarEntry = jarFile.getJarEntry(configFileName);
            if (jarEntry == null)
            {
                throw new ConfigurationException("File " + configFileName
                        + " not found in Jar file.");
            }
            inputStream = jarFile.getInputStream(jarEntry);
        }
        catch (IOException e)
        {
            throw new ConfigurationException("Could not read entry "
                        + configFileName
                        + " in jar file "
                        + projectPaths.getConfigurationPath(),
                    e);
        }
        BufferedInputStream bis = new BufferedInputStream(inputStream);
        if (log.isInfoEnabled())
        {
            log.info("Using control file: '"
                    + configFileName
                    + "' in jar file "
                    + projectPaths.getConfigurationPath());
        }
        return bis;
    }

    public InputStream getTemplateInputStream(String name)
        throws ConfigurationException
    {
        String templateFileName
                = configurationPaths.getTemplateSubdirectory()
                    + "/"
                    + name;

        InputStream inputStream = null;
        try
        {
            JarEntry jarEntry = jarFile.getJarEntry(templateFileName);
            inputStream = jarFile.getInputStream(jarEntry);
        }
        catch (IOException e)
        {
            throw new ConfigurationException("Could not read template file "
                        + templateFileName
                        + " in jar file "
                        + projectPaths.getConfigurationPath(),
                    e);
        }
        BufferedInputStream bis = new BufferedInputStream(inputStream);
        if (log.isInfoEnabled())
        {
            log.info("Reading template: '"
                    + projectPaths.getConfigurationPath()
                    + "' in jar file "
                    + projectPaths.getConfigurationPath());
        }
        return bis;
    }

    public InputStream getGeneratorConfigurationInputStream(String name)
        throws ConfigurationException
    {
        String generatorConfigFileName
                = configurationPaths.getGeneratorDefDirectory()
                    + "/"
                    + name;

        InputStream inputStream = null;
        try
        {
            JarEntry jarEntry = jarFile.getJarEntry(generatorConfigFileName);
            inputStream = jarFile.getInputStream(jarEntry);
        }
        catch (IOException e)
        {
            throw new ConfigurationException(
                    "Could not read generator configuration file "
                        + generatorConfigFileName
                        + " in jar file "
                        + projectPaths.getConfigurationPath(),
                    e);
        }
        BufferedInputStream bis = new BufferedInputStream(inputStream);
        if (log.isInfoEnabled())
        {
            log.info("Reading generator configuration file: '"
                    + projectPaths.getConfigurationPath()
                    + "' in jar file "
                    + projectPaths.getConfigurationPath());
        }
        return bis;
   }

    public Collection<String> getGeneratorConfigurationNames()
        throws ConfigurationException
    {
        return getGeneratorConfigurationNames(
                jarFile,
                configurationPaths.getGeneratorDefDirectory());
    }

    /**
     * Extracts the generator configuration files from a jar file.
     * @param jarFile the jar file to process, not null.
     * @param generatorConfigurationDirectory the name of the directory
     *        which contains the generator configuration files. Cannot be
     *        a composite path like parent/child.
     * @return a set with the names of all generator configuration files
     *         contained in the jar file.
     * @throws NullPointerException if jarFile
     *         or generatorConfigurationDirectory is null
     */
    static Collection<String> getGeneratorConfigurationNames(
            JarFile jarFile,
            String generatorConfigurationDirectory)
    {
        if (log.isDebugEnabled())
        {
            log.debug("Analyzing jar file " + jarFile.getName()
                    +  " seeking Directory " + generatorConfigurationDirectory);
        }

        List<String> result = new ArrayList<String>();

        Enumeration<JarEntry> entries = jarFile.entries();
        while (entries.hasMoreElements())
        {
            JarEntry jarEntry = entries.nextElement();
            if (jarEntry.isDirectory())
            {
                continue;
            }
            String rawName = jarEntry.getName();
            if (!rawName.startsWith(generatorConfigurationDirectory))
            {
                continue;
            }
            String name = rawName.substring(rawName.lastIndexOf('/') + 1);

            int expectedRawNameLength
                    = generatorConfigurationDirectory.length()
                        + name.length()
                        + 1;
            if (rawName.length() != expectedRawNameLength)
            {
                // file is in a subdirectory of generatorConfigurationSubdir,
                // we only consider files directly in
                // generatorConfigurationSubdir
                continue;
            }
            result.add(name);
        }
        if (log.isDebugEnabled())
        {
            log.debug("Found the following generator configuration files "
                    + result);
        }
        return result;
    }

    public InputStream getOptionsInputStream(String name)
        throws ConfigurationException
    {
        String optionsFileName
                = configurationPaths.getConfigurationDirectory()
                    + "/"
                    + name;

        InputStream inputStream = null;
        try
        {
            JarEntry jarEntry = jarFile.getJarEntry(optionsFileName);
            inputStream = jarFile.getInputStream(jarEntry);
        }
        catch (IOException e)
        {
            throw new ConfigurationException("Could not read options file "
                        + optionsFileName
                        + " in jar file "
                        + projectPaths.getConfigurationPath(),
                    e);
        }
        BufferedInputStream bis = new BufferedInputStream(inputStream);
        if (log.isInfoEnabled())
        {
            log.info("Reading options file: '"
                    + projectPaths.getConfigurationPath()
                    + "' in jar file "
                    + projectPaths.getConfigurationPath());
        }
        return bis;
    }
}
