package org.apache.torque.gf.configuration;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.torque.gf.configuration.controller.Loglevel;
import org.apache.torque.gf.configuration.controller.Output;
import org.apache.torque.gf.configuration.generator.GeneratorConfiguration;
import org.apache.torque.gf.option.Options;

/**
 * A container for the configuration for a generation unit.
 * Provides state checking, i.e. getters can only be called
 * after all setters has been called.
 */
public class UnitConfiguration
{
    /** The list of all activities of the generation unit. */
    private List<Output> outputFiles = new ArrayList<Output>();

   /**
     * All options for the generation unit.
     */
    private Options options;

    /**
     * The configuration for the available generators.
     */
    private GeneratorConfiguration generatorConfiguration;

    /**
     * The root directory where the generated file which are generated
     * each time anew are put into.
     */
    private File newFileTargetDirectory;

    /**
     * The root directory where the generated file which are not generated
     * each time anew are put into.
     */
    private File modifiedFileTargetDirectory;

    /** The Loglevel for generation. */
    private Loglevel loglevel = Loglevel.INFO;

    /**
     * Returns the configuration of the generators in this generation unit.
     *
     * @return the generator configuration, or null if not set.
     *
     * @throws IllegalStateException if generatorConfiguration was not set.
     */
    public GeneratorConfiguration getGeneratorConfiguration()
    {
        if (generatorConfiguration == null)
        {
            throw new IllegalStateException(
                    "generatorConfiguration is not initialized");
        }
        return generatorConfiguration;
    }

    /**
     * Sets the generator configuration of the associated configuration unit.
     *
     * @param generatorConfiguration the generator configuration, not null.
     *
     * @throws NullPointerException if generatorConfiguration is null.
     */
    public void setGeneratorConfiguration(
            GeneratorConfiguration generatorConfiguration)
    {
        if (generatorConfiguration == null)
        {
            throw new NullPointerException(
                    "generatorConfiguration cannot be null");
        }
        this.generatorConfiguration = generatorConfiguration;
    }

    /**
     * Returns the options of the associated configuration unit.
     *
     * @return the options, not null.
     *
     * @throws IllegalStateException if options were not yet set.
     */
    public Options getOptions()
    {
        if (options == null)
        {
            throw new IllegalStateException(
                    "options are not initialized");
        }
        return options;
    }

    /**
     * Sets the options of the associated configuration unit.
     *
     * @param options the options, not null.
     *
     * @throws NullPointerException if options is null.
     */
    public void setOptions(Options options)
    {
        if (options == null)
        {
            throw new NullPointerException(
                    "options cannot be null");
        }
        this.options = options;
    }

    /**
     * Returns the target directory for the files which are generated
     * each time anew of the associated configuration unit.
     *
     * @return the target directory for the files which are generated
     *         each time anew, not null.
     *
     * @throws IllegalStateException if the target directory was not yet set.
     */
    public File getNewFileTargetDirectory()
    {
        if (newFileTargetDirectory == null)
        {
            throw new IllegalStateException(
                    "newFileTargetDirectory is not initialized");
        }
        return newFileTargetDirectory;
    }

    /**
     * Returns the target directory for the files which are not generated
     * each time anew of the associated configuration unit.
     *
     * @return the target directory for the files which are generated
     *         each time anew, not null.
     *
     * @throws IllegalStateException if the target directory was not yet set.
     */
    public File getModifiedFileTargetDirectory()
    {
        if (modifiedFileTargetDirectory == null)
        {
            throw new IllegalStateException(
                    "modifiedFileTargetDirectory is not initialized");
        }
        return modifiedFileTargetDirectory;
    }

    /**
     * Sets the target directory for the files which are generated
     * each time anew of the associated configuration unit.
     *
     * @param newFileTargetDirectory the target directory, not null.
     *
     * @throws NullPointerException if targetDirectory is null.
     */
    public void setNewFileTargetDirectory(File newFileTargetDirectory)
    {
        if (newFileTargetDirectory == null)
        {
            throw new NullPointerException(
                    "newFileTargetDirectory cannot be null");
        }
        this.newFileTargetDirectory = newFileTargetDirectory;
    }

    /**
     * Sets the target directory for the files which are not generated
     * each time anew of the associated configuration unit.
     *
     * @param modifiedFileTargetDirectory the target directory, not null.
     *
     * @throws NullPointerException if targetDirectory is null.
     */
    public void setModifiedFileTargetDirectory(File modifiedFileTargetDirectory)
    {
        if (modifiedFileTargetDirectory == null)
        {
            throw new NullPointerException(
                    "modifiedFileTargetDirectory cannot be null");
        }
        this.modifiedFileTargetDirectory = modifiedFileTargetDirectory;
    }

    /**
     * Sets the output files of the associated configuration unit.
     *
     * @param outputFiles the output files, not null.
     *
     * @throws NullPointerException if outputFiles is null.
     */
    public void setOutputFiles(List<Output> outputFiles)
    {
        if (outputFiles == null)
        {
            throw new NullPointerException(
                    "outputFiles cannot be null");
        }
        this.outputFiles = outputFiles;
    }

    /**
     * Returns the output files of the associated configuration unit.
     *
     * @return the output files, not null.
     *
     * @throws IllegalStateException if the output files were not yet set.
     */
    public List<Output> getOutputFiles()
    {
        if (outputFiles == null)
        {
            throw new IllegalStateException(
                    "outputFiles are not initialized");
        }
        return Collections.unmodifiableList(outputFiles);
    }

    /**
     * Returns the Loglevel during generation.
     *
     * @return the Loglevel, not null.
     */
    public Loglevel getLoglevel()
    {
        return loglevel;
    }

    /**
     * Sets the Loglevel during generation.
     *
     * @param loglevel the Loglevel, not null.
     *
     * @throws NullPointerException if loglevel is set to null.
     */
    public void setLoglevel(Loglevel loglevel)
    {
        if (loglevel == null)
        {
            throw new NullPointerException("loglevel is null");
        }
        this.loglevel = loglevel;
    }

    /**
     * Checks whether the unit configuration is fully initialized.
     *
     * @return true if the unit configuration is fully initialized,
     *          false otherwise.
     */
    public boolean isInit()
    {
        if (outputFiles == null
            || options == null
            || generatorConfiguration == null
            || newFileTargetDirectory == null
            || modifiedFileTargetDirectory == null)
        {
            return false;
        }
        return true;
    }

    @Override
    public String toString()
    {
        StringBuffer result = new StringBuffer();
        result.append("(outputFiles=")
                .append(outputFiles.toString())
                .append(", options=")
                .append(options)
                .append(", generatorConfiguration=")
                .append(generatorConfiguration)
                .append(", newFileTargetDirectory=")
                .append(newFileTargetDirectory)
                .append(", modifiedFileTargetDirectory=")
                .append(modifiedFileTargetDirectory)
                .append(", loglevel=")
                .append(loglevel)
                .append(")");
        return result.toString();
    }
}
