package org.apache.torque.gf.configuration;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.apache.torque.gf.configuration.controller.Loglevel;
import org.apache.torque.gf.configuration.option.OptionsConfiguration;
import org.apache.torque.gf.configuration.paths.TorqueGfPaths;
import org.apache.torque.gf.configuration.paths.ProjectPaths;

/**
 * Contains all necessary information about a generation unit.
 */
public class UnitDescriptor
{
    /**
     * Possible packaging forms of a unit of generation.
     */
    public enum Packaging
    {
        /** the generation unit is provided as a jar file .*/
        JAR,
        /** the generation unit is provided in a local directory. */
        DIRECTORY,
        /** the generation unit is provided in the classpath. */
        CLASSPATH
    }

    /**
     * The packaging of the generation unit.
     */
    private Packaging packaging;

   /**
    * The paths the code generator must know about the surrounding project.
    */
    private ProjectPaths projectPaths;

    /**
     * How the code generator's configuration is organized internally.
     */
    private TorqueGfPaths configurationPaths;

    /**
     * The parent of this generation unit, or null if it has no parent.
     */
    private UnitDescriptor inheritsFrom;

    /** Options to override the settings in the project Directory. */
    private OptionsConfiguration overrideOptions;

    /** The Loglevel to override the loglevel in the generator configuration. */
    private Loglevel loglevel;

    /**
     * Constructor without inheritance, override options and loglevel.
     *
     * @param packaging The packaging of the generation unit, not null.
     * @param projectPaths The paths the code generator must know about
     *        the surrounding project, not null.
     * @param configurationPaths The paths within the configuration
     *        of the configuration unit, not null.
     */
    public UnitDescriptor(
            Packaging packaging,
            ProjectPaths projectPaths,
            TorqueGfPaths configurationPaths)
    {
        this(packaging, projectPaths, configurationPaths, null, null, null);
    }

    /**
     * Constructor.
     *
     * @param packaging The packaging of the generation unit, not null.
     * @param projectPaths The paths the code generator must know about
     *        the surrounding project, not null.
     * @param configurationPaths The paths within the configuration
     *        of the configuration unit, not null.
     * @param inheritsFrom the parent of this generation unit,
     *        or null if it does not inherit from a generation unit.
     * @param overrideOptions Options to override the settings in the project
     *        Directory, or null if no options are overridden.
     * @param loglevel The loglevel to override the log level, or null to use
     *        the log level defined in the configuration.
     */
    public UnitDescriptor(
            Packaging packaging,
            ProjectPaths projectPaths,
            TorqueGfPaths configurationPaths,
            UnitDescriptor inheritsFrom,
            OptionsConfiguration overrideOptions,
            Loglevel loglevel)
    {
        if (packaging == null)
        {
            throw new NullPointerException("packaging must not be null");
        }
        if (projectPaths == null)
        {
            throw new NullPointerException(
                    "projectPaths must not be null");
        }
        if (configurationPaths == null)
        {
            throw new NullPointerException(
                    "configurationPaths must not be null");
        }

        this.packaging = packaging;
        this.projectPaths = projectPaths;
        this.configurationPaths = configurationPaths;
        this.inheritsFrom = inheritsFrom;
        this.overrideOptions = overrideOptions;
        this.loglevel = loglevel;
    }

    /**
     * Returns the packaging of the unit of generation.
     *
     * @return the packaging of the unit of generation, not null.
     */
    public Packaging getPackaging()
    {
        return packaging;
    }

    /**
     * Returns the paths which the code generator must know about the
     * surrounding project.
     *
     * @return the paths of the surrounding project, not null.
     */
    public ProjectPaths getProjectPaths()
    {
        return projectPaths;
    }

    /**
     * Returns the paths in the configuration of this generation unit.
     * @return the paths in the configuration of this generation unit,
     *          not null.
     */
    public TorqueGfPaths getConfigurationPaths()
    {
        return configurationPaths;
    }

    /**
     * Returns the descriptor of the generation unit from which this generation
     * unit inherits, or null if this generation unit does not inherit from
     * another generation unit.
     *
     * @return the parents unit descriptor, or null if no parent exists.
     */
    public UnitDescriptor getInheritsFrom()
    {
        return inheritsFrom;
    }

    /**
     * Returns the configuration of the overriding options, if any.
     *
     * @return the configuration of the overriding options, or null.
     */
    public OptionsConfiguration getOverrideOptions()
    {
        return overrideOptions;
    }

    /**
     * Returns the log level overriding the loglevel defined in the
     * configuration unit.
     *
     * @return the log level, or null.
     */
    public Loglevel getLoglevel()
    {
        return loglevel;
    }
}
