package org.apache.torque.gf.configuration.controller;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;

/**
 * The possible log levels. This implementation uses log4j internally,
 * but the interface is agnostic of the logging framework.
 */
public enum Loglevel
{
    /** Loglevel trace. */
    TRACE("trace", Level.TRACE),
    /** Loglevel debug. */
    DEBUG("debug", Level.DEBUG),
    /** Loglevel info. */
    INFO("info", Level.INFO),
    /** Loglevel warn. */
    WARN("warn", Level.WARN),
    /** Loglevel error. */
    ERROR("error", Level.ERROR);

    /** The key of the loglevel. */
    private String key;

    /** The log4j loglevel. */
    private Level log4jLevel;

    /** The logger. */
    private static Log log = LogFactory.getLog(Loglevel.class);

    /**
     * Constructor.
     *
     * @param key the key, not null.
     */
    private Loglevel(String key, Level log4jLevel)
    {
        this.key = key;
        this.log4jLevel = log4jLevel;
    }

    /**
     * Returns the key of the Loglevel.
     *
     * @return the key of the Loglevel, not null.
     */
    public String getKey()
    {
        return key;
    }

    /**
     * Applies the log level.
     */
    public void apply()
    {
        if (Logger.getRootLogger().getLevel() != log4jLevel)
        {
            log.info("apply() : Setting loglevel to " + this);
            Logger.getRootLogger().setLevel(log4jLevel);
        }
    }

    /**
     * Returns the Loglevel for a given key.
     *
     * @param key the key to look for.
     *
     * @return the corresponding Loglevel, not null.
     *
     * @throws IllegalArgumentException if no Loglevel can be found
     *         for the key.
     */
    public static Loglevel getByKey(String key)
    {
        for (Loglevel loglevel : values())
        {
            if (loglevel.getKey().equals(key))
            {
                return loglevel;
            }
        }
        throw new IllegalArgumentException(
                "Key " + key + " does not exist");
    }

    /**
     * Returns the current loglevel.
     *
     * @return the current loglevel, or INFO if the current loglevel cannot
     *         be determined.
     */
    public static Loglevel getCurrentLoglevel()
    {
        Level level = Logger.getRootLogger().getLevel();
        for (Loglevel loglevel : values())
        {
            if (loglevel.log4jLevel.equals(level))
            {
                return loglevel;
            }
        }
        return INFO;
    }
}
