package org.apache.torque.gf.configuration.controller;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.apache.torque.gf.generator.Generator;
import org.apache.torque.gf.qname.QualifiedName;
import org.apache.torque.gf.source.Sources;

/**
 * The configuration for an output (typically one or more files).
 */
public class Output
{
    /**
     * The name by which this output can be identified.
     */
    private QualifiedName name;

    /**
     * The sources to be used as input.
     */
    private Sources sources;

    /**
     * The generator which generates the content.
     */
    private GeneratorReference contentGenerator;

    /**
     * The generator which generates the filename, or null if the
     * path to the generated file is explicitly given in <code>path</code>.
     */
    private Generator filenameGenerator;

    /**
     * The filename of the generated file, or null if the filename must
     * still be generated using a filenameGenerator.
     * This attribute is also used to store the generated filename
     * if it was generated using the filenameGenerator.
     */
    private String filename;

    /**
     * Whether generation should be skipped if the file already exists.
     */
    private boolean skipIfExists;

    /**
     * Constructor.
     *
     * @param name the name by which this output can be identified.
     */
    public Output(QualifiedName name)
    {
        this.name = name;
    }

    /**
     * Returns the name by which this output can be identified.
     *
     * @return the name by which this output can be identified, not null.
     */
    public QualifiedName getName()
    {
        return name;
    }

    /**
     * Returns the sources which should be used as input for generating the
     * output file's contents.
     *
     * @return the sources which should be used as input for generation.
     */
    public Sources getSources()
    {
        return sources;
    }

    /**
     * Sets the sources which should be used as input for generating the
     * output file's contents.
     *
     * @param sources the sources which should be used as input for generation.
     */
    public void setSources(Sources sources)
    {
        this.sources = sources;
    }

    /**
     * Sets the name of the file to generate.
     * This is also used to store the generated filename
     * if it was generated using the filenameGenerator.
     *
     * @param filename the name of the file to generate.
     */
    public void setFilename(String filename)
    {
        this.filename = filename;
    }

    /**
     * Returns the name of the file to generate. Either this name was
     * given explicitly or it was set using the filenameGenerator.
     *
     * @return the name of the file to generate.
     */
    public String getFilename()
    {
        return filename;
    }

    /**
     * Returns the reference to the generator which should produce the content.
     *
     * @return the reference to the generator responsible for producing the
     *         content.
     */
    public GeneratorReference getContentGenerator()
    {
        return contentGenerator;
    }

    /**
     * Sets the reference to the generator which should produce the content.
     *
     * @param contentGenerator the reference to the generator responsible
     *        for producing the content.
     */
    public void setContentGenerator(GeneratorReference contentGenerator)
    {
        this.contentGenerator = contentGenerator;
    }

    /**
     * Returns the reference to the generator which should produce
     * the file name. If this attribute is set, it takes precedence over a
     * set filename.
     *
     * @return the reference to the generator responsible for producing the
     *         file name, or null if the file name is explicitly given.
     */
    public Generator getFilenameGenerator()
    {
        return filenameGenerator;
    }

    /**
     * Sets the reference to the generator which should produce
     * the file name.
     *
     * @param filenameGenerator the reference to the generator responsible
     *        for producing the file name.
     */
    public void setFilenameGenerator(Generator filenameGenerator)
    {
        this.filenameGenerator = filenameGenerator;
    }

    /**
     * Returns whether the generation of this file should be skipped
     * if the file already exists.
     *
     * @return true if generation should be skipped if the file already exists.
     */
    public boolean isSkipIfExists()
    {
        return skipIfExists;
    }

    /**
     * Sets whether the generation of this file should be skipped
     * if the file already exists.
     *
     * @param skipIfExists true if generation should be skipped if the file
     *        already exists, false otherwise.
     */
    public void setSkipIfExists(boolean skipIfExists)
    {
        this.skipIfExists = skipIfExists;
    }

    /**
     * Returns whether the target file is always generated anew.
     *
     * @return true if the target file is always generated anew,
     *         false otherwise.
     */
    public boolean isAlwaysNew()
    {
        return !skipIfExists;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString()
    {
        StringBuffer result = new StringBuffer();
        result.append("(OutputFile: sources=").append(sources)
                .append(",skipIfExists=").append(skipIfExists)
                .append(",filenameGenerator=")
                .append(filenameGenerator)
                .append(",contentGenerator=")
                .append(contentGenerator).append(")");
        return result.toString();
    }
}
