package org.apache.torque.gf.configuration.controller;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import static org.apache.torque.gf.configuration.controller.OutputConfigurationTags.FILENAME_GENERATOR_TAG;
import static org.apache.torque.gf.configuration.controller.OutputConfigurationTags.FILE_ATTRIBUTE;
import static org.apache.torque.gf.configuration.controller.OutputConfigurationTags.GENERATOR_TAG;
import static org.apache.torque.gf.configuration.controller.OutputConfigurationTags.NAME_ATTRIBUTE;
import static org.apache.torque.gf.configuration.controller.OutputConfigurationTags.OUTPUT_TAG;
import static org.apache.torque.gf.configuration.controller.OutputConfigurationTags.SKIP_IF_EXISTS_ATTRIBUTE;
import static org.apache.torque.gf.configuration.controller.SourceConfigurationTags.SOURCE_TAG;
import static org.apache.torque.gf.configuration.generator.GeneratorConfigurationTags.INPUT_TAG;

import org.apache.torque.gf.configuration.ConfigurationHandlers;
import org.apache.torque.gf.configuration.ConfigurationProvider;
import org.apache.torque.gf.configuration.generator.GeneratorConfigurationSaxHandler;
import org.apache.torque.gf.configuration.generator.GeneratorSaxHandler;
import org.apache.torque.gf.configuration.paths.ProjectPaths;
import org.apache.torque.gf.qname.QualifiedName;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Reads an output declaration from the controller configuration file.
 */
public class OutputSaxHandler extends DefaultHandler
{
    /** The qualified name for the filename generator. */
    private static final QualifiedName FILENAME_GENERATOR_NAME
            = new QualifiedName(
                "org.apache.torque.gf.configuration.filenameGenerator");

    /** The access object for the configuration files, not null. */
    private ConfigurationProvider configurationProvider;

    /** The paths of the surrounding project, not null. */
    private ProjectPaths projectPaths;

    /** All known configuration handlers. */
    private ConfigurationHandlers configurationHandlers;

    /** The output declaration which is currently parsed. */
    private Output output;

    /**
     * The SAX handler which handles the reference to the content generator,
     * or null if no content generator is currently processed.
     */
    private GeneratorReferenceSaxHandler contentGeneratorSaxHandler;

    /**
     * The SAX handler which handles source tags, or null if source tags
     * need not be handled in the current context.
     */
    private SourceSaxHandler sourceSaxHandler;

    /**
     * The SAX handler which handles the filename generator configuration,
     * or null if filename generator tags need not be handled
     * in the current context.
     */
    private GeneratorSaxHandler filenameGeneratorSaxHandler;

    /**
     * Constructor.
     *
     * @param configurationProvider The access object for the configuration
     *        files, not null.
     * @param projectPaths The paths of the surrounding project, not null.
     * @param generatorTypes the configured types of generators, not null.
     * @param sourceTypes All known source types, not null.
     *
     * @throws NullPointerException if an argument is null.
     */
    public OutputSaxHandler(
            ConfigurationProvider configurationProvider,
            ProjectPaths projectPaths,
            ConfigurationHandlers configurationHandlers)
    {
        if (configurationProvider == null)
        {
            throw new NullPointerException(
                    "configurationProvider must not be null");
        }
        if (projectPaths == null)
        {
            throw new NullPointerException(
                    "projectPaths must not be null");
        }
        if (configurationHandlers == null)
        {
            throw new NullPointerException(
                    "configurationHandlers must not be null");
        }
        this.configurationProvider = configurationProvider;
        this.configurationHandlers = configurationHandlers;
        this.projectPaths = projectPaths;
    }

    /**
     * Returns the Configuration filled with the contents of the parsed snippet.
     *
     * @return the configuration representing the parsed snippet.
     *         Not null if the mathcing xml snippet was parsed.
     */
    public Output getOutputFile()
    {
        return output;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void startElement(
            String uri,
            String localName,
            String rawName,
            Attributes attributes)
        throws SAXException
    {
        if (contentGeneratorSaxHandler != null)
        {
            contentGeneratorSaxHandler.startElement(
                    uri,
                    localName,
                    rawName,
                    attributes);
        }
        else if (filenameGeneratorSaxHandler != null)
        {
            filenameGeneratorSaxHandler.startElement(
                    uri,
                    localName,
                    rawName,
                    attributes);
        }
        else if (sourceSaxHandler != null)
        {
            sourceSaxHandler.startElement(
                    uri,
                    localName,
                    rawName,
                    attributes);
        }
        else if (OUTPUT_TAG.equals(rawName))
        {
            String name = attributes.getValue(NAME_ATTRIBUTE);
            if (name == null)
            {
                throw new SAXException("The attribute "
                        + NAME_ATTRIBUTE
                        + " must be set for the tag "
                        + INPUT_TAG);
            }
            this.output = new Output(new QualifiedName(name));

            if (attributes.getValue(FILE_ATTRIBUTE) != null)
            {
                output.setFilename(attributes.getValue(FILE_ATTRIBUTE));
            }
            if (attributes.getValue(SKIP_IF_EXISTS_ATTRIBUTE) != null)
            {
                output.setSkipIfExists(
                        Boolean.parseBoolean(
                            attributes.getValue(SKIP_IF_EXISTS_ATTRIBUTE)));
            }
        }
        else if (SOURCE_TAG.equals(rawName))
        {
            sourceSaxHandler = new SourceSaxHandler(
                    configurationProvider,
                    projectPaths,
                    configurationHandlers);
            sourceSaxHandler.startElement(uri, localName, rawName, attributes);
        }
        else if (GENERATOR_TAG.equals(rawName))
        {
            contentGeneratorSaxHandler
                    = new GeneratorReferenceSaxHandler();
            contentGeneratorSaxHandler.startElement(
                    uri,
                    localName,
                    rawName,
                    attributes);
        }
        else if (FILENAME_GENERATOR_TAG.equals(rawName))
        {
            GeneratorConfigurationSaxHandler generatorConfigurationSaxHandler
                    = new GeneratorConfigurationSaxHandler(
                            configurationProvider,
                            projectPaths,
                            configurationHandlers);
            String generatorType
                    = GeneratorConfigurationSaxHandler.getGeneratorType(
                            attributes);
            filenameGeneratorSaxHandler
                    = generatorConfigurationSaxHandler.getGeneratorHandler(
                            FILENAME_GENERATOR_NAME,
                            generatorType);
            filenameGeneratorSaxHandler.startElement(
                    uri,
                    localName,
                    rawName,
                    attributes);
        }
        else
        {
            throw new SAXException("Unknown element " + rawName);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void endElement(String uri, String localName, String rawName)
        throws SAXException
    {
        if (contentGeneratorSaxHandler != null)
        {
            contentGeneratorSaxHandler.endElement(
                    uri,
                    localName,
                    rawName);
            if (contentGeneratorSaxHandler.isFinished())
            {
                output.setContentGenerator(
                        contentGeneratorSaxHandler
                            .getGeneratorReference());
                contentGeneratorSaxHandler = null;
            }
        }
        else if (sourceSaxHandler != null)
        {
            sourceSaxHandler.endElement(uri, localName, rawName);
            if (sourceSaxHandler.isFinished())
            {
                output.setSources(sourceSaxHandler.getSources());
                sourceSaxHandler = null;
            }
        }
        else if (filenameGeneratorSaxHandler != null)
        {
            filenameGeneratorSaxHandler.endElement(uri, localName, rawName);
            if (filenameGeneratorSaxHandler.isFinished())
            {
                output.setFilenameGenerator(
                        filenameGeneratorSaxHandler
                            .getGenerator());
                filenameGeneratorSaxHandler = null;
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void characters(char[] ch, int start, int length)
            throws SAXException
    {
        if (contentGeneratorSaxHandler != null)
        {
            contentGeneratorSaxHandler.characters(
                    ch, start, length);
        }
        else if (sourceSaxHandler != null)
        {
            sourceSaxHandler.characters(ch, start, length);
        }
        else if (filenameGeneratorSaxHandler != null)
        {
            filenameGeneratorSaxHandler.characters(ch, start, length);
        }
    }
}
