package org.apache.torque.gf.configuration.controller;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import static org.apache.torque.gf.configuration.controller.SourceConfigurationTags.ELEMENTS_ATTRIBUTE;
import static org.apache.torque.gf.configuration.controller.SourceConfigurationTags.PATH_ATTRIBUTE;
import static org.apache.torque.gf.configuration.controller.SourceConfigurationTags.SOURCE_FILTER_ATTRIBUTE;
import static org.apache.torque.gf.configuration.controller.SourceConfigurationTags.SOURCE_TAG;
import static org.apache.torque.gf.configuration.controller.SourceConfigurationTags.TRANSFORMER_TAG;
import static org.apache.torque.gf.configuration.controller.SourceConfigurationTags.TYPE_ATTRIBUTE;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.apache.torque.gf.configuration.ConfigurationException;
import org.apache.torque.gf.configuration.ConfigurationHandlers;
import org.apache.torque.gf.configuration.ConfigurationProvider;
import org.apache.torque.gf.configuration.paths.ProjectPaths;
import org.apache.torque.gf.source.FileSourcesImpl;
import org.apache.torque.gf.source.SourceType;
import org.apache.torque.gf.source.Sources;
import org.apache.torque.gf.source.TransformerDefinition;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Reads an Source from the controller configuration file.
 */
public class SourceSaxHandler extends DefaultHandler
{
    /** The access object for the configuration files, not null. */
    private ConfigurationProvider configurationProvider;

    /** The paths of the surrounding project, not null. */
    private ProjectPaths projectPaths;

    /** The known configuration handlers. */
    private ConfigurationHandlers configurationHandlers;

    /** The file type of the sources element which is currently parsed. */
    private String type;

    /** The file path of the sources element which is currently parsed. */
    private String path;

    /**
     * The start elements path of the sources element
     * which is currently parsed.
     */
    private String startElementsPath;

    /**
     * The source filter for the sources Element which is currently parsed.
     */
    private String sourceFilter;

    /**
     * The transformer definitions from inside the source file configuration.
     */
    private List<TransformerDefinition> transformerDefinitions
            = new ArrayList<TransformerDefinition>();

    /**
     * The sources element with the configuration in the current element.
     */
    private Sources sources;

    /** The handler which handles transformer elements. */
    private TransformerSaxHandler transformerSaxHandler;

    /** whether this handler has completed its task. */
    private boolean finished = false;

    /**
     * Constructor.
     *
     * @param configurationProvider The access object for the configuration
     *        files, not null.
     * @param projectPaths The paths of the surrounding project, not null.
     * @param configurationHandlers All known configuration handlers, not null.
     *
     * @throws NullPointerException if an argument is null.
     */
    public SourceSaxHandler(
            ConfigurationProvider configurationProvider,
            ProjectPaths projectPaths,
            ConfigurationHandlers configurationHandlers)
    {
        if (configurationProvider == null)
        {
            throw new NullPointerException(
                    "configurationProvider must not be null");
        }
        if (projectPaths == null)
        {
            throw new NullPointerException(
                    "projectPaths must not be null");
        }
        if (configurationHandlers == null)
        {
            throw new NullPointerException(
                    "configurationHandlers must not be null");
        }
        this.configurationProvider = configurationProvider;
        this.projectPaths = projectPaths;
        this.configurationHandlers = configurationHandlers;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void startElement(String uri, String localName, String rawName,
                             Attributes attributes)
            throws SAXException
    {
        if (transformerSaxHandler != null)
        {
            transformerSaxHandler.startElement(
                    uri, localName, rawName, attributes);
        }
        else if (TRANSFORMER_TAG.equals(rawName))
        {
            transformerSaxHandler
                    = new TransformerSaxHandler(
                            configurationProvider, projectPaths);
            transformerSaxHandler.startElement(
                    uri, localName, rawName, attributes);
        }
        else if (rawName.equals(SOURCE_TAG))
        {
            type = attributes.getValue(TYPE_ATTRIBUTE);
            path = attributes.getValue(PATH_ATTRIBUTE);
            if (path == null)
            {
                throw new SAXException(
                        "path must not be null for source");
            }
            startElementsPath = attributes.getValue(ELEMENTS_ATTRIBUTE);
            sourceFilter = attributes.getValue(SOURCE_FILTER_ATTRIBUTE);
        }
        else
        {
            throw new SAXException("Unknown element " + rawName);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void endElement(String uri, String localName, String rawName)
        throws SAXException
    {
        if (transformerSaxHandler != null)
        {
            transformerSaxHandler.endElement(
                    uri,
                    localName,
                    rawName);
            if (transformerSaxHandler.isFinished())
            {
                transformerDefinitions.add(new TransformerDefinition(
                        transformerSaxHandler.getSourceTransformer(),
                        transformerSaxHandler.getElements()));
                transformerSaxHandler = null;
            }
        }
        else if (rawName.equals(SOURCE_TAG))
        {
            try
            {
                SourceType sourceType = null;
                if (type != null)
                {
                    Set<SourceType> sourceTypes
                            = configurationHandlers.getSourceTypes();
                    for (SourceType candidate : sourceTypes)
                    {
                        if (type.equals(candidate.getKey()))
                        {
                            sourceType = candidate;
                            break;
                        }
                    }
                    if (sourceType == null)
                    {
                        throw new SAXException("Unknown source type : " + type
                                + " Known types are: " + sourceTypes);
                    }
                }
                sources = new FileSourcesImpl(
                        sourceType,
                        path,
                        startElementsPath,
                        sourceFilter,
                        transformerDefinitions,
                        configurationHandlers,
                        projectPaths.getDefaultSourcePath());
            }
            catch (ConfigurationException e)
            {
                throw new SAXException("Could not create source: "
                            + e.getMessage(),
                        e);
            }
            finished = true;
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void characters(char[] ch, int start, int length)
            throws SAXException
    {
        if (transformerSaxHandler != null)
        {
            transformerSaxHandler.characters(ch, start, length);
        }
    }

    /**
     * Returns the configuration filled with the contents of the parsed snippet.
     *
     * @return the configuration which was filled, not null if a
     *         matching snippet was processed.
     */
    public Sources getSources()
    {
        return sources;
    }

    /**
     * Returns whether the matching snippet was completely parsed.
     *
     * @return true if the matching snippet was completely parsed,
     *         false otherwise.
     */
    public boolean isFinished()
    {
        return finished;
    }
}
