package org.apache.torque.gf.configuration.generator;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.apache.torque.gf.configuration.ConfigurationException;
import org.apache.torque.gf.configuration.mergepoint.MergepointMapping;
import org.apache.torque.gf.generator.Generator;
import org.apache.torque.gf.qname.QualifiedName;

/**
 * Administers the available Generators.
 */
public class GeneratorConfiguration
{
    /** A map containing all generators, keyed by their name. */
    private Map<QualifiedName, Generator> generators
            = new HashMap<QualifiedName, Generator>();

    /** A map containing all isolated mergepoint mappings,
     *  keyed by their name. */
    private Map<String, MergepointMapping> mergepointMappings
            = new HashMap<String, MergepointMapping>();

    /**
     * Whether the mergepoint mappings have been resolved and added to
     * the respective generators.
     */
    private boolean mergepointMappingsResolved = false;

    /**
     * Creates a GeneratorConfiguration containing a list of Generators.
     *
     * @param generators the list of generators, not null.
     *
     * @throws NullPointerException if generatorList is null.
     */
    public GeneratorConfiguration(Collection<Generator> generators,
            Collection<MergepointMapping> mergepointMappings)
    {
        if (generators == null)
        {
            throw new NullPointerException("generators must not be null");
        }
        if (mergepointMappings == null)
        {
            throw new NullPointerException(
                    "mergepointMappings must not be null");
        }
        for (Generator generator : generators)
        {
            this.generators.put(generator.getName(), generator);
        }
        for (MergepointMapping mergepointMapping : mergepointMappings)
        {
            this.mergepointMappings.put(
                    mergepointMapping.getName(),
                    mergepointMapping);
        }
    }

    /**
     * Resolves the isolated mergepoint mappings and adds them to
     * the relevant generators.
     * @throws ConfigurationException
     */
    public void resolveMergepointMappings() throws ConfigurationException
    {
        for (Map.Entry<String, MergepointMapping> entry
                : mergepointMappings.entrySet())
        {
            String name = entry.getKey();
            QualifiedName qualifiedMergepointName
                = new QualifiedName(name);
            if (qualifiedMergepointName.getNamespace().isRoot())
            {
                // generator name is missing
                throw new ConfigurationException(
                        "The isolated mergepoint mapping with the name "
                        + name
                        + " needs to be qualified with the generator name");
            }
            QualifiedName generatorName = new QualifiedName(
                    qualifiedMergepointName.getNamespace().toString());
            Generator generator = generators.get(generatorName);
            if (generator == null)
            {
                throw new ConfigurationException(
                        "No generator with name "
                        + generatorName
                        + "exists (required by the isolated mergepoint mapping"
                        + " with the name "
                        + name
                        + ")");
            }
            MergepointMapping originalMergepointMapping = entry.getValue();
            MergepointMapping resolvedMergepointMapping
                    = new MergepointMapping(
                            qualifiedMergepointName.getName(),
                            originalMergepointMapping.getActions());
            generator.setMergepointMapping(resolvedMergepointMapping);
        }
        mergepointMappingsResolved = true;
        mergepointMappings.clear();
    }

    /**
     * Returns a map containing all the configured generators, keyed by their
     * name.
     *
     * @return all generators, not null.
     *
     * @throws IllegalStateException if the mergepoint mappings have not
     *         yet been resolved.
     */
    public Map<QualifiedName, Generator> getGenerators()
    {
        if (!mergepointMappingsResolved)
        {
            throw new IllegalStateException(
                    "Mergepoint mappings must be resoved first");
        }
        return Collections.unmodifiableMap(generators);
    }

    /**
     * Returns the generator with the name <code>name</code>.
     *
     * @param name the name of the generator to be returned.
     *
     * @return The generator with the given name, or null if it does not
     *         exist.
     *
     * @throws IllegalStateException if the mergepoint mappings have not
     *         yet been resolved.
     */
    public Generator getGenerator(QualifiedName name)
    {
        if (!mergepointMappingsResolved)
        {
            throw new IllegalStateException(
                    "Mergepoint mappings must be resoved first");
        }
        return generators.get(name);
    }

    /**
     * Adds a generator.
     *
     * @param generator the generator to be added, not null.
     *
     * @throws ConfigurationException if a generator with the generator's name
     *         already exists in the configuration.
     * @throws NullPointerException if generator is null.
     */
    public void addGenerator(Generator generator)
        throws ConfigurationException
    {
        if (generator == null)
        {
            throw new NullPointerException("generator must not be null");
        }
        if (generators.get(generator.getName()) != null)
        {
            throw new ConfigurationException("Trying to add the generator "
                    + generator.getName()
                    + " and class "
                    + generator.getClass().getName()
                    + " : A generator with the same name "
                    + " already exists, it has the class "
                    + getGenerator(generator.getName()).getClass().getName());
        }
        generators.put(generator.getName(), generator);
    }

    /**
     * Creates a String view of this object for debuggung purposes.
     *
     * @return a String view of this object, never null.
     *
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString()
    {
        StringBuffer result = new StringBuffer("(generators=[")
                .append(generators)
                .append(")");
        return result.toString();
    }

    /**
     * returns whether a generator for the given name exists.
     * Name and namespace must match exactly.
     *
     * @param qualifiedName the name of the generator.
     *
     * @return true if a generator with the name exists, false otherwise.
     */
    public boolean generatorExists(QualifiedName qualifiedName)
    {
        return generators.containsKey(qualifiedName);
    }
}
