package org.apache.torque.gf.configuration.generator;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import static org.apache.torque.gf.configuration.generator.GeneratorConfigurationTags.GENERATORS_TAG;
import static org.apache.torque.gf.configuration.generator.GeneratorConfigurationTags.GENERATOR_TAG;
import static org.apache.torque.gf.configuration.mergepoint.MergepointConfigurationTags.MERGEPOINT_TAG;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.torque.gf.configuration.ConfigurationEntityResolver;
import org.apache.torque.gf.configuration.ConfigurationHandlers;
import org.apache.torque.gf.configuration.ConfigurationProvider;
import org.apache.torque.gf.configuration.SaxHelper;
import org.apache.torque.gf.configuration.XMLConstants;
import org.apache.torque.gf.configuration.mergepoint.MergepointMapping;
import org.apache.torque.gf.configuration.mergepoint.MergepointSaxHandler;
import org.apache.torque.gf.configuration.paths.ProjectPaths;
import org.apache.torque.gf.generator.Generator;
import org.apache.torque.gf.qname.QualifiedName;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * A SAX handler which parses Generator configuration files.
 * Relies on delegate handlers for parsing the configuration for
 * the different generator types.
 */
public class GeneratorConfigurationSaxHandler extends DefaultHandler
{
    /** The access object for the configuration files, not null. */
    private ConfigurationProvider configurationProvider;

    /** The paths of the surrounding project, not null. */
    private ProjectPaths projectPaths;

    /** The list of generators which configuration was already parsed. */
    private List<Generator> generators = new ArrayList<Generator>();

    /**
     * The list of separate mergepoint mappings (outside the generators)
     * which configuration was already parsed.
     */
    private List<MergepointMapping> mergepointMappings
            = new ArrayList<MergepointMapping>();

    /** The available configuration handlers. */
    private ConfigurationHandlers configurationHandlers;

    /**
     * The current delegate handler for parsing the current generator's
     * configuration. Is null if no generator is currently parsed.
     */
    private GeneratorSaxHandler generatorHandler;

    /**
     * The current delegate handler for parsing the current mergepoint's
     * configuration. Is null if no mergepoint is currently parsed.
     */
    private MergepointSaxHandler mergepointHandler;

    /** The log. */
    private static Log log = LogFactory.getLog(GeneratorSaxHandler.class);

    /**
     * Constructor.
     *
     * @param configurationProvider The access object for the configuration
     *        files, not null.
     * @param projectPaths The paths of the surrounding project, not null.
     * @param configurationHandlers The available configuration handlers,
     *        not null.
     *
     * @throws NullPointerException if an argument is null.
     */
    public GeneratorConfigurationSaxHandler(
            ConfigurationProvider configurationProvider,
            ProjectPaths projectPaths,
            ConfigurationHandlers configurationHandlers)
    {
        if (configurationProvider == null)
        {
            log.error("GeneratorConfigurationSaxHandler: "
                    + " configurationProvider is null");
            throw new NullPointerException("configurationProvider is null");
        }
        if (projectPaths == null)
        {
            log.error("GeneratorConfigurationSaxHandler: "
                    + " projectPaths is null");
            throw new NullPointerException("projectPaths is null");
        }
        if (configurationHandlers == null)
        {
            log.error("GeneratorConfigurationSaxHandler: "
                    + " configurationHandlers is null");
            throw new NullPointerException("configurationHandlers is null");
        }
        this.configurationProvider = configurationProvider;
        this.projectPaths = projectPaths;
        this.configurationHandlers = configurationHandlers;
    }

    /**
     * Returns all generators which were configured in the parsed generator
     * configuration file.
     *
     * @return all created generators, not null.
     */
    public List<Generator> getGenerators()
    {
        return generators;
    }

    /**
     * Returns all mergepoint mappings which were configured
     * outside the generators in the parsed generator configuration file.
     *
     * @return all created mergepoint mappings, not null.
     */
    public List<MergepointMapping> getMergepointMappings()
    {
        return mergepointMappings;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void startElement(String uri, String localName, String qName,
                             Attributes attributes)
            throws SAXException
    {
        String unqualifiedName = SaxHelper.getUnqualifiedName(localName, qName);
        if (generatorHandler != null)
        {
            generatorHandler.startElement(
                    uri, localName, qName, attributes);
        }
        else if (mergepointHandler != null)
        {
            mergepointHandler.startElement(
                    uri, localName, qName, attributes);
        }
        else if (GENERATOR_TAG.equals(unqualifiedName))
        {
            String generatorType = getGeneratorType(attributes);
            generatorHandler = getGeneratorHandler(
                    null,
                    generatorType);
            generatorHandler.startElement(
                    uri, localName, qName, attributes);
        }
        else if (MERGEPOINT_TAG.equals(unqualifiedName))
        {
            mergepointHandler = new MergepointSaxHandler(
                    configurationProvider,
                    projectPaths,
                    configurationHandlers);
            mergepointHandler.startElement(
                    uri, localName, qName, attributes);
        }
        else if (!GENERATORS_TAG.equals(unqualifiedName))
        {
            throw new SAXException(
                    "Unknown element : " + unqualifiedName
                    + ". First element must be "
                    + GENERATORS_TAG
                    + " or "
                    + GENERATOR_TAG);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void endElement(String uri, String localName, String rawName)
            throws SAXException
    {
        if (generatorHandler != null)
        {
            generatorHandler.endElement(uri, localName, rawName);
            if (generatorHandler.isFinished())
            {
                Generator generator = generatorHandler.getGenerator();
                generators.add(generator);
                if (log.isDebugEnabled())
                {
                    log.debug("Parsed configuration for the generator "
                            + generator.getName());
                }
                generatorHandler = null;
            }
        }
        else if (mergepointHandler != null)
        {
            mergepointHandler.endElement(uri, localName, rawName);
            if (mergepointHandler.isFinished())
            {
                MergepointMapping mergepointMapping
                        = mergepointHandler.getMergepointMapping();
                mergepointMappings.add(mergepointMapping);
                if (log.isDebugEnabled())
                {
                    log.debug("Parsed configuration for the mergepoint "
                            + mergepointMapping.getName());
                }
                mergepointHandler = null;
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void characters (char[] ch, int start, int length)
            throws SAXException
    {
        if (generatorHandler != null)
        {
            generatorHandler.characters(ch, start, length);
        }
    }

    /**
     * Returns the correct handler for a generator tag.
     * The method uses the type attribute to determine which handler is needed.
     * I.e. it looks up the handler factory type in the generator types,
     * and asks the factory for a handler.
     *
     * @param generatorName the name for the generator which configuration
     *        will be read in by the generated SaxHandlerFactory,
     *        or null if the name of the generator should be determined from
     *        the parsed XML.
     * @param generatorType the type of the generator, not null.
     *
     * @return the handler for the tag, not null.
     *
     * @throws SAXException if no matching handler can be identified,
     *         or if an error occurs while creating the handler.
     */
    public GeneratorSaxHandler getGeneratorHandler(
            QualifiedName generatorName,
            String generatorType)
        throws SAXException
    {
        GeneratorSaxHandlerFactory delegateHandlerFactory
                = configurationHandlers.getGeneratorTypes()
                    .getGeneratorHandlerFactories()
                    .get(generatorType);
        if (delegateHandlerFactory == null)
        {
            throw new SAXException(
                    "Unknown generator type: "
                        + generatorType);
        }
        GeneratorSaxHandler generatorSaxHandler
                = delegateHandlerFactory.getGeneratorSaxHandler(
                        generatorName,
                        configurationProvider,
                        projectPaths,
                        configurationHandlers);
        return generatorSaxHandler;
    }

    /**
     * Reads the generator type from the attributes of the generator XML tag.
     *
     * @param attributes the attributes of the XML tag, not null.
     *
     * @return the generator type, not null.
     *
     * @throws SAXException if the xsi:type attribute is not set.
     */
    public static String getGeneratorType(Attributes attributes)
            throws SAXException
    {
        String generatorType
                = attributes.getValue(
                        XMLConstants.XSI_NAMESPACE,
                        XMLConstants.XSI_TYPE_ATTRBUTE_NAME);
        if (generatorType == null)
        {
            throw new SAXException("The tag " + GENERATOR_TAG
                    + " requires the attribute "
                    + XMLConstants.XSI_NAMESPACE
                    + ":"
                    + XMLConstants.XSI_TYPE_ATTRBUTE_NAME);
        }
        return generatorType;
    }

    /**
     * EntityResolver implementation. Called by the XML parser
     *
     * @param publicId The public identifier of the external entity.
     * @param systemId The system identifier of the external entity.
     *
     * @return an InputSource for the entity, or null if the URI is not known.
     *
     * @see ConfigurationEntityResolver#resolveEntity(String, String)
     */
    public InputSource resolveEntity(String publicId, String systemId)
            throws SAXException, IOException
    {
        return new ConfigurationEntityResolver().resolveEntity(
                publicId, systemId);
    }

    @Override
    public void error(SAXParseException exception) throws SAXParseException
    {
        throw exception;
    }

    @Override
    public void fatalError(SAXParseException exception)
            throws SAXParseException
    {
        throw exception;
    }

    @Override
    public void warning(SAXParseException exception) throws SAXParseException
    {
        throw exception;
    }
}
