package org.apache.torque.gf.configuration.generator;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.torque.gf.configuration.ConfigurationException;
import org.apache.torque.gf.configuration.ConfigurationHandlers;
import org.apache.torque.gf.configuration.ConfigurationProvider;
import org.apache.torque.gf.configuration.mergepoint.MergepointMapping;
import org.apache.torque.gf.configuration.paths.ProjectPaths;
import org.apache.torque.gf.generator.Generator;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.SAXParseException;

/**
 * Parses generator configuration files and creates a GeneratorConfiguration.
 */
public class GeneratorConfigurationXmlParser
{
    /**
     * The SaxParserFactory to create the sax parsers for reading in the
     * generator configuration files.
     */
    private static SAXParserFactory saxFactory;

    /** The logger. */
    private static Log log
            = LogFactory.getLog(GeneratorConfigurationXmlParser.class);

    static
    {
        saxFactory = SAXParserFactory.newInstance();
        saxFactory.setNamespaceAware(true);
        try
        {
            saxFactory.setFeature(
                    "http://xml.org/sax/features/validation",
                    true);
            saxFactory.setFeature(
                    "http://apache.org/xml/features/validation/schema", true);
        }
        catch (SAXNotSupportedException e)
        {
            throw new RuntimeException(e);
        }
        catch (SAXNotRecognizedException e)
        {
            throw new RuntimeException(e);
        }
        catch (ParserConfigurationException e)
        {
            throw new RuntimeException(e);
        }
    }

    /**
     * Reads all generator configuration files creates the generator
     * configuration from them.
     * All the generator configuration files known to the provide are parsed.
     * These are typically all XML files in the generator configuration
     * directory and its subdirectories.
     *
     * @param configurationProvider The access object for the configuration
     *        files, not null.
     * @param projectPaths The paths of the surrounding project, not null.
     * @param generatorTypes the configured types of generators, not null.
     *
     * @return the generator configuration.
     *
     * @throws ConfigurationException if the Configuration cannot be read
     *           or errors exists in the generator configuration files.
     */
    public GeneratorConfiguration readGeneratorConfiguration(
            ConfigurationProvider configurationProvider,
            ProjectPaths projectPaths,
            ConfigurationHandlers configurationHandlers)
        throws ConfigurationException
    {
        if (configurationHandlers == null)
        {
            log.error("GeneratorConfiguration: "
                    + " configurationHandlers is null");
            throw new NullPointerException("generatorTypes is null");
        }
        if (configurationProvider == null)
        {
            log.error("GeneratorConfiguration: "
                    + " configurationProvider is null");
            throw new NullPointerException("configurationProvider is null");
        }
        if (projectPaths == null)
        {
            log.error("GeneratorConfiguration: "
                    + " projectPaths is null");
            throw new NullPointerException("projectPaths is null");
        }

        List<Generator> allGenerators = new ArrayList<Generator>();
        List<MergepointMapping> allMergepointMappings
                = new ArrayList<MergepointMapping>();

        // Generators from all files
        Collection<String> generatorConfigNames
                = configurationProvider.getGeneratorConfigurationNames();

        for (String generatorConfigName : generatorConfigNames)
        {
            InputStream inputStream = null;
            try
            {
                inputStream
                        = configurationProvider.getGeneratorConfigurationInputStream(
                            generatorConfigName);
                GeneratorConfigFileContent fileContent
                        = readGeneratorConfig(
                                inputStream,
                                configurationProvider,
                                projectPaths,
                                configurationHandlers);
                allGenerators.addAll(fileContent.getGenerators());
                allMergepointMappings.addAll(
                        fileContent.getMergepointMappings());
            }
            catch (SAXParseException e)
            {
                throw new ConfigurationException(
                        "Error parsing generator configuration "
                            + generatorConfigName
                            + " at line "
                            + e.getLineNumber()
                            + " column "
                            + e.getColumnNumber()
                            + " : "
                            + e.getMessage(),
                        e);

            }
            catch (Exception e)
            {
                throw new ConfigurationException(
                        "Error parsing generator configuration "
                            + generatorConfigName,
                        e);
            }
            finally
            {
                if (inputStream != null)
                {
                    try
                    {
                        inputStream.close();
                    }
                    catch (IOException e)
                    {
                        log.warn("Could not close "
                                    + "generatorConfigurationInputStream "
                                    + generatorConfigName,
                                e);
                    }
                }
            }
        }
        return new GeneratorConfiguration(allGenerators, allMergepointMappings);
    }


    /**
     * Reads a generator configuration file and returns the generators
     * and isolated mergepoint mappings which are configured in this file.
     *
     * @param generatorConfigurationInputStream the stream containing the
     *        generator configuration.
     * @param configurationProvider The access object for the configuration
     *        files, not null.
     * @param projectPaths The paths of the surrounding project, not null.
     * @param generatorTypes the configured types of generators, not null.
     *
     * @return All the generators and isolated mergepoint mappings
     *         configured in the file.
     * @throws SAXException if an error occurs while parsing the configuration
     *         file.
     * @throws IOException if the file cannot be read.
     * @throws ParserConfigurationException if a serious parser configuration
     *         error occurs..
     */
    private GeneratorConfigFileContent readGeneratorConfig(
            InputStream generatorConfigurationInputStream,
            ConfigurationProvider configurationProvider,
            ProjectPaths projectPaths,
            ConfigurationHandlers configurationHandlers)
        throws SAXException, IOException, ParserConfigurationException
    {
        SAXParser parser = saxFactory.newSAXParser();
        GeneratorConfigurationSaxHandler saxHandler
                = new GeneratorConfigurationSaxHandler(
                        configurationProvider,
                        projectPaths,
                        configurationHandlers);
        InputSource is = new InputSource(generatorConfigurationInputStream);
        parser.parse(is, saxHandler);

        return new GeneratorConfigFileContent(
                saxHandler.getGenerators(),
                saxHandler.getMergepointMappings());
    }

    /**
     * The parsed content of a generator definition file.
     * Contains the parsed generators and the isolated
     * (i.e. outside generator context) mergepoints.
     */
    private static final class GeneratorConfigFileContent
    {
        /** The parsed generators. */
        private List<Generator> generators;

        /** The parsed isolated mergepoint mappings. */
        private List<MergepointMapping> mergepointMappings;

        /**
         * Constructor.
         *
         * @param generators the parsed generators.
         * @param mergepointMappings the isolated mergepoint mappings.
         */
        public GeneratorConfigFileContent(
                List<Generator> generators,
                List<MergepointMapping> mergepointMappings)
        {
            this.generators = generators;
            this.mergepointMappings = mergepointMappings;
        }

        /**
         * Returns the parsed generators.
         *
         * @return the parsed generators.
         */
        public List<Generator> getGenerators()
        {
            return generators;
        }

        /**
         * Returns the parsed isolated mergepoint mappings.
         *
         * @return the isolated mergepoint mappings.
         */
        public List<MergepointMapping> getMergepointMappings()
        {
            return mergepointMappings;
        }
    }
}
