package org.apache.torque.gf.configuration.generator;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import static org.apache.torque.gf.configuration.generator.GeneratorConfigurationTags.GENERATOR_CLASS_ATTRIBUTE;
import static org.apache.torque.gf.configuration.generator.GeneratorConfigurationTags.GENERATOR_NAME_ATTRIBUTE;
import static org.apache.torque.gf.configuration.mergepoint.MergepointConfigurationTags.MERGEPOINT_TAG;
import static org.apache.torque.gf.configuration.generator.GeneratorConfigurationTags.INPUT_TAG;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.torque.gf.configuration.ConfigurationHandlers;
import org.apache.torque.gf.configuration.ConfigurationProvider;
import org.apache.torque.gf.configuration.paths.ProjectPaths;
import org.apache.torque.gf.generator.Generator;
import org.apache.torque.gf.qname.QualifiedName;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * Handles a declaration of a java generator within a generator configuration
 * file.
 */
class JavaGeneratorSaxHandler extends GeneratorSaxHandler
{
    /** The logger of the class. */
    private static Log log = LogFactory.getLog(JavaGeneratorSaxHandler.class);

    /** the name of the property which is currently processed. */
    private String propertyName;

    /** The value of the property which is currently processed. */
    private StringBuffer propertyValue;

    /** The current nesting level inside the processed element. */
    private int level = 0;

    /**
     * Constructor.
     *
     * @param generatorName the name for the generator which configuration
     *        will be read in by the generated SaxHandlerFactory,
     *        or null if the name of the generator should be determined from
     *        the parsed xml.
     * @param configurationProvider The access object for the configuration
     *        files, not null.
     * @param projectPaths The paths of the surrounding project, not null.
     * @param configurationHandlers the available configuration handlers,
     *        not null.
     *
     * @throws SAXException if an error occurs during creation of the generator.
     */
    public JavaGeneratorSaxHandler(
            QualifiedName generatorName,
            ConfigurationProvider configurationProvider,
            ProjectPaths projectPaths,
            ConfigurationHandlers configurationHandlers)
        throws SAXException
    {
        super(generatorName,
                configurationProvider,
                projectPaths,
                configurationHandlers);
    }

    /**
     * Instantiates a java generator.
     *
     * @param generatorName the name for the generator which configuration
     *        will be read in by the generated SaxHandlerFactory,
     *        or null if the name of the generator should be determined from
     *        the parsed xml.
     * @param uri - The Namespace URI, or the empty string if the
     *        element has no Namespace URI or if Namespace processing is not
     *        being performed.
     * @param localName - The local name (without prefix), or
     *        the empty string if Namespace processing is not being performed.
     * @param rawName - The qualified name (with prefix), or the empty string if
     *        qualified names are not available.
     * @param attributes - The attributes attached to the element.
     *          If there are no attributes, it shall be an empty Attributes
     *          object.
     *
     * @return the created generator, not null.
     *
     * @throws SAXException if an error occurs during creation.
     */
    protected Generator createGenerator(
            QualifiedName generatorName,
            String uri,
            String localName,
            String rawName,
            Attributes attributes)
        throws SAXException
    {
        if (generatorName == null)
        {
            String nameAttribute
                    = attributes.getValue(GENERATOR_NAME_ATTRIBUTE);
            if (nameAttribute == null)
            {
                throw new SAXException("The attribute "
                        + GENERATOR_NAME_ATTRIBUTE
                        + " must be set on the element "
                        + rawName
                        + " for Java Generators");
            }
            generatorName = new QualifiedName(nameAttribute);
        }


        String className;
        {
            className = attributes.getValue(GENERATOR_CLASS_ATTRIBUTE);
            if (className == null)
            {
                throw new SAXException("The attribute "
                        + GENERATOR_CLASS_ATTRIBUTE
                        + " must be set on the element "
                        + rawName
                        + " for java Generators");
            }
        }

        Class<?> generatorClass;
        try
        {
            generatorClass = Class.forName(className);
        }
        catch (ClassNotFoundException e)
        {
            throw new SAXException(
                    "Error  while initializing the java generator "
                    + generatorName
                    + " : Could not load class " + className, e);
        }
        catch (ExceptionInInitializerError e)
        {
            log.error(
                    "Error  while initializing the java generator "
                    + generatorName
                    + " : Could not initialize class " + className
                    + " : " + e.getMessage());
            throw e;
        }
        catch (LinkageError e)
        {
            log.error(
                    "Error  while initializing the java generator "
                    + generatorName
                    + " : Could not link class " + className
                    + " : " + e.getMessage());
            throw e;
        }

        Generator result;
        try
        {
            Constructor<?> constructor
                    = generatorClass.getConstructor(QualifiedName.class);
            result = (Generator) constructor.newInstance(generatorName);
        }
        catch (NoSuchMethodException e)
        {
            throw new SAXException(
                    "Error  while instantiating the java generator "
                    + generatorName
                    + " : The class " + className
                    + " has no constructor which takes a qualified name",
                e);
        }
        catch (ClassCastException e)
        {
            throw new SAXException(
                    "Error  while instantiating the java generator "
                    + generatorName
                    + " : The class " + className
                    + " is not an instance of "
                    + Generator.class.getName(),
                e);
        }
        catch (IllegalAccessException e)
        {
            throw new SAXException(
                    "Error  while instantiating the java generator "
                    + generatorName
                    + " : The constructor of class "
                    + className + " could not be accessed",
                e);
        }
        catch (InvocationTargetException e)
        {
            throw new SAXException(
                    "Error  while instantiating the java generator "
                    + generatorName
                    + " : The constructor of class "
                    + className + " could not be called",
                e);
        }
        catch (InstantiationException e)
        {
            throw new SAXException(
                    "Error  while instantiating the java generator "
                    + generatorName
                    + " : The class " + className
                    + " represents an abstract class, "
                    + "an interface, an array class, a primitive type, "
                    + "or void, or the class has no parameterless constructor, "
                    + "or the instantiation fails for some other reason.",
                e);
        }
        catch (SecurityException e)
        {
            throw new SAXException(
                    "Error  while instantiating the java generator "
                    + generatorName
                    + " : The security manager denies instantiation",
                e);
        }

        return result;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void startElement(
            String uri,
            String localName,
            String rawName,
            Attributes attributes)
        throws SAXException
    {
        level++;
        if ((level == 2 && MERGEPOINT_TAG.equals(rawName))
            || super.isProcessingMergepointTag())
        {
            super.startElement(uri, localName, rawName, attributes);
        }
        else if (level == 2 && INPUT_TAG.equals(rawName))
        {
            super.startElement(uri, localName, rawName, attributes);
        }
        else if (level > 1)
        {
            propertyName = rawName;
            propertyValue = new StringBuffer();
        }
        else
        {
            super.startElement(uri, localName, rawName, attributes);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void endElement(String uri, String localName, String rawName)
        throws SAXException
    {
        level--;
        if (propertyName != null)
        {
            if (!PropertyUtils.isWriteable(getGenerator(), propertyName))
            {
                throw new SAXException("No setter found for property "
                        + propertyName
                        + " in class "
                        + getGenerator().getClass().getName());
            }
            try
            {
                BeanUtils.copyProperty(
                        getGenerator(), propertyName, propertyValue.toString());
            }
            catch (InvocationTargetException e)
            {
                throw new SAXException("error while setting Property "
                        + propertyName
                        + " for java generator "
                        + getGenerator().getName()
                        + " with value "
                        + propertyValue.toString(),
                    e);
            }
            catch (IllegalAccessException e)
            {
                throw new SAXException("error while setting Property "
                        + propertyName
                        + " for java generator "
                        + getGenerator().getName()
                        + " with value "
                        + propertyValue.toString(),
                    e);
            }
            propertyName = null;
            propertyValue = null;
        }
        else
        {
            super.endElement(uri, localName, rawName);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void characters(char[] ch, int start, int length)
            throws SAXException
    {
        if (propertyName == null)
        {
            return;
        }
        for (int i = start; i < start + length; ++i)
        {
            propertyValue.append(ch[i]);
        }
    }
}
