package org.apache.torque.gf.configuration.generator;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import static org.apache.torque.gf.configuration.generator.GeneratorConfigurationTags.GENERATOR_ENCODING_ATTRIBUTE;
import static org.apache.torque.gf.configuration.generator.GeneratorConfigurationTags.GENERATOR_NAME_ATTRIBUTE;
import static org.apache.torque.gf.configuration.generator.GeneratorConfigurationTags.GENERATOR_OPTIONS_IN_CONTEXT_ATTRIBUTE;
import static org.apache.torque.gf.configuration.generator.GeneratorConfigurationTags.GENERATOR_PATH_ATTRIBUTE;
import static org.apache.torque.gf.configuration.generator.GeneratorConfigurationTags.GENERATOR_SOURCE_ATTRIBUTES_IN_CONTEXT_ATTRIBUTE;
import static org.apache.torque.gf.configuration.generator.GeneratorConfigurationTags.GENERATOR_VARIABLES_IN_CONTEXT_ATTRIBUTE;

import org.apache.torque.gf.configuration.ConfigurationException;
import org.apache.torque.gf.configuration.ConfigurationHandlers;
import org.apache.torque.gf.configuration.ConfigurationProvider;
import org.apache.torque.gf.configuration.SaxHelper;
import org.apache.torque.gf.configuration.paths.ProjectPaths;
import org.apache.torque.gf.qname.QualifiedName;
import org.apache.torque.gf.template.velocity.VelocityGenerator;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

/**
 * Handles a declaration of a velocity generator within a generator
 * configuration file.
 */
class VelocityGeneratorSaxHandler extends GeneratorSaxHandler
{
    /**
     * Constructor.
     *
     * @param generatorName the name for the generator which configuration
     *        will be read in by the generated SaxHandlerFactory,
     *        or null if the name of the generator should be determined from
     *        the parsed XML.
     * @param configurationProvider The access object for the configuration
     *        files, not null.
     * @param projectPaths The paths of the surrounding project, not null.
     * @param configurationHandlers the available configuration handlers,
     *        not null.
     *
     * @throws SAXException if an error occurs during creation of the generator.
     */
    public VelocityGeneratorSaxHandler(
            QualifiedName generatorName,
            ConfigurationProvider configurationProvider,
            ProjectPaths projectPaths,
            ConfigurationHandlers configurationHandlers)
       throws SAXException
    {
        super(generatorName,
               configurationProvider,
               projectPaths,
               configurationHandlers);
    }

    /**
     * Instantiates and configures a velocity generator.
     *
     * @param generatorName the name for the generator which configuration
     *        will be read in by the generated SaxHandlerFactory,
     *        or null if the name of the generator should be determined from
     *        the parsed xml.
     * @param uri - The Namespace URI, or the empty string if the
     *        element has no Namespace URI or if Namespace processing is not
     *        being performed.
     * @param localName - The local name (without prefix), or
     *        the empty string if Namespace processing is not being performed.
     * @param rawName - The qualified name (with prefix), or the empty string if
     *        qualified names are not available.
     * @param attributes - The attributes attached to the element.
     *          If there are no attributes, it shall be an empty Attributes
     *          object.
     *
     * @return the created generator, not null.
     *
     * @throws SAXException if an error occurs during creation.
     */
    protected VelocityGenerator createGenerator(
            QualifiedName generatorName,
            String uri,
            String localName,
            String rawName,
            Attributes attributes)
        throws SAXException
    {
        if (generatorName == null)
        {
            String nameAttribute
                    = attributes.getValue(GENERATOR_NAME_ATTRIBUTE);
            if (nameAttribute == null)
            {
                throw new SAXException("The attribute "
                        + GENERATOR_NAME_ATTRIBUTE
                        + " must be set on the element "
                        + rawName
                        + " for Velocity Generators");
            }
            generatorName = new QualifiedName(nameAttribute);
        }

        String encoding = attributes.getValue(GENERATOR_ENCODING_ATTRIBUTE);
        String path = attributes.getValue(GENERATOR_PATH_ATTRIBUTE);

        try
        {
            VelocityGenerator result
                    = new VelocityGenerator(
                        generatorName,
                        getConfigurationProvider(),
                        path,
                        encoding);
            Boolean optionsInContext = SaxHelper.getBooleanAttribute(
                    GENERATOR_OPTIONS_IN_CONTEXT_ATTRIBUTE,
                    attributes,
                    "the velocityGenerator" + generatorName);
            if (optionsInContext != null)
            {
                result.setOptionsInContext(optionsInContext);
            }
            Boolean sourceElementAttributesInContext
                    = SaxHelper.getBooleanAttribute(
                            GENERATOR_SOURCE_ATTRIBUTES_IN_CONTEXT_ATTRIBUTE,
                            attributes,
                            "the velocityGenerator" + generatorName);
            if (sourceElementAttributesInContext != null)
            {
                result.setSourceAttributesInContext(
                        sourceElementAttributesInContext);
            }
            Boolean variablesInContext = SaxHelper.getBooleanAttribute(
                    GENERATOR_VARIABLES_IN_CONTEXT_ATTRIBUTE,
                    attributes,
                    "the velocityGenerator" + generatorName);
            if (variablesInContext != null)
            {
                result.setVariablesInContext(variablesInContext);
            }
            return result;
        }
        catch (ConfigurationException e)
        {
            throw new SAXException(e);
        }
    }
}
