package org.apache.torque.gf.configuration.paths;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.File;

/**
 * Implementation of ProjectPaths with custom paths.
 */
public class CustomProjectPaths implements ProjectPaths
{
    /**
     * The configuration root directory.
     */
    private File configurationDir;

    /**
     * the configuration root package.
     */
    private String configurationPackage;

    /**
     * The directory where the source files are located.
     */
    private File sourceDir;

    /**
     * The output directory for the files which are generated each time anew.
     */
    private File newFileTargetDir;

    /**
     * The output directory for the files which are modified on generation.
     */
    private File modifiedFileTargetDir;

    /**
     * Copy-constructor.
     * @param toCopy the default project paths to copy, not null.
     *
     * @throws NullPointerException if toCopy is null.
     */
    public CustomProjectPaths(ProjectPaths toCopy)
    {
        this.configurationDir = toCopy.getConfigurationPath();
        this.configurationPackage = toCopy.getConfigurationPackage();
        this.sourceDir = toCopy.getDefaultSourcePath();
        this.newFileTargetDir = toCopy.getNewFileTargetPath();
        this.modifiedFileTargetDir = toCopy.getModifiedFileTargetPath();
    }

    /**
     * Constructor.
     * @param configurationDir the configuration directory for the Torque
     *        generator files, or null.
     * @param sourceDir the default directory for the source files, or null.
     * @param targetDir the default directory for the generated files, or null.
     */
    public CustomProjectPaths(
            File configurationDir,
            String configurationPackage,
            File sourceDir,
            File newFileTargetDir,
            File modifiedFileTargetDir)
    {
        this.configurationDir = configurationDir;
        this.configurationPackage = configurationPackage;
        this.sourceDir = sourceDir;
        this.newFileTargetDir = newFileTargetDir;
        this.modifiedFileTargetDir = modifiedFileTargetDir;
    }

    /**
     * Sets the root directory for the Torque generator files,
     * absolute or relative to the project root.
     *
     * @param configurationDir the configuration directory for the Torque
     *        generator files, null to invalidate the current setting.
     */
    public void setConfigurationDir(File configurationDir)
    {
        this.configurationDir = configurationDir;
    }

    /**
     * Sets the root package for the Torque generator files,
     * relative to the project root.
     *
     * @param configurationPackage the configuration root package
     *        for the Torque generator files.
     */
    public void setConfigurationPackage(String configurationPackage)
    {
        this.configurationPackage = configurationPackage;
    }

    /**
     * Sets the default directory for the source files,
     * relative to the current directory, or absolute.
     * "Default" means that the setting can be overridden in a unit of
     * generation.
     *
     * @param sourceDir the default directory for the source files,
     *        null to invalidate the current setting.
     */
    public void setSourceDir(File sourceDir)
    {
        this.sourceDir = sourceDir;
    }

    /**
     * Sets the default directory for the files which are generated each time
     * anew, relative to the current directory, or absolute.
     *
     * @param targetDir the directory for the files which are generated
     * each time anew, null to invalidate the current setting.
     */
    public void setNewFileTargetDir(File targetDir)
    {
        this.newFileTargetDir = targetDir;
    }

    /**
     * Sets the default directory for the files which are modified on
     * generation, relative to the current directory, or absolute.
     *
     * @param targetDir the default directory for the generated files,
     *         null to invalidate the current setting.
     */
    public void setModifiedFileTargetDir(File targetDir)
    {
        this.modifiedFileTargetDir = targetDir;
    }

    /**
     * Returns the root directory for the Torque generator files,
     * relative to the project root.
     *
     * @return the directory for the Torque generator files, not null.
     * @throws IllegalStateException if one of the required parameters
     *         is not set.
     */
    public File getConfigurationPath()
    {
        if (!isInit())
        {
            throw new IllegalStateException(
                    "CustomProjectPaths is not initialized");
        }
        return configurationDir;
    }

    /**
     * Returns the root package of the Torque generator files.
     *
     * @return the root package of the Torque generator files.
     * @throws IllegalStateException if one of the required parameters
     *         is not set.
     */
    public String getConfigurationPackage()
    {
        if (!isInit())
        {
            throw new IllegalStateException(
                    "CustomProjectPaths is not initialized");
        }
        return configurationPackage;
    }


    /**
     * Returns the default directory for the source files,
     * relative to the project root. "Default" means that the setting
     * can be overridden in a unit of generation.
     *
     * @return the directory for the source files, not null.
     * @throws IllegalStateException if one of the required parameters
     *         is not set.
     */
    public File getDefaultSourcePath()
    {
        if (!isInit())
        {
            throw new IllegalStateException(
                    "CustomProjectPaths is not initialized");
        }
        return sourceDir;
    }

    /**
     * Returns the default subdirectory for files which are generated each
     * time anew, relative to the project root.
     *
     * @return the subdirectory for the files which are generated each
     *         time anew, not null.
     * @throws IllegalStateException if one of the required parameters
     *         is not set.
     */
    public File getNewFileTargetPath()
    {
        if (!isInit())
        {
            throw new IllegalStateException(
                    "CustomProjectPaths is not initialized");
        }
        return newFileTargetDir;
    }

    /**
     * Returns the default subdirectory for files which are not generated each
     * time anew, relative to the project root.
     *
     * @return the subdirectory for the files which are not generated each
     *         time anew, not null.
     * @throws IllegalStateException if one of the required parameters
     *         is not set.
     */
    public File getModifiedFileTargetPath()
    {
        if (!isInit())
        {
            throw new IllegalStateException(
                    "CustomProjectPaths is not initialized");
        }
        return modifiedFileTargetDir;
    }

    /**
     * returns whether the current settings are valid.
     *
     * It is checked whether all necessary informations are set.
     *
     * @return true if the current settings are valid, false otherwise.
     */
    public boolean isInit()
    {
        return (configurationDir != null || configurationPackage != null)
            && sourceDir != null
            && newFileTargetDir != null
            && modifiedFileTargetDir != null;
    }

    @Override
    public String toString()
    {
        return "(CustomProjectPaths: configurationDir=" + configurationDir
            + ", configurationPackage=" + configurationPackage
            + ", sourceDir=" + sourceDir
            + ", newFileTargetDir=" + newFileTargetDir
            + ", modifiedFileTargetDir=" + modifiedFileTargetDir
            + ")";
    }
}
