package org.apache.torque.gf.configuration.paths;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.File;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Contains information of the default integration of the generator
 * into a maven 2 project.
 */
public abstract class Maven2ProjectPaths implements ProjectPaths
{
    /** The log for the class. */
    private static Log log
            = LogFactory.getLog(Maven2ProjectPaths.class);

    /**
     * Default configuration root directory,
     * relative to the project root.
     */
    protected static final String CONFIG_DIR = "src/main/torque-gf/";

    /**
     * Default generation source file directory,
     * relative to the configuration root directory.
     */
    protected static final String SOURCE_DIR = CONFIG_DIR + "src";

    /**
     * Default generation target directory for new files,
     * relative to the project root.
     */
    protected static final String NEW_FILE_TARGET_DIR
            = "target/generated-sources";

    /**
     * Default generation target directory for modified files,
     * relative to the project root.
     */
    protected static final String MODIFIED_FILE_TARGET_DIR
            = "src/main/generated-sources";

    /**
     * the root directory of the whole maven2 project.
     */
    private File projectRoot;

    /**
     * Constructor.
     *
     * @param projectRoot path to the project root directory, not null.
     *        The path must either be absolute or relative to the current
     *        working directory.
     *
     * @throws NullPointerException if projectRoot is null.
     */
    protected Maven2ProjectPaths(File projectRoot)
    {
        if (projectRoot == null)
        {
            log.error("projectRoot is null");
            throw new NullPointerException("projectRoot is null");
        }
        this.projectRoot = projectRoot;
    }

    public abstract File getConfigurationPath();

    public abstract String getConfigurationPackage();

    /**
     * Returns the path to the source files.
     *
     * @return the path to for the source files, not null.
     */
    public File getDefaultSourcePath()
    {
        return new File(projectRoot, SOURCE_DIR);
    }

    /**
     * Returns the default subdirectory for the files which are generated
     * each time anew, relative to the project root.
     *
     * @return the subdirectory for the generated files, not null.
     */
    public File getNewFileTargetPath()
    {
        return new File(projectRoot, NEW_FILE_TARGET_DIR);
    }

    /**
     * Returns the default subdirectory for files which are not generated
     * each time anew, relative to the project root.
     *
     * @return the subdirectory for the generated files, not null.
     */
    public File getModifiedFileTargetPath()
    {
        return new File(projectRoot, MODIFIED_FILE_TARGET_DIR);
    }

    /**
     * returns the root directory of the whole maven 2 project.
     *
     * @return the root directory of the whole project, not null.
     */
    protected File getProjectRoot()
    {
        return projectRoot;
    }
}
