package org.apache.torque.gf.control;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Properties;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.log4j.PropertyConfigurator;
import org.apache.torque.gf.configuration.Configuration;
import org.apache.torque.gf.configuration.ConfigurationException;
import org.apache.torque.gf.configuration.UnitConfiguration;
import org.apache.torque.gf.configuration.UnitDescriptor;
import org.apache.torque.gf.configuration.controller.GeneratorReference;
import org.apache.torque.gf.configuration.controller.Output;
import org.apache.torque.gf.configuration.generator.GeneratorConfiguration;
import org.apache.torque.gf.generator.Generator;
import org.apache.torque.gf.generator.GeneratorException;
import org.apache.torque.gf.source.Source;
import org.apache.torque.gf.source.SourceElement;
import org.apache.torque.gf.source.SourceException;
import org.apache.torque.gf.source.SourcePath;
import org.apache.torque.gf.source.Sources;
import org.apache.torque.gf.source.TransformerDefinition;
import org.apache.torque.gf.source.transform.SourceTransformer;
import org.apache.torque.gf.source.transform.SourceTransformerException;

/**
 * Reads the configuration and generates the output accordingly.
 */
public class Controller
{
    /** The logger. */
    private static Log log = LogFactory.getLog(Controller.class);


    /**
     * Executes the controller action.
     *
     * @param unitDescriptors the units of generation to execute.
     *
     * @throws ControllerException if a ControllerException occurs during
     *         processing.
     * @throws ConfigurationException if a ConfigurationException occurs during
     *         processing.
     * @throws GeneratorException if a GeneratorException occurs during
     *         processing.
     * @throws SourceException if a SourceException occurs during
     *         processing.
     * @throws SourceTransformerException if a SourceTransformerException
     *         occurs during processing.
     * @throws IOException if a IOException occurs during processing.
     */
    public void run(List<UnitDescriptor> unitDescriptors)
        throws ControllerException, GeneratorException, SourceException,
            SourceTransformerException, IOException
    {
        initLogging();

        log.info("run() : Starting to read configuration files");
        Configuration configuration = new Configuration();
        configuration.addUnits(unitDescriptors);
        configuration.read();
        log.info("run() : Configuration read.");

        List<UnitConfiguration> unitConfigurations
                = configuration.getUnitConfigurations();
        ControllerState controllerState = new ControllerState();
        for (UnitConfiguration unitConfiguration : unitConfigurations)
        {
            unitConfiguration.getLoglevel().apply();
            log.debug("run() : Loglevel applied.");
            controllerState.setUnitConfiguration(unitConfiguration);
            List<Output> outputList = unitConfiguration.getOutputFiles();
            for (Output output : outputList)
            {
                log.debug("Processing output " + output.getFilename());
                controllerState.setOutput(output);
                File targetDirectory;
                if (output.isAlwaysNew())
                {
                    targetDirectory
                        = unitConfiguration.getNewFileTargetDirectory();
                    log.debug("Using newFileTargetDirectory "
                            + targetDirectory);
                }
                else
                {
                    targetDirectory
                        = unitConfiguration.getModifiedFileTargetDirectory();
                    log.debug("Using modifiedFileTargetDirectory "
                            + targetDirectory);
                }

                Sources sources = output.getSources();
                if (!sources.hasNext())
                {
                    log.info("No sources found");
                }

                while (sources.hasNext())
                {
                    Source source = sources.next();
                    log.debug("Processing source " + source.getDescription());
                    SourceElement rootElement = source.getRootElement();
                    controllerState.setSourceFile(source.getSourceFile());
                    rootElement = transformSource(
                            rootElement,
                            source.getTransformerDefinitions(),
                            controllerState);
                    controllerState.setRootElement(rootElement);

                    String startElementsPath = source.getStartElementsPath();
                    List<SourceElement> startElements
                            = SourcePath.getElementsFromRoot(
                                    rootElement,
                                    startElementsPath);
                    if (startElements.isEmpty())
                    {
                        log.info("No start Elements found for path "
                                + startElementsPath);
                    }
                    for (SourceElement startElement : startElements)
                    {
                        processStartElement(
                                startElement,
                                output,
                                source,
                                targetDirectory,
                                unitConfiguration,
                                controllerState);
                    }
                }
            }
        }
        controllerState.getVariableStore().endGeneration();
    }

    /**
     * Initializes the Logging.
     */
    protected void initLogging()
    {
        InputStream log4jStream
                = Controller.class.getClassLoader().getResourceAsStream(
                    "org/apache/torque/gf/log4j.properties");
        Properties log4jProperties = new Properties();
        try
        {
            log4jProperties.load(log4jStream);
        }
        catch (IOException e)
        {
            throw new RuntimeException(e);
        }
        PropertyConfigurator.configure(log4jProperties);
    }

    /**
     * Creates the output file name and sets it in the output.
     * The filename is calculated either by the filenameConfigurator in
     * <code>output</code> or is given explicitly (in the latter case
     * nothing needs to be done).
     *
     * @param controllerState the controller state, not null.
     * @param output The output to  process, not null.
     *
     * @throws ConfigurationException if an incorrect configuration is
     *         encountered, e.g. if neither filename nor filenameGenerator is
     *         set in output.
     * @throws GeneratorException if an error occurs during generation of
     *         the output filename.
     */
    protected void createOutputFilename(
                Output output,
                ControllerState controllerState)
            throws GeneratorException
    {
        if (output.getFilenameGenerator() == null)
        {
            if (output.getFilename() == null)
            {
                throw new ConfigurationException(
                    "neither filename nor filenameGenerator are set"
                        + " on output" + output);
            }
        }
        else
        {
            if (log.isDebugEnabled())
            {
                log.debug("Start generation of Output File path");
            }
            controllerState.setOutputFile(null);

            Generator filenameGenerator
                    = output.getFilenameGenerator();
            GeneratorReference contentGeneratorReference
                    = new GeneratorReference(
                            filenameGenerator.getName());
            controllerState.setRootGeneratorReference(
                    contentGeneratorReference);
            // use the namespace not of the filenameGenerator
            // but of the real generator
            // TODO: is this a good idea ? make configurable ?
            controllerState.setGeneratorNamespace(
                    output.getContentGenerator().getNamespace());
            filenameGenerator.beforeExecute(controllerState);
            String filename = filenameGenerator.execute(controllerState);
            filenameGenerator.afterExecute(controllerState);
            if (log.isDebugEnabled())
            {
                log.debug("End generation of Output File path, result is "
                        + filename);
            }
            output.setFilename(filename);
        }
    }

    /**
     * Processes the generation for a single start Element in a source.
     *
     * @param startElement the start element to process.
     * @param output the current output, not null.
     * @param source the current source, not null.
     * @param targetDirectory the directory in which the output files
     *        should be processed, not null
     * @param unitConfiguration the current unit configuration, not null.
     * @param controllerState the current controller state, not null.
     *
     * @throws ControllerException if startElement is null or the configured
     *         generator does not exist.
     * @throws GeneratorException if the generator throws an exception
     *         during execution.
     * @throws IOException if the output directory cannot be created or the
     *         output file cannot be written.
     */
    private void processStartElement(
                SourceElement startElement,
                Output output,
                Source source,
                File targetDirectory,
                UnitConfiguration unitConfiguration,
                ControllerState controllerState)
            throws ControllerException, GeneratorException, IOException
    {
        if (startElement == null)
        {
            throw new ControllerException(
                "Null start element found in source "
                    + "for generating the filename "
                    + "of output file "
                    + output);
        }
        controllerState.setSourceElement(startElement);
        log.debug("Processing new startElement "
                + startElement.getName());

        createOutputFilename(output, controllerState);
        controllerState.setOutputFile(
                new File(
                    targetDirectory,
                    output.getFilename()));

        if (output.isSkipIfExists())
        {
            File outputFile
                    = controllerState.getOutputFile();
            if (outputFile.exists())
            {
                log.info("Output file "
                        + outputFile.getAbsolutePath()
                        + " exists and skipIfExists"
                        + " is set to true, skipping");
                return;
            }
        }
        if (log.isInfoEnabled())
        {
            log.info("Start generation of File "
                    + controllerState.getOutputFile());
        }

        GeneratorReference contentGeneratorConfiguration
                = output.getContentGenerator();
        controllerState.setGeneratorNamespace(
                contentGeneratorConfiguration.getNamespace());
        controllerState.setRootGeneratorReference(
                contentGeneratorConfiguration);

        GeneratorConfiguration generatorConfiguration
                = unitConfiguration.getGeneratorConfiguration();

        Generator generator
                = generatorConfiguration.getGenerator(
                        contentGeneratorConfiguration.getName());
        if (generator == null)
        {
            throw new ControllerException(
                    "No generator configured for generator name \""
                        + contentGeneratorConfiguration.getName()
                        + "\"");
        }

        if (source.getSourceFilter() != null)
        {
            if (!source.getSourceFilter().proceed(controllerState))
            {
                log.debug("Source filter decided to skip "
                        + "generation of file "
                        + controllerState.getOutputFile());
                return;
            }
            else
            {
                log.debug("Source filter decided to proceed");
            }
        }

        {
            File parentOutputDir
                    = controllerState.getOutputFile().getParentFile();
            if (parentOutputDir != null
                    && !parentOutputDir.isDirectory())
            {
                boolean success = parentOutputDir.mkdirs();
                if (!success)
                {
                    throw new IOException(
                            "Could not create directory \""
                                + parentOutputDir.getAbsolutePath()
                                + "\"");
                }
            }
        }

        generator.beforeExecute(controllerState);
        String result = generator.execute(controllerState);
        generator.afterExecute(controllerState);

        FileWriter fileWriter = null;
        try
        {
            fileWriter = new FileWriter(
                    controllerState.getOutputFile());
            fileWriter.append(result);
        }
        finally
        {
            if (fileWriter != null)
            {
                fileWriter.close();
            }
        }

        controllerState.getVariableStore().endFile();
        if (log.isDebugEnabled())
        {
            log.debug("End generation of Output File "
                    + controllerState.getOutputFile());
        }
    }

    /**
     * Applies all tarnsformer definitions to the current source.
     *
     * @param rootElement the root element of the source to transform,
     *        not null.
     * @param transformerDefinitions the transformer definitions to apply,
     *        not null.
     * @param controllerState the current controller state, not null.
     *
     * @return the transformed root element, not null.
     */
    protected SourceElement transformSource(
                    final SourceElement rootElement,
                    final List<TransformerDefinition> transformerDefinitions,
                    final ControllerState controllerState)
            throws SourceTransformerException, SourceException
    {
        SourceElement result = rootElement;
        for (TransformerDefinition transformerDefinition
                : transformerDefinitions)
        {
            SourceTransformer sourceTransformer
                    = transformerDefinition.getTransformer();
            String elements = transformerDefinition.getElements();
            log.debug("Applying transformer "
                    + sourceTransformer.getClass().getName()
                    + (elements == null
                            ? " to the root element"
                            : " to the elements " + elements));

            List<SourceElement> toTransform
                    = SourcePath.getElementsFromRoot(rootElement, elements);
            if (toTransform.isEmpty())
            {
                log.debug("No element found, nothing transformed");
            }
            for (SourceElement sourceElement : toTransform)
            {
                log.debug("transforming element " + sourceElement);
                SourceElement transformedElement = sourceTransformer.transform(
                        sourceElement,
                        controllerState);
                if (transformedElement == null)
                {
                    throw new SourceTransformerException("Transformer "
                            + sourceTransformer.getClass().getName()
                            + " returned null for element "
                            + sourceElement.getName());
                }
                SourceElement parent = sourceElement.getParent();
                if (parent == null)
                {
                    result = transformedElement;
                }
                else
                {
                    int index = parent.getChildIndex(sourceElement);
                    parent.removeChild(sourceElement);
                    parent.addChild(index, transformedElement);
                }
            }
            log.debug("Transformation ended");
        }
        return result;
    }
}
