package org.apache.torque.gf.control.action;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.util.List;

import org.apache.torque.gf.control.ControllerState;
import org.apache.torque.gf.control.TokenReplacer;
import org.apache.torque.gf.generator.Generator;
import org.apache.torque.gf.generator.GeneratorException;
import org.apache.torque.gf.qname.Namespace;
import org.apache.torque.gf.qname.QualifiedName;
import org.apache.torque.gf.source.SourceElement;
import org.apache.torque.gf.source.SourcePath;

/**
 * Applies a generator to the matching element.
 * The output of the generator is appended to the output.
 */
public class ApplyAction implements MergepointAction
{
    /**
     * The path to the source element to be processed, not null.
     */
    private String path;

    /**
     * The name of the generator to be invoked on the element, not null.
     */
    private String generatorName;

    /**
     * Whether it is correct to have no element to apply.
     */
    private boolean acceptNotSet = true;

    /**
     * Constructor.
     *
     * @param path The path to the source element to be processed.
     *        <code>null</code> means ".".
     * @param generatorName the name of the generator to apply.
     * @param acceptNotSet true if no error should be thrown if no element
     *        matches the given path, true if an error should be thrown in
     *        such a case. null means true.
     */
    public ApplyAction(String path, String generatorName, Boolean acceptNotSet)
    {
        if (path == null)
        {
            this.path = ".";
        }
        else
        {
            this.path = path;
        }

        if (generatorName == null)
        {
            throw new IllegalArgumentException(
                    "ApplyAction: generatorName must not be null");
        }
        this.generatorName = generatorName;
        if (acceptNotSet != null)
        {
            this.acceptNotSet = acceptNotSet.booleanValue();
        }
    }

    /**
     * Applies a generator to the matching element.
     * The output of the generator is appended to the output.
     * ${...} Tokens are replaced within generatorName and path.
     */
    public String execute(ControllerState controllerState)
        throws GeneratorException
    {
        TokenReplacer tokenReplacer = new TokenReplacer(controllerState);
        String detokenizedGeneratorName = tokenReplacer.process(generatorName);
        QualifiedName qualifiedGeneratorName = new QualifiedName(
                detokenizedGeneratorName,
                Namespace.ROOT_NAMESPACE);
        Generator generator
                = controllerState.getUnitConfiguration()
                    .getGeneratorConfiguration()
                    .getGenerator(qualifiedGeneratorName);
        if (generator == null)
        {
            throw new GeneratorException("ApplyAction : The generator "
                    + generatorName
                    + " does not exist");
        }

        SourceElement currentElement = controllerState.getSourceElement();
        String detokenizedPath = tokenReplacer.process(path);

        List<SourceElement> selectedElements
                = SourcePath.getElements(currentElement, detokenizedPath);
        if (!acceptNotSet && selectedElements.isEmpty())
        {
            throw new GeneratorException(
                    "ApplyAction : selected path "
                        + path
                        + " does not match an element"
                        + " and acceptNotSet was set to false");
        }
        if (selectedElements.size() > 1)
        {
            throw new GeneratorException(
                    "ApplyAction : selected path "
                        + path
                        + " contains more than one element ("
                        + selectedElements.size()
                        + " times)");
        }
        StringBuffer result = new StringBuffer();
        for (SourceElement sourceElement : selectedElements)
        {
            controllerState.setSourceElement(sourceElement);
            generator.beforeExecute(controllerState);
            result.append(generator.execute(controllerState));
            generator.afterExecute(controllerState);
        }
        controllerState.setSourceElement(currentElement);
        return result.toString();
    }

    @Override
    public String toString()
    {
        return "(ApplyAction: path = "
                + path
                + ", generator = "
                + generatorName
                + ", acceptNotSet = "
                + acceptNotSet
                + ")";
    }

    @Override
    public int hashCode()
    {
        final int prime = 31;
        final int secondPrime = 1231;
        final int thirdPrime = 1237;
        int result = 1;
        result = prime * result + (acceptNotSet ? secondPrime : thirdPrime);
        result = prime * result + path.hashCode();
        result = prime * result + generatorName.hashCode();
        return result;
    }

    @Override
    public boolean equals(Object obj)
    {
        if (this == obj)
        {
            return true;
        }
        if (obj == null)
        {
            return false;
        }
        if (getClass() != obj.getClass())
        {
            return false;
        }
        final ApplyAction other = (ApplyAction) obj;
        if (acceptNotSet != other.acceptNotSet)
        {
            return false;
        }
        if (!path.equals(other.path))
        {
            return false;
        }
        if (!generatorName.equals(other.generatorName))
        {
            return false;
        }
        return true;
    }
}
