package org.apache.torque.gf.control.action;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.util.List;

import org.apache.torque.gf.configuration.generator.GeneratorConfiguration;
import org.apache.torque.gf.control.ControllerState;
import org.apache.torque.gf.control.TokenReplacer;
import org.apache.torque.gf.generator.Generator;
import org.apache.torque.gf.generator.GeneratorException;
import org.apache.torque.gf.qname.Namespace;
import org.apache.torque.gf.qname.QualifiedName;
import org.apache.torque.gf.source.SourceElement;
import org.apache.torque.gf.source.SourcePath;

/**
 * Traverses all matching elements, and applies a generator to each matching
 * element. The output of each generator is appended to the output.
 */
public class TraverseAllAction implements MergepointAction
{
    /**
     * The filter for the elements to traverse, not null.
     */
    private String elementsToTraverseName;

    /**
     * The name of the generator to be invoked for each element, not null.
     */
    private String generatorName;

    /**
     * Whether it is correct to have no element to traverse.
     */
    private boolean acceptEmpty = true;

    /**
     * Constructor.
     *
     * @param elementsToTraverseName the elements to traverse, not null.
     * @param generatorName the name of the generator to be invoked for each
     *        element, not null.
     * @param acceptEmpty true if it is correct if no matching elements are
     *        found, false to throw an error if no matching elements are found,
     *        null means true.
     */
    public TraverseAllAction(
            String elementsToTraverseName,
            String generatorName,
            Boolean acceptEmpty)
    {
        if (elementsToTraverseName == null)
        {
            throw new IllegalArgumentException(
                    "elementsToTraverseName must not be null");
        }
        this.elementsToTraverseName = elementsToTraverseName;

        if (generatorName == null)
        {
            throw new IllegalArgumentException(
                    "generatorName must not be null");
        }
        this.generatorName = generatorName;
        if (acceptEmpty != null)
        {
            this.acceptEmpty = acceptEmpty.booleanValue();
        }
    }

    /**
     * Traverses all matching elements, and applies a generator to each matching
     * element. The output of each generator is appended to the output.
     * ${...} Tokens are replaced within generatorName and
     * elementsToTraverseName.
     */
    public String execute(ControllerState controllerState)
        throws GeneratorException
    {
        TokenReplacer tokenReplacer = new TokenReplacer(controllerState);

        Generator generator;
        {
            GeneratorConfiguration generatorConfiguration
                    = controllerState.getUnitConfiguration()
                        .getGeneratorConfiguration();
            String detokenizedGeneratorName
                    = tokenReplacer.process(generatorName);
            QualifiedName generatorQName = new QualifiedName(
                    detokenizedGeneratorName,
                    Namespace.ROOT_NAMESPACE);

            generator = generatorConfiguration.getGenerator(generatorQName);
            if (generator == null)
            {
                throw new GeneratorException("TraverseAllAction : The generator "
                        + generatorName
                        + " does not exist");
            }
        }

        SourceElement currentElement = controllerState.getSourceElement();
        String detokenizedElementToTraverseName
                = tokenReplacer.process(elementsToTraverseName);

        List<SourceElement> selectedElements
                = SourcePath.getElements(
                        currentElement,
                        detokenizedElementToTraverseName);
        if (!acceptEmpty && selectedElements.isEmpty())
        {
            throw new GeneratorException(
                    "TraverseAllAction : selected element "
                        + elementsToTraverseName
                        + " does not exist and acceptEmpty was set to false");
        }

        StringBuffer result = new StringBuffer();
        for (SourceElement sourceElement : selectedElements)
        {
            controllerState.setSourceElement(sourceElement);
            generator.beforeExecute(controllerState);
            result.append(generator.execute(controllerState));
            generator.afterExecute(controllerState);
        }
        controllerState.setSourceElement(currentElement);
        return result.toString();
    }

    @Override
    public String toString()
    {
        return "(TraverseAllAction: element = "
                + elementsToTraverseName
                + ", generator = "
                + generatorName
                + ", acceptEmpty = "
                + acceptEmpty
                + ")";
    }

    @Override
    public int hashCode()
    {
        final int prime = 31;
        final int secondPrime = 1231;
        final int thirdPrime = 1237;
        int result = 1;
        result = prime * result + (acceptEmpty ? secondPrime : thirdPrime);
        result = prime * result + elementsToTraverseName.hashCode();
        result = prime * result + generatorName.hashCode();
        return result;
    }

    @Override
    public boolean equals(Object obj)
    {
        if (this == obj)
        {
            return true;
        }
        if (obj == null)
        {
            return false;
        }
        if (getClass() != obj.getClass())
        {
            return false;
        }

        final TraverseAllAction other = (TraverseAllAction) obj;
        if (acceptEmpty != other.acceptEmpty)
        {
            return false;
        }
        if (!elementsToTraverseName.equals(other.elementsToTraverseName))
        {
            return false;
        }
        if (!generatorName.equals(other.generatorName))
        {
            return false;
        }
        return true;
    }
}
