package org.apache.torque.gf.source;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.torque.gf.configuration.ConfigurationException;
import org.apache.torque.gf.source.filter.SourceFilter;
import org.apache.torque.gf.source.xml.SourceToXml;

/**
 * A Source which uses a file as input.
 */
public class FileSource extends SourceImpl
{
    /**
     * The log of the class.
     */
    private static Log log = LogFactory.getLog(FileSource.class);

    /**
     * The type of the file.
     */
    private SourceType type;

    /**
     * The path of the file.
     */
    private File path;

    /**
     * Constructor.
     *
     * @param type the source type, not null.
     * @param path the path to the file to read, not null.
     * @param elements the element which should be used as input, or null
     *         if the root element should be used.
     * @param sourceFilter the source filter class for the input.
     *
     * @throws ConfigurationException if the SourceFilter cannot be
     *         instantiated.
     * @throws IllegalArgumentException if the source type is unknown.
     * @throws NullPointerException if path is null.
     */
    public FileSource(
            SourceType type,
            File path,
            String elements,
            SourceFilter sourceFilter,
            List<TransformerDefinition> transformerDefinitions)
        throws ConfigurationException
    {
        super(elements, sourceFilter, transformerDefinitions);
        if (path == null)
        {
            throw new NullPointerException("path must not be null");
        }
        if (type == null)
        {
            throw new NullPointerException("type must not be null");
        }
        this.type = type;
        this.path = path;
    }

    /**
     * Returns the type of the source.
     *
     * @return the source type, not null.
     */
    public SourceType getType()
    {
        return type;
    }

    /**
     * Returns the path to the source file.
     *
     * @return the path to the source file, not null.
     */
    public File getPath()
    {
        return path;
    }

    /**
     * Reads and parses the input file and creates the element tree from it.
     *
     * @throws SourceException if the input file cannot be read or parsed.
     *
     * @return the root element of the element tree.
     */
    @Override
    public SourceElement createRootElement() throws SourceException
    {
        if (log.isDebugEnabled())
        {
            log.debug("start creating root Element");
        }
        SourceElement result;
        InputStream inputStream = null;
        try
        {
            inputStream = new FileInputStream(path);
            log.debug("Reading file "
                    + path.getAbsolutePath()
                    + " of type "
                    + type.getKey());
            result = type.getParser().parse(inputStream);
        }
        catch (FileNotFoundException e)
        {
            throw new SourceException(
                    "File not found: " + path.getAbsolutePath(),
                    e);
        }
        finally
        {
            if (inputStream != null)
            {
                try
                {
                    inputStream.close();
                }
                catch (IOException e)
                {
                    log.error("error closing input stream", e);
                }
            }
        }


        if (log.isDebugEnabled())
        {
            log.debug("finished creating root Element, source is\n"
                    + new SourceToXml().toXml(result));
        }
        return result;
    }

    /**
     * Returns the path of the file as a description.
     *
     * @return path of the file, not null.
     *
     * @see Source#getDescription()
     */
    public String getDescription()
    {
        return path.getAbsolutePath();
    }

    /**
     * Returns the source file, if it exists.
     *
     * @return the source file, or null if the source is not read from a file.
     * @return
     */
    public File getSourceFile()
    {
        return path;
    }


    @Override
    public String toString()
    {
        StringBuffer result = new StringBuffer();
        result.append("(path=").append(path)
                .append(",type=").append(type)
                .append(",startElementsPath=").append(getStartElementsPath())
                .append(",transformerDefinitions=")
                .append(getTransformerDefinitions());
        return result.toString();
    }
}
