package org.apache.torque.gf.source;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.IOCase;
import org.apache.commons.lang.StringUtils;
import org.apache.torque.gf.configuration.ConfigurationException;
import org.apache.torque.gf.configuration.ConfigurationHandlers;

/**
 * A collection of Sources, which all have the same elements,
 * Source filters and transformerDefinitions and which are read from files
 * in the file system.
 *
 * $Id: $
 */
public class FileSourcesImpl extends SimpleSourcesImpl
{
    /**
     * The type of the file(s), or null if the type should be determined
     * from the file name extension.
     */
    private SourceType type;

    /**
     * The path to the file(s).
     */
    private String path;

    /** The paths of all contained Files. */
    private List<File> paths;

    /** The iterator over all contained Files. */
    private Iterator<File> pathIt;

    /**
     * All known configuration handlers.
     */
    private ConfigurationHandlers configurationHandlers;

    /**
     * Constructor.
     *
     * @param type the source type, or null if the source type should
     *        be determined from the file extension.
     * @param path the path to the file(s) to read, not null.
     * @param elements the path to the source elements to read, or null
     *        if the root element should be used.
     * @param sourceFilter the fully qualified class name of the source filter,
     *        or null. The referenced class must have a standard constructor.
     * @param configurationHandlers All known configuration handlers, not null.
     * @param sourcesDir The directory where the sources should be read from,
     *        not null.
     *
     * @throws IllegalArgumentException if the source type is unknown.
     * @throws NullPointerException if path is null.
     * @throws ConfigurationException if the source filter cannot be
     *         instantiated.
     */
    public FileSourcesImpl(
                    SourceType type,
                    String path,
                    String elements,
                    String sourceFilter,
                    List<TransformerDefinition> transformerDefinitions,
                    ConfigurationHandlers configurationHandlers,
                    File sourcesDir)
            throws ConfigurationException
    {
        super(elements, sourceFilter, transformerDefinitions);
        if (path == null)
        {
            throw new NullPointerException("path must not be null");
        }
        if (configurationHandlers == null)
        {
            throw new NullPointerException(
                    "configurationHandlers must not be null");
        }
        if (sourcesDir == null)
        {
            throw new NullPointerException("sourcesDir must not be null");
        }
        this.type = type;
        this.path = path;
        this.configurationHandlers = configurationHandlers;

        int baseDirSeparatorPos = getBaseDirSeparatorPos(path);
        File baseDir = new File(
                sourcesDir,
                getBaseDir(path, baseDirSeparatorPos));
        String wildcardPattern = getWildcardPattern(path, baseDirSeparatorPos);
        String[] wildcardParts = StringUtils.split(wildcardPattern, "\\/");
        List<String> wildcardList = Arrays.asList(wildcardParts);

        paths = getFiles(baseDir, wildcardList);
        pathIt = paths.iterator();
    }

    List<File> getFiles(File baseDir, List<String> pathParts)
            throws ConfigurationException
    {
        List<String> pathPartsCopy = new ArrayList<String>(pathParts);
        String partToProcess = pathPartsCopy.remove(0);
        if (pathPartsCopy.size() == 0)
        {
            File[] result = baseDir.listFiles(
                    new WildcardFilter(partToProcess, false, true));
            if (result == null)
            {
                throw new ConfigurationException(
                        "Could not list files in the following Directory "
                            + "while reading the sources: "
                            + baseDir.getAbsolutePath());
            }
            return Arrays.asList(result);
        }
        if ("..".equals(partToProcess))
        {
            return getFiles(baseDir.getParentFile(), pathPartsCopy);
        }
        File[] matchingDirs = baseDir.listFiles(
                new WildcardFilter(partToProcess, true, false));
        List<File> result = new ArrayList<File>();
        for (File dir : matchingDirs)
        {
            result.addAll(getFiles(dir, pathPartsCopy));
        }
        return result;
    }

    /**
     * Returns the position of the separator which separates the base part
     * of the path which does not contain any wildcards from the rest.
     * Example:
     * <ul>
     *   <li>*.txt returns -1(no separator)</li>
     *   <li>schema*.xml returns -1(no separator)</li>
     *   <li>xml\schema*.xml returns 3 (backslash position)</li>
     *   <li>/xml/???/schema*.xml returns 4 (middle slash position)</li>
     * </ul>
     * @param path the path to compute the position from, not null.
     * @return the separator position, -1 if no base part exists.
     */
    static int getBaseDirSeparatorPos(String path)
    {
        int asteriskIndex = path.indexOf("*");
        int questionMarkIndex = path.indexOf("?");
        if (asteriskIndex != -1)
        {
            if (questionMarkIndex != -1)
            {
                int min = Math.min(asteriskIndex, questionMarkIndex);
                return getLargestSeparatorPos(path, min);
            }
            else
            {
                return getLargestSeparatorPos(path, asteriskIndex);
            }
        }
        return getLargestSeparatorPos(path, questionMarkIndex);
    }

    /**
     * Returns the largest position of a path separator within the path
     * which is smaller than endIndex. An endIndex of -1 means that
     * the largest separator pos should be given. -1 is returned if no
     * separator is present in the region.
     * <ul>
     *   <li>getBaseDir("/xml/xxxx", 5) returns 4</li>
     *   <li>getBaseDir("/xml/x/y", -1) returns 6</li>
     *   <li>getBaseDir("/xml/x/y/", -1) returns 8</li>
     * </ul>
     */
    private static int getLargestSeparatorPos(String path, int maxIndex)
    {
        String baseString;
        if (maxIndex == -1)
        {
            baseString = path;
        }
        else
        {
            baseString = path.substring(0, maxIndex);
        }
        return FilenameUtils.indexOfLastSeparator(baseString);
    }

    static String getBaseDir(String path, int baseDirSeparatorPos)
    {
        if (baseDirSeparatorPos == -1)
        {
            return ".";
        }
        else
        {
            String resultString = path.substring(0, baseDirSeparatorPos);
            if (StringUtils.EMPTY.equals(resultString))
            {
                resultString = ".";
            }
            return resultString;
        }

    }

    static String getWildcardPattern(String path, int baseDirSeparatorPos)
    {
        String resultString = path.substring(baseDirSeparatorPos + 1);
        return resultString;
    }

    public boolean hasNext()
    {
        return pathIt.hasNext();
    }

    public Source next()
    {
        File currentPath = pathIt.next();
        SourceType currentType = type;
        if (currentType == null)
        {
            Set<SourceType> sourceTypes
                    = configurationHandlers.getSourceTypes();
            for (SourceType sourceType : sourceTypes)
            {
                String filenameExtension = sourceType.getFilenameExtension();
                if (filenameExtension != null
                        && currentPath.getName().endsWith(
                                "." + filenameExtension))
                {
                    currentType = sourceType;
                    break;
                }
            }
        }
        if (currentType == null)
        {
            throw new RuntimeException(
                    "Type is not set and file extension is unknown for file "
                        + currentPath.getName());
        }
        try
        {
            return new FileSource(
                    currentType,
                    currentPath,
                    getStartElementsPath(),
                    getSourceFilter(),
                    getTransformerDefinitions());
        }
        catch (ConfigurationException e)
        {
            throw new RuntimeException(e);
        }
    }

    public void remove()
    {
        throw new UnsupportedOperationException();
    }

    @Override
    public String toString()
    {
        StringBuffer result = new StringBuffer();
        result.append("(path=").append(path)
                .append(",type=").append(type)
                .append(",sourceFilter=").append(getSourceFilter())
                .append(",startElementsPath=").append(getStartElementsPath())
                .append(",transformerDefinitions=")
                .append(getTransformerDefinitions())
                .append(")");
        return result.toString();
    }

    public SourceType getType()
    {
        return type;
    }

    public String getPath()
    {
        return path;
    }

    public List<File> getPaths()
    {
        return Collections.unmodifiableList(paths);
    }

    /**
     * A filter evaluating a file name against a wildcard expression.
     */
    private static class WildcardFilter implements FileFilter
    {
        /**
         * The wildcard expression against which the file names are
         * checked.
         */
        private String expression;

        /**
         * Whether directories are accepted at all.
         */
        private boolean acceptDir;

        /**
         * Whether files are accepted at all.
         */
        private boolean acceptFile;

        /**
         * Constructor.
         *
         * @param expression The wildcard expression against which
         *        the file names are checked.
         * @param acceptDir Whether directories are accepted at all.
         * @param acceptFile Whether files are accepted at all.
         */
        public WildcardFilter(
                String expression,
                boolean acceptDir,
                boolean acceptFile)
        {
            this.expression = expression;
            this.acceptDir = acceptDir;
            this.acceptFile = acceptFile;
        }

        /**
         * Returns whether a file matches the criteria of this filter.
         * If the file is a directory and <code>acceptDir</code> is false,
         * the file is rejected.
         * If the file is regular file and <code>acceptFile</code> is false,
         * the file is rejected.
         * If the filename does not match the wildcard filter, the file is
         * rejected.
         * If none of the above applies, the file is accepted.
         *
         * @return false if the file is rejected, true if it is accepted.
         */
        public boolean accept(File file)
        {
            if (!acceptDir && file.isDirectory())
            {
                return false;
            }
            if (!acceptFile && file.isFile())
            {
                return false;
            }
            if (FilenameUtils.wildcardMatch(
                    file.getName(),
                    expression,
                    IOCase.SENSITIVE))
            {
                return true;
            }
            return false;
        }
    }
}
