package org.apache.torque.gf.source;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.util.List;

/**
 * A source element which has useful shortcut methods for usage in templates.
 *
 */
public interface RichSourceElement extends SourceElement
{
    /**
     * Returns whether children with the given name exist.
     * @param name the name of the child element, not null.
     * @return true if children with the given name exist, false otherwise.
     * @throws NullPointerException if name is null.
     */
    boolean hasChild(String name);

    /**
     * Returns whether a following element exists as a child of the parent of
     * this element.
     * @return true if a following element exists, false if not.
     */
    boolean hasFollowing();

    /**
     * Returns whether an preceding exists as a child of the parent of
     * this element.
     * @return true if a preceding element exists, false if not.
     */
    boolean hasPreceding();

    /**
     * Returns whether a following element exists as a child of the parent of
     * this element, which has the same name as this source element.
     * @return true if a following sibling exists, false if not.
     */
    boolean hasFollowingSibling();

    /**
     * Returns whether an preceding exists as a child of the parent of
     * this element, which has the same name as this source element.
     * @return true if a preceding sibling exists, false if not.
     */
    boolean hasPrecedingSibling();

    /**
     * Returns all the preceding elements before this element
     * with the given name.
     * If name is null, all preceding elements are returned.
     * If this element has no parent, an empty list is returned.
     * @param name the name of the preceding elements to select,
     *        or null to select all preceding elements.
     * @return a list containing the following elements with the given name,
     *         never null.
     *
     * @see <a href="http://www.w3.org/TR/xpath#axes"></a>
     */
    List<SourceElement> getPreceding(String name);

    /**
     * Returns all the following elements after this element
     * with the given name.
     * If name is null, all following elements are returned.
     * If this element has no parent, an empty list is returned.
     * @param name the name of the following elements to select,
     *        or null to select all following elements.
     * @return a list containing the following elements with the given name,
     *         never null.
     *
     * @see <a href="http://www.w3.org/TR/xpath#axes"></a>
     */
    List<SourceElement> getFollowing(String name);
}
