package org.apache.torque.gf.source;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Implementation of the RichSourceElement interface.
 */
public class RichSourceElementImpl
    implements RichSourceElement, SourceElement
{
    /**
     * The name of the source element.
     */
    private String name;

    /**
     * The parents of this  element; may be empty but not null.
     */
    private List<SourceElement> parents = new ArrayList<SourceElement>();

    /**
     * All children elements.
     */
    private List<SourceElement> children = new ArrayList<SourceElement>();

    /**
     * the source element's attributes.
     */
    private Map<String, Object> attributes = new HashMap<String, Object>();

    /**
     * Constructor.
     *
     * @param name the name of the element, not null.
     *
     * @throws NullPointerException if name is null.
     */
    public RichSourceElementImpl(String name)
    {
        if (name == null)
        {
            throw new NullPointerException("name must not be null");
        }
        this.name = name;
    }

    /**
     * Constructor.
     *
     * @param name sourceElementName name of the element, not null.
     *
     * @throws NullPointerException if sourceElementName is null.
     */
    public RichSourceElementImpl(SourceElementName sourceElementName)
    {
        this(sourceElementName.getName());
    }

    public String getName()
    {
        return name;
    }

    public SourceElement getParent()
    {
        if (parents.size() == 0)
        {
            return null;
        }
        return parents.get(0);
    }

    public void addParent(SourceElement parent)
    {
        if (parent == null)
        {
            throw new NullPointerException("parent must not be null");
        }
        parents.add(parent);
        // We can use contains here as we did not overwrite the equals()
        // and hassCode() methods.
        if (!parent.getChildren().contains(this))
        {
            parent.addChild(this);
        }
    }

    public List<SourceElement> getParents()
    {
        return parents;
    }

    public List<SourceElement> getChildren()
    {
        return children;
    }

    public void addChild(SourceElement child)
    {
        if (child == null)
        {
            throw new NullPointerException("child must not be null");
        }
        children.add(child);

        // We can use contains here as we did not overwrite the equals()
        // and hassCode() methods.
        if (!child.getParents().contains(this))
        {
            child.addParent(this);
        }
    }

    public void addChild(int position, SourceElement child)
    {
        if (child == null)
        {
            throw new NullPointerException("child must not be null");
        }
        children.add(position, child);

        // We can use contains here as we did not overwrite the equals()
        // and hashCode() methods.
        if (!child.getParents().contains(this))
        {
            child.addParent(this);
        }
    }

    public SourceElement removeChild(SourceElement toRemove)
    {
        if (toRemove == null)
        {
            throw new NullPointerException("toRemove must not be null");
        }
        Iterator<SourceElement> childIt = children.iterator();
        while (childIt.hasNext())
        {
            SourceElement candidate = childIt.next();
            if (candidate == toRemove)
            {
                childIt.remove();
                return candidate;
            }
        }
        return null;
    }

    public List<SourceElement> getChildren(String name)
    {
        if (name == null)
        {
            throw new NullPointerException("name must not be null");
        }
        List<SourceElement> result = new ArrayList<SourceElement>();
        for (SourceElement sourceElement : children)
        {
            if (name.equals(sourceElement.getName()))
            {
                result.add(sourceElement);
            }
        }
        return result;
    }

    public List<SourceElement> getChildren(SourceElementName sourceElementName)
    {
        return getChildren(sourceElementName.getName());
    }

    public SourceElement getChild(String name)
    {
        for (SourceElement sourceElement : children)
        {
            if (name.equals(sourceElement.getName()))
            {
                return sourceElement;
            }
        }
        return null;
    }

    public SourceElement getChild(SourceElementName sourceElementName)
    {
        return getChild(sourceElementName.getName());
    }

    public boolean hasChild(String name)
    {
        return SourcePath.hasChild(this, name);
    }

    public int getChildIndex(SourceElement child)
    {
        return children.indexOf(child);
    }


    /**
     * Returns all the following elements after this element
     * with the given name.
     * If name is null, all following elements are returned.
     * If this element has no parent, an empty list is returned.
     * @param name the name of the following elements to select,
     *        or null to select all following elements.
     * @return a list containing the following elements with the given name,
     *         never null.
     *
     * @see <a href="http://www.w3.org/TR/xpath#axes"></a>
     */
    public List<SourceElement> getFollowing(String name)
    {
        return SourcePath.getFollowing(this, name);
    }

    public boolean hasFollowing()
    {
        return SourcePath.hasFollowing(this);
    }

    public boolean hasPreceding()
    {
        return SourcePath.hasPreceding(this);
    }

    public boolean hasFollowingSibling()
    {
        return SourcePath.hasFollowingSibling(this);
    }

    public boolean hasPrecedingSibling()
    {
        return SourcePath.hasPrecedingSibling(this);
    }

    /**
     * Returns all the preceding elements before this element
     * with the given name.
     * If name is null, all preceding elements are returned.
     * If this element has no parent, an empty list is returned.
     * @param name the name of the preceding elements to select,
     *        or null to select all preceding elements.
     * @return a list containing the following elements with the given name,
     *         never null.
     *
     * @see <a href="http://www.w3.org/TR/xpath#axes"></a>
     */
    public List<SourceElement> getPreceding(String name)
    {
        return SourcePath.getPreceding(this, name);
    }

    public Object getAttribute(String name)
    {
        return attributes.get(name);
    }

    public Object getAttribute(SourceAttributeName sourceAttributeName)
    {
        return getAttribute(sourceAttributeName.getName());
    }

    public Object setAttribute(String name, Object value)
    {
        if (value == null)
        {
            return attributes.remove(name);
        }
        return attributes.put(name, value);
    }

    public Object setAttribute(
            SourceAttributeName sourceAttributeName,
            Object value)
    {
        return setAttribute(sourceAttributeName.getName(), value);
    }

    public Set<String> getAttributeNames()
    {
        return attributes.keySet();
    }

    /**
     * Creates a deep copy of this RichSourceelementImpl object.
     *
     * @return the copy, not null.
     */
    public RichSourceElementImpl copy()
    {
        Map<SourceElement, RichSourceElementImpl> copied
                = new HashMap<SourceElement, RichSourceElementImpl>();
        return copy(this, copied);
    }

    /**
     * Deep copies the content of one RichSourceElementImpl object into another.
     *
     * @param toCopy the source element, not null.
     * @param copiedElements Map containing all source elements which are
     *        already copied.
     *
     * @return the copy of the source, not null.
     */
    private RichSourceElementImpl copy(SourceElement toCopy,
            Map<SourceElement, RichSourceElementImpl> copiedElements)
    {
        RichSourceElementImpl copied = copiedElements.get(toCopy);
        if (copied != null)
        {
            return copied;
        }
        copied = new RichSourceElementImpl(toCopy.getName());
        copiedElements.put(toCopy, copied);

        for (String attributeName : toCopy.getAttributeNames())
        {
            copied.setAttribute(attributeName, toCopy.getAttribute(attributeName));
        }

        for (SourceElement child : toCopy.getChildren())
        {
            RichSourceElementImpl copiedChild = copy(child, copiedElements);
            copied.addChild(copiedChild);
        }

        for (SourceElement parent : toCopy.getParents())
        {
            RichSourceElementImpl copiedParent
                    = copy(parent, copiedElements);
            // do not use setParent because this will add copied as a child
            // to copiedParent. Later on, when processing the children of
            // copiedParent, it will be added again.
            copied.parents.add(copiedParent);
        }

        return copied;
    }

    /**
     * Checks whether the source element graph of this sourceElement,
     * and its position therein, equals the source element graph
     * and the position of the provided SourceElement.
     * This is an expensive operation if the graphs are large.
     *
     * @param toCompare the source element to compare, may be null.
     *
     * @return true if all source elements in the toCompare tree have the equal
     *          content as the source elements in this tree.
     */
    public boolean graphEquals(SourceElement toCompare)
    {
        Set<SourceElement> alreadyCompared = new HashSet<SourceElement>();
        return graphEquals(this, toCompare, alreadyCompared);
    }

    /**
     * Checks whether the source element graph of one sourceElement,
     * and its position therein, equals the source element graph
     * and the position of another SourceElement.
     * This is an expensive operation if the graphs are large.
     *
     * @param reference the reference element, may be null.
     * @param toCompare the element which is to the referenced element,
     *        may be null.
     * @param compared a set of elements which are already compared
     *        and which attributes and relative positions to the other
     *        compared elements were equal so far.
     * @return true if the elements are equal or if equality is currently
     *          checked in another recursive iteration.
     */
    private boolean graphEquals(SourceElement reference,
            SourceElement toCompare,
            Set<SourceElement> compared)
    {
        if ((reference == null && toCompare != null)
                || (reference != null && toCompare == null))
        {
            return false;
        }
        if (reference == null && toCompare == null)
        {
            return true;
        }

        if (compared.contains(reference))
        {
            // although it is not certain that reference is equal to toCompare
            // if it is contained in compared, it does mean that equality
            // was or is being checked and that place will return false
            // if equality is not given; so we can return true here.
            return true;
        }

        compared.add(reference);

        if (!reference.getName().equals(toCompare.getName()))
        {
            return false;
        }

        if (reference.getAttributeNames().size()
                != toCompare.getAttributeNames().size())
        {
            return false;
        }

        for (String attributeName : reference.getAttributeNames())
        {
            Object referenceAttributeContent
                    = reference.getAttribute(attributeName);
            Object toCompareAttributeContent
                    = toCompare.getAttribute(attributeName);
            if (referenceAttributeContent == null)
            {
                if (toCompareAttributeContent != null)
                {
                    return false;
                }
            }
            else
            {
                if (!referenceAttributeContent.equals(
                        toCompareAttributeContent))
                {
                    return false;
                }
            }
        }

        if (!graphEquals(reference.getParent(), toCompare.getParent(), compared))
        {
            return false;
        }

        if (reference.getChildren().size()
                != toCompare.getChildren().size())
        {
            return false;
        }

        Iterator<SourceElement> referenceChildIt
                = reference.getChildren().iterator();
        Iterator<SourceElement> toCompareChildIt
                = toCompare.getChildren().iterator();
        while (referenceChildIt.hasNext())
        {
            SourceElement referenceChild = referenceChildIt.next();
            SourceElement toCompareChild = toCompareChildIt.next();
            if (!graphEquals(referenceChild, toCompareChild, compared))
            {
                return false;
            }
        }
        return true;
    }

    @Override
    public String toString()
    {
        Set<SourceElement> alreadyProcessed = new HashSet<SourceElement>();
        StringBuilder result = new StringBuilder();
        toString(alreadyProcessed, result);
        return result.toString();
    }

    /**
     * Creates a String representation of the element for debugging purposes.
     * @param alreadyProcessed the elements which are already processed
     *        (for avoiding loops). The current element is added to this set.
     * @param result the String builder to which the string representation
     *        should be appended.
     */
    private void toString(
            Set<SourceElement> alreadyProcessed,
            StringBuilder result)
    {
        alreadyProcessed.add(this);
        result.append("(name=").append(name)
                .append(",attributes=(");
        Iterator<Map.Entry<String, Object>> entryIt
                = attributes.entrySet().iterator();
        while (entryIt.hasNext())
        {
            Map.Entry<String, Object> entry = entryIt.next();
            result.append(entry.getKey()).append("=").append(entry.getValue());
            if (entryIt.hasNext())
            {
                result.append(",");
            }
        }
        result.append("),children=(");
        Iterator<SourceElement> childIt = children.iterator();
        while (childIt.hasNext())
        {
            SourceElement child = childIt.next();
            if (alreadyProcessed.contains(child))
            {
                result.append("<<loop detected>>");
            }
            else
            {
                if (child instanceof RichSourceElementImpl)
                {

                    ((RichSourceElementImpl) child).toString(
                            alreadyProcessed, result);
                }
                else
                {
                    result.append(child);
                }
            }
            if (childIt.hasNext())
            {
                result.append(",");
            }
        }
        result.append("))");
    }
}
