package org.apache.torque.gf.source;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.torque.gf.configuration.ClassHelper;
import org.apache.torque.gf.configuration.ConfigurationException;
import org.apache.torque.gf.source.filter.SourceFilter;

/**
 * A collection of Sources, which all have the same elements,
 * Source filters and transformerDefinitions.
 */
public abstract class SimpleSourcesImpl implements Sources
{
    /**
     * The elements which should be used as starting points for generation.
     */
    private String startElementsPath;

    /**
     * The source transformers which are executed when transform() is called.
     */
    private List<TransformerDefinition> transformerDefinitions
            = new ArrayList<TransformerDefinition>();

    /**
     * The source filter, or null if none exists.
     */
    private SourceFilter sourceFilter;

    /**
     * Constructor.
     *
     * @param startElementsPath the path to the elements which are used
     *        as starting points for generation,
     *        or null if the root element should be used.
     * @param sourceFilter the fully qualified class name of the source filter,
     *        or null. The referenced class must have a standard constructor.
     *
     * @throws ConfigurationException if the source filter cannot be
     *         instantiated.
     */
    protected SimpleSourcesImpl(
                    String startElementsPath,
                    String sourceFilter,
                    List<TransformerDefinition> transformerDefinitions)
            throws ConfigurationException
    {
        this.startElementsPath = startElementsPath;
        if (transformerDefinitions == null)
        {
            this.transformerDefinitions
                    = new ArrayList<TransformerDefinition>();
        }
        else
        {
            this.transformerDefinitions
                    = new ArrayList<TransformerDefinition>(
                            transformerDefinitions);
        }

        if (sourceFilter != null)
        {
            this.sourceFilter = (SourceFilter) ClassHelper.getInstance(
                    sourceFilter,
                    SourceFilter.class);
        }
    }

    /**
     * Returns the path to elements which should be used as starting points for
     * generation.
     *
     * @return the elements to use, or null, in which case the root element
     *         is used.
     */
    public String getStartElementsPath()
    {
        return startElementsPath;
    }

    /**
     * Return all currently registered transformer definitions.
     *
     * @return the transformer definitions, not null.
     */
    public List<TransformerDefinition> getTransformerDefinitions()
    {
        return Collections.unmodifiableList(transformerDefinitions);
    }

    /**
     * Returns the current source filter.
     *
     * @return the current source filter, or null.
     */
    public SourceFilter getSourceFilter()
    {
        return sourceFilter;
    }
}
