package org.apache.torque.gf.source;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.File;
import java.util.List;

import org.apache.torque.gf.source.filter.SourceFilter;

/**
 * An entity which serves as an input for the generation process.
 */
public interface Source
{
    /**
     * Returns the root element of the source.
     *
     * @return the root element of the source, not null.
     *
     * @throws SourceException if the source cannot be constructed.
     */
    SourceElement getRootElement() throws SourceException;

    /**
     * Returns the path to the elements which are selected as start elements
     * from the transformed source.
     *
     * @return a path to the start elements, null for the root element.
     */
    String getStartElementsPath();

    /**
     * Returns the definitions for the source transformers which are
     * executed before the generation process
     *
     * @return the source transformer definitions, not null.
     *         The first transformer in the list is executed first.
     */
    List<TransformerDefinition> getTransformerDefinitions()
            throws SourceException;

    /**
     * The configured source filter, or null if there is no source filter
     * for this source.
     *
     * The source filter must be executed after the file name is generated
     * and the current source element and the options are set to the controller
     * state.
     *
     * If its proceed(ControllerState) method returns false, generation of
     * the current output file must be skipped. If no SourceFilter exists,
     * generation proceeds.
     *
     * @return the source filter, or null.
     */
    SourceFilter getSourceFilter();

    /**
     * Gets a description of this source for debugging purposes.
     *
     * @return the description, which should make it possible to identify
     *         the currently processed output.
     */
    String getDescription();

    /**
     * Returns the source file, if it exists.
     *
     * @return the source file, or null if the source is not read from a file.
     * @return
     */
    File getSourceFile();
}
