package org.apache.torque.gf.source;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.util.List;
import java.util.Set;

/**
 * A Source element within the Source Graph.
 */
public interface SourceElement
{
    /**
     * Returns the name of this source element.
     *
     * @return the name of this source element, never null.
     */

    String getName();

    /**
     * Returns the primary parent of this SourceElement.
     *
     * @return the primary parent of this SourceElement,
     *         or null if this is a root element of the source graph.
     */
    SourceElement getParent();

    /**
     * Returns the list of parents of this SourceElement.
     *
     * @return the list of parents of this source element, never null.
     */
    List<SourceElement> getParents();

    /**
     * Adds a parent to the parents of this SourceElement.
     * This automatically adds this element as a child of the passed
     * SourceElement, if it is not yet a child.
     *
     * @param parent the source element which should be added as a parent,
     *        not null.
     */
    void addParent(SourceElement parent);

    /**
     * Returns the list of children of this SourceElement.
     *
     * @return the list of children of this source element, never null.
     */
    List<SourceElement> getChildren();

    /**
     * Returns the list of children of this SourceElement which have
     * the given name.
     *
     * @param name the name of the children to select, not null.
     *
     * @return the list of children of this source element with the given name,
     *         never null.
     *
     * @throws NullPointerException if name is null.
     */
    List<SourceElement> getChildren(String name);

    /**
     * Returns the list of children of this SourceElement which have
     * the given name.
     *
     * @param sourceElementName contains the name of the child to select,
     *        not null.
     *
     * @return the list of children of this source element with the given name,
     *         never null.
     *
     * @throws NullPointerException if sourceElementName is null.
     */
    List<SourceElement> getChildren(SourceElementName sourceElementName);

    /**
     * Returns the first child of this SourceElement which has
     * the given name.
     *
     * @param name the name of the child to select, not null.
     *
     * @return the first child with the given name, or null if no child with
     *         the given name exits.
     *
     * @throws NullPointerException if name is null.
     */
    SourceElement getChild(String name);

    /**
     * Returns the first child of this SourceElement which has
     * the given name.
     *
     * @param sourceElementName contains the name of the child to select,
     *        not null.
     *
     * @return the first child with the given name, or null if no child with
     *         the given name exits.
     *
     * @throws NullPointerException if sourceElementName is null.
     */
    SourceElement getChild(SourceElementName sourceElementName);

    /**
     * Adds a Child element to this SourceElement. This element is added to the
     * list of parents of the Source element.
     *
     * @param child the child to be added, not null.
     *
     * @throws NullPointerException if child is null.
     */
    void addChild(SourceElement child);

    void addChild(int position, SourceElement child);

    SourceElement removeChild(SourceElement child);

    int getChildIndex(SourceElement child);

    /**
     * Returns the object stored in a given attribute.
     *
     * @param name the name of the attribute, can be null.
     *
     * @return the stored object, or null if no object is stored under that key.
     */
    Object getAttribute(String name);

    /**
     * Returns the object stored in a given attribute.
     *
     * @param sourceAttributeName contains the name of the attribute, not null.
     *
     * @return the stored object, or null if no object is stored under that key.
     *
     * @throws NullPointerException if sourceAttributeName is null.
     */
    Object getAttribute(SourceAttributeName sourceAttributeName);

    /**
     * Returns the name of all set attributes. Note : null may be contained
     * in the set.
     *
     * @return the name of all set values.
     */
    Set<String> getAttributeNames();

    /**
     * Sets the attribute of a Source element.
     *
     * @param name the name of the attribute.
     * @param value the value of the attribute,
     *        or null to remove the attribute.
     *
     * @return the previous value of this attribute.
     */
    Object setAttribute(String name, Object value);

    /**
     * Sets the attribute of a Source element.
     *
     * @param sourceAttributeName contains the name of the attribute, not null.
     * @param value the value of the attribute,
     *        or null to remove the attribute.
     *
     * @return the previous value of this attribute.
     */
    Object setAttribute(SourceAttributeName sourceAttributeName, Object value);
}
