package org.apache.torque.gf.source;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.torque.gf.configuration.ConfigurationException;
import org.apache.torque.gf.source.filter.SourceFilter;

/**
 * A Source which uses a file as input.
 */
public abstract class SourceImpl implements Source
{
    /**
     * The path to the elements which should be used as starting points
     * for generation.
     */
    private String startElementsPath;

    /**
     * The root element of the parsed source.
     */
    private transient SourceElement rootElement;

    /**
     * The source transformers which are executed when transform() is called.
     */
    private List<TransformerDefinition> transformerDefinitions
            = new ArrayList<TransformerDefinition>();

    /**
     * The source filter, or null if none exists.
     */
    private SourceFilter sourceFilter;

    /**
     * Constructor.
     *
     * @param startElementsPath the element which should be used as input,
     *        or null if the root element should be used.
     * @param sourceFilter the fully qualified class name of the source filter.
     *        The class must have a standard constructor.
     *
     * @throws ConfigurationException if the source filter cannot be
     *         instantiated.
     */
    protected SourceImpl(
                    String startElementsPath,
                    SourceFilter sourceFilter,
                    List<TransformerDefinition> transformerDefinitions)
            throws ConfigurationException
    {
        this.startElementsPath = startElementsPath;
        this.sourceFilter = sourceFilter;
        this.transformerDefinitions = new ArrayList<TransformerDefinition>(
                transformerDefinitions);

    }

    /**
     * Reads the root element and the whole untransformed source tree.
     *
     * @return the root element, not null.
     *
     * @throws SourceException if the SourceElement cannot be created.
     */
    protected abstract SourceElement createRootElement() throws SourceException;

    public synchronized SourceElement getRootElement()
        throws SourceException
    {
        if (rootElement == null)
        {
            rootElement = createRootElement();
        }
        return rootElement;
    }

    /**
     * Returns the path to the elements which are selected as start elements
     * from the transformed source.
     *
     * @return a path to the start elements, null for the root element.
     */
    public String getStartElementsPath()
    {
        return startElementsPath;
    }

    /**
     * Return all currently registered transformer definitions.
     *
     * @return the transformer definitions, not null.
     */
    public List<TransformerDefinition> getTransformerDefinitions()
    {
        return Collections.unmodifiableList(transformerDefinitions);
    }

    /**
     * Returns the current source filter.
     *
     * @return the current source filter, or null.
     */
    public SourceFilter getSourceFilter()
    {
        return sourceFilter;
    }

    /**
     * Sets a new source filter.
     *
     * @param sourceFilter the new source filter, or null to remove the old
     *        source filter.
     */
    public void setSourceFilter(SourceFilter sourceFilter)
    {
        this.sourceFilter = sourceFilter;
    }

    @Override
    public int hashCode()
    {
        final int prime = 31;
        int result = 1;
        result = prime * result
            + ((startElementsPath == null) ? 0 : startElementsPath.hashCode());
        result = prime * result + transformerDefinitions.hashCode();
        result = prime * result
            + ((sourceFilter == null) ? 0 : sourceFilter.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj)
    {
        if (this == obj)
        {
            return true;
        }
        if (obj == null)
        {
            return false;
        }
        if (getClass() != obj.getClass())
        {
            return false;
        }

        final SourceImpl other = (SourceImpl) obj;
        if (startElementsPath == null)
        {
            if (other.startElementsPath != null)
            {
                return false;
            }
        }
        else if (!startElementsPath.equals(other.startElementsPath))
        {
            return false;
        }
        if (!transformerDefinitions.equals(other.transformerDefinitions))
        {
            return false;
        }
        if (sourceFilter == null)
        {
            if (other.sourceFilter != null)
            {
                return false;
            }
        }
        else if (!sourceFilter.equals(other.sourceFilter))
        {
            return false;
        }
        return true;
    }
}
