package org.apache.torque.gf.source.properties;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.IOException;
import java.io.InputStream;
import java.util.Collections;
import java.util.Properties;
import java.util.Set;

import org.apache.commons.collections.set.ListOrderedSet;
import org.apache.torque.gf.source.RichSourceElementImpl;
import org.apache.torque.gf.source.SourceElement;
import org.apache.torque.gf.source.SourceException;
import org.apache.torque.gf.source.SourceParser;

/**
 * Parses a stream containing properties data into a source element hierarchy.
 * The source element hierarchy is processed according to the java properties
 * DTD.
 * See http://java.sun.com/dtd/properties.dtd
 */
public class PropertiesSourceParser implements SourceParser
{
    /**
     * The name of the root element.
     */
    public static final String ROOT_ELEMENT_NAME = "properties";

    /**
     * The name of the entry element.
     */
    public static final String ENTRY_ELEMENT_NAME = "entry";

    /**
     * The name of the key attribute of the entry element.
     */
    public static final String KEY_ATTRIBUTE_NAME = "key";

    /**
     * parses a Stream in Properties format and returns the root element of the
     * created element tree.
     *
     * @param inputStream the input stream to parse, not null.
     *
     * @return the root element of the created tree, not null.
     *
     * @throws SourceException if an error occurred when reading from the
     *         input stream.
     * @throws IllegalArgumentException if the input stream contains a
     *         malformed unicode escape sequence.
     * @throws NullPointerException if inputStream is null.
     */
    public SourceElement parse(InputStream inputStream)
            throws SourceException
    {
        if (inputStream == null)
        {
            throw new NullPointerException("inputStream is null");
        }
        OrderedProperties properties = new OrderedProperties();
        try
        {
            properties.load(inputStream);
        }
        catch (IOException e)
        {
            throw new SourceException(
                    "Error parsing Properties source file: " + e.getMessage(),
                    e);
        }
        catch (IllegalArgumentException e)
        {
            throw new SourceException(
                    "Error parsing Properties source file: " + e.getMessage(),
                    e);
        }

        RichSourceElementImpl result
                = new RichSourceElementImpl(ROOT_ELEMENT_NAME);

        for (String key : properties.orderedKeySet())
        {
            String value = properties.getProperty(key);
            RichSourceElementImpl entryElement
                    = new RichSourceElementImpl(ENTRY_ELEMENT_NAME);
            entryElement.setAttribute(KEY_ATTRIBUTE_NAME, key);
            entryElement.setAttribute((String) null, value);
            result.addChild(entryElement);
        }
        return result;
    }

    /**
     * Properties which remember the order of the added elements.
     *
     * The class is NOT serializable although it implements the
     * <code>Serializable</code> interface.
     */
    private static class OrderedProperties extends Properties
    {
        /**
         * version for serialization and deserialisation purposes.
         */
        private static final long serialVersionUID = 1L;

        /**
         * The ordered set of keys.
         */
        @SuppressWarnings("unchecked")
        private Set<String> keySet = new ListOrderedSet();

        @Override
        public Object put(Object key, Object value)
        {
            keySet.add(key.toString());
            return super.put(key, value);
        }

        /**
         * Returns the set of keys, ordered by the order of addition.
         * @return the ordered set of keys, not null.
         *         The returned set is unmodifiable.
         */
        public Set<String> orderedKeySet()
        {
            return Collections.unmodifiableSet(keySet);
        }
    }
}
