package org.apache.torque.gf.source.xml;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.IOException;
import java.io.InputStream;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.torque.gf.source.SourceElement;
import org.apache.torque.gf.source.SourceException;
import org.apache.torque.gf.source.SourceParser;
import org.xml.sax.SAXException;

/**
 * Parses a stream containing xml data and creates a source element hierarchy
 * from it.
 */
public class XmlSourceParser implements SourceParser
{
    /**
     * The factory from which the parser is created.
     */
    private static final SAXParserFactory SAX_FACTORY;

    static
    {
        SAX_FACTORY = SAXParserFactory.newInstance();
        SAX_FACTORY.setValidating(true);
    }

    /**
     * Parses a stream containing xml data and creates a source element
     * hierarchy from it.
     *
     * @param inputStream the stream containing the xml data.
     *
     * @return the root element of the created hierarchy.
     *
     * @throws SourceException if an error occurs reading the input stream,
     *         parsing the XML data or if the SAX parser is not configured
     *         correctly.
     */
    public SourceElement parse(InputStream inputStream)
            throws SourceException
    {
        if (inputStream == null)
        {
            throw new NullPointerException("No Input path specified");
        }
        try
        {
            SAXParser parser = SAX_FACTORY.newSAXParser();

            XmlSourceSaxHandler handler = new XmlSourceSaxHandler();

            parser.parse(inputStream, handler);
            return handler.getRoot();
        }
        catch (IOException e)
        {
            throw new SourceException(
                    "Error reading XML source file: " + e.getMessage(),
                    e);
        }
        catch (SAXException e)
        {
            throw new SourceException(
                    "Error parsing XML source file: " + e.getMessage(),
                    e);
        }
        catch (ParserConfigurationException e)
        {
            throw new SourceException(
                    "Parser configuration error parsing Properties"
                        + " source file: "
                        + e.getMessage(),
                    e);
        }
    }
}
