package org.apache.torque.gf.template.velocity;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.io.StringWriter;
import java.io.Writer;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.torque.gf.configuration.ConfigurationException;
import org.apache.torque.gf.configuration.ConfigurationProvider;
import org.apache.torque.gf.control.ControllerState;
import org.apache.torque.gf.generator.GeneratorException;
import org.apache.torque.gf.option.Option;
import org.apache.torque.gf.option.Options;
import org.apache.torque.gf.qname.Namespace;
import org.apache.torque.gf.qname.QualifiedName;
import org.apache.torque.gf.qname.QualifiedNameMap;
import org.apache.torque.gf.source.SourceElement;
import org.apache.torque.gf.template.TemplateGeneratorImpl;
import org.apache.torque.gf.variable.Variable;
import org.apache.torque.gf.variable.VariableStore;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.context.Context;
import org.apache.velocity.util.StringUtils;

/**
 * A Generator which uses a velocity template for generation.
 */
public class VelocityGenerator extends TemplateGeneratorImpl
{
    /**
     * The name under which the Torque generator will be put
     * into the context.
     */
    public static final String TORQUE_GF_CONTEXT_NAME = "torqueGf";

    /**
     * The name under which the Torque generator will be put
     * into the context.
     */
    public static final String STRING_UTILS_CONTEXT_NAME = "stringUtils";

    /**
     * The key under which the null attribute of a source element is put
     * into the context.
     */
    public static final String NULL_KEY_CONTEXT_NAME = "value";

    /** The log. */
    private static Log log = LogFactory.getLog(VelocityGenerator.class);

    /**
     * Whether the options should be put into the context.
     */
    private boolean optionsInContext = true;

    /**
     * Whether the variables should be put into the context.
     */
    private boolean variablesInContext = true;

    /**
     * Whether the attributes of the current source element should be put
     * into the context.
     */
    private boolean sourceAttributesInContext = true;

    /**
     * Constructs a new VelocityTemplateGenerator.
     *
     * @param name the name of this generator, not null.
     * @param configurationProvider the provider for reading the templates,
     *        not null.
     * @param path the path to the templates, not null.
     *        May contain tokens of the form ${....}, these are parsed.
     * @param encoding the encoding of the file, or null if the system's
     *        default encoding should be used.
     *
     * @throws NullPointerException if name, path or directories are null.
     * @throws ConfigurationException if an error occurs while reading the
     *         template.
     */
    public VelocityGenerator(
            QualifiedName name,
            ConfigurationProvider configurationProvider,
            String path,
            String encoding)
        throws ConfigurationException
    {
        super(name,
              configurationProvider,
              path,
              encoding,
              new VelocityTemplateFilter());
    }

    /**
     * Tells the generator to put all options which name space is visible to
     * the name space of this generator into the context.
     * Only the variable names are used as keys in the context, the name spaces
     * are stripped.
     *
     * @param optionsInContext whether to put the options into the context.
     */
    public void setOptionsInContext(boolean optionsInContext)
    {
        this.optionsInContext = optionsInContext;
    }

    /**
     * Returns whether all options which name spaces are visible to
     * the name space of this generator are put into the context.
     *
     * @return whether the generator puts the options into the context.
     */
    public boolean isOptionsInContext()
    {
        return optionsInContext;
    }

    /**
     * Tells the generator to put all variables which are visible to this
     * generator into the context.
     * Only the variable names are used as keys in the context, the name spaces
     * are stripped.
     *
     * @param variablesInContext whether to put the variables into the context.
     */
    public void setVariablesInContext(boolean variablesInContext)
    {
        this.variablesInContext = variablesInContext;
    }

    /**
     * Returns whether all variables which are visible to this
     * generator are put into the context.
     *
     * @return whether the generator puts the variables into the context.
     */
    public boolean isVariablesInContext()
    {
        return variablesInContext;
    }

    /**
     * Tells the generator to put the attributes of the current source element
     * into the context or not.
     *
     * @param sourceAttributesInContext whether to put the source attributes
     *        into the context.
     */
    public void setSourceAttributesInContext(
            boolean sourceAttributesInContext)
    {
        this.sourceAttributesInContext = sourceAttributesInContext;
    }

    /**
     * Returns whether the attributes of the current source element
     * are put into the context.
     *
     * @return whether the generator puts the attributes of the current source
     *         element into the context.
     */
    public boolean isSourceAttributesInContext()
    {
        return sourceAttributesInContext;
    }

    /**
     * Executes the generation process; the result is returned.
     *
     * @param controllerState the current controller state.
     *
     * @return the result of the generation, not null.
     *
     * @see org.apache.torque.gf.generator.Generator#execute(ControllerState)
     */
    @Override
    public String execute(ControllerState controllerState)
        throws GeneratorException

    {
        if (log.isDebugEnabled())
        {
            log.debug("Start executing VelocityGenerator " + getName());
        }
        controllerState.getVariableStore().startGenerator();

        try
        {
            try
            {
                Velocity.init();
            }
            catch (Exception e)
            {
                throw new GeneratorException(
                        "Could not initialize velocity",
                        e);
            }

            SourceElement sourceElement = controllerState.getSourceElement();

            String inputElementName = getInputElementName();
            if (inputElementName != null
                    && !inputElementName.equals(sourceElement.getName()))
            {
                throw new GeneratorException("Input element name, "
                        + sourceElement.getName()
                        + ", is not the expected name, "
                        + getInputElementName()
                        + ", for generator "
                        + getName());
            }

            Context context = createVelocityContext(controllerState);

            Writer writer = new StringWriter();
            try
            {
                Velocity.evaluate(context, writer,
                        "VelocityTemplateGenerator:" + getName(),
                        getContent(controllerState));
                writer.flush();
                writer.close();
            }
            catch (Exception e)
            {
                log.error("error during execution of generator "
                            + getName()
                            + " : " + e.getMessage(),
                        e);
                throw new GeneratorException(
                        "Error during execution of generator "
                            + getName()
                            + " : "
                            + e.getMessage(),
                        e);
            }

            return writer.toString();
        }
        finally
        {
            controllerState.getVariableStore().endGenerator();
            if (log.isDebugEnabled())
            {
                log.debug("End executing VelocityGenerator " + getName());
            }
        }
    }

    /**
     * Creates the velocity context for the generator.
     *
     * @param controllerState the controller state, not null.
     * @return
     */
    private Context createVelocityContext(
            ControllerState controllerState)
    {
        Context context = new VelocityContext();
        context.put(
                TORQUE_GF_CONTEXT_NAME,
                new TorqueGfVelocity(this, controllerState));
        context.put(
                STRING_UTILS_CONTEXT_NAME,
                new StringUtils());

        if (optionsInContext)
        {
            // Only consider options visible from the current namespace.
            Options visibleOptions
                    = controllerState.getVisibleOptions();
            for (Option option : visibleOptions.values())
            {
                QualifiedName qualifiedName = option.getQualifiedName();
                context.put(qualifiedName.getName(), option.getValue());
            }
            log.debug("Put options in context " + visibleOptions.keySet());
        }
        else
        {
            log.debug("options in context are disabled");
        }

        SourceElement sourceElement = controllerState.getSourceElement();
        if (sourceAttributesInContext)
        {
            Set<String> attributes = sourceElement.getAttributeNames();
            for (String key : attributes)
            {
                Object value = sourceElement.getAttribute(key);
                if (key == null)
                {
                    // The null key cannot be accessed in the context.
                    // So if the attribute NULL_KEY_CONTEXT_NAME does not
                    // exist, use this as attribute name.
                    if (sourceElement.getAttributeNames().contains(
                            NULL_KEY_CONTEXT_NAME))
                    {
                        continue;
                    }
                    key = NULL_KEY_CONTEXT_NAME;
                }
                context.put(key, value);
            }
            log.debug("Put attributes in context " + attributes);
        }
        else
        {
            log.debug("source attributes in context are disabled");
        }

        if (variablesInContext)
        {
            // Only consider variables visible from the namespace
            // of this generator. If a name exists in different
            // namespaces visible from this namespace,
            // only consider the most significant name.
            Namespace namespace = getName().getNamespace();
            VariableStore variableStore
                    = controllerState.getVariableStore();
            QualifiedNameMap<Variable> visibleVariables
                    = variableStore
                        .getContent()
                        .getInHierarchy(namespace);
            for (Variable variable : visibleVariables.values())
            {
                QualifiedName qualifiedName = variable.getName();
                context.put(
                        qualifiedName.getName(),
                        variable.getValue());
            }
            log.debug("Put variables in context "
                    + visibleVariables.keySet());
        }
        else
        {
            log.debug("variables in context are disabled");
        }

        return context;
    }
}
