package org.apache.torque.gf.configuration;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.torque.gf.BaseTest;
import org.apache.torque.gf.configuration.controller.GeneratorReference;
import org.apache.torque.gf.configuration.controller.Loglevel;
import org.apache.torque.gf.configuration.controller.Output;
import org.apache.torque.gf.configuration.generator.GeneratorConfiguration;
import org.apache.torque.gf.configuration.mergepoint.MergepointMapping;
import org.apache.torque.gf.configuration.option.MapOptionsConfiguration;
import org.apache.torque.gf.configuration.option.OptionsConfiguration;
import org.apache.torque.gf.configuration.paths.CustomProjectPaths;
import org.apache.torque.gf.configuration.paths.DefaultTorqueGfPaths;
import org.apache.torque.gf.configuration.paths.Maven2DirectoryProjectPaths;
import org.apache.torque.gf.configuration.paths.ProjectPaths;
import org.apache.torque.gf.control.ControllerState;
import org.apache.torque.gf.control.action.ApplyAction;
import org.apache.torque.gf.control.action.MergepointAction;
import org.apache.torque.gf.control.action.TraverseAllAction;
import org.apache.torque.gf.generator.Generator;
import org.apache.torque.gf.java.JavaGenerator;
import org.apache.torque.gf.option.Option;
import org.apache.torque.gf.option.OptionImpl;
import org.apache.torque.gf.option.Options;
import org.apache.torque.gf.qname.QualifiedName;
import org.apache.torque.gf.source.FileSourcesImpl;
import org.apache.torque.gf.source.TransformerDefinition;
import org.apache.torque.gf.source.properties.PropertiesSourceType;
import org.apache.torque.gf.source.xml.XmlSourceType;
import org.apache.torque.gf.template.velocity.VelocityGenerator;
import org.junit.Test;


/**
 * Tests whether the configuration is read correctly.
 */
public class ReadConfigurationTest extends BaseTest
{
    private ControllerState controllerState = new ControllerState();

    @Test
    public void testReadConfiguration() throws Exception
    {
        ProjectPaths projectPaths = new Maven2DirectoryProjectPaths(
                new File("src/test/configuration"));
        UnitDescriptor unitDescriptor = new UnitDescriptor(
                UnitDescriptor.Packaging.DIRECTORY,
                projectPaths,
                new DefaultTorqueGfPaths());
        UnitConfigurationReader configurationReader
                = new UnitConfigurationReader();
        ConfigurationHandlers configurationHandlers
                = new ConfigurationHandlers();
        UnitConfiguration unitConfiguration
                = configurationReader.read(
                        unitDescriptor,
                        configurationHandlers);

        {
            File newFileTargetDirectory
                    = unitConfiguration.getNewFileTargetDirectory();
            File expected = new File(
                    "src/test/configuration/target/generated-sources");
            assertEquals(expected, newFileTargetDirectory);
        }
        {
            File modifiedFileTargetDirectory
                    = unitConfiguration.getModifiedFileTargetDirectory();
            File expected = new File(
                    "src/test/configuration/src/main/generated-sources");
            assertEquals(expected, modifiedFileTargetDirectory);
        }

        assertEquals(Loglevel.DEBUG, unitConfiguration.getLoglevel());

        {
            Options options = unitConfiguration.getOptions();

            Map<QualifiedName, Option> globalScope = options.getGlobalScope();
            assertEquals(4, globalScope.size());

            // check normal options
            {
                QualifiedName optionWithNamespaceQName
                        = new QualifiedName(
                                "org.apache.torque.gf.optionWithNamespace");
                Option optionWithNamespace
                        = globalScope.get(optionWithNamespaceQName);
                assertEquals(
                        new OptionImpl(
                                optionWithNamespaceQName,
                                "optionWithNamespaceValue"),
                        optionWithNamespace);
            }
            {
                QualifiedName optionWithNamespaceQName
                        = new QualifiedName(
                                "org.apache.optionWithNamespace");
                Option optionWithNamespace
                        = globalScope.get(optionWithNamespaceQName);
                assertEquals(
                        new OptionImpl(
                                optionWithNamespaceQName,
                                "org.apache.optionWithNamespaceValue"),
                        optionWithNamespace);
            }
            {
                QualifiedName optionWithNamespaceQName
                        = new QualifiedName(
                                "optionWithoutNamespace", "");
                Option optionWithNamespace
                        = globalScope.get(optionWithNamespaceQName);
                assertEquals(
                        new OptionImpl(
                                optionWithNamespaceQName,
                                "optionWithoutNamespaceValue"),
                        optionWithNamespace);
            }
            // check option from last file takes precedence
            {
                QualifiedName optionWithNamespaceQName
                        = new QualifiedName(
                                "optionPrecedenceLastFile");
                Option optionWithNamespace
                        = globalScope.get(optionWithNamespaceQName);
                assertEquals(
                        new OptionImpl(
                                optionWithNamespaceQName,
                                "value from xml"),
                        optionWithNamespace);
            }
        }

        {
            List<Output> outputFiles = unitConfiguration.getOutputFiles();
            assertEquals(3, outputFiles.size());

            {
                Output outputFile = outputFiles.get(0);
                assertEquals(
                        outputFile.getName(),
                        new QualifiedName("org.apache.torque.gf.firstOutput"));

                {
                    GeneratorReference generatorConfiguration
                            = outputFile.getContentGenerator();
                    assertEquals(
                            new QualifiedName(
                                "org.apache.torque.gf.test.readConfiguration.javaGenerator"),
                            generatorConfiguration.getName());
                }

                assertNull(outputFile.getFilename());
                assertTrue(outputFile.isSkipIfExists());

                {
                    JavaGenerator filenameGenerator
                            = (JavaGenerator) outputFile.getFilenameGenerator();
                    assertEquals("Foo", filenameGenerator.getFoo());
                    assertEquals("Bar", filenameGenerator.getBar());
                    assertEquals(
                            new QualifiedName(
                                    "org.apache.torque.gf.configuration.filenameGenerator"),
                            filenameGenerator.getName());
                }

                {
                    FileSourcesImpl sources = new FileSourcesImpl(
                            new XmlSourceType(),
                            "source",
                            null,
                            null,
                            new ArrayList<TransformerDefinition>(),
                            configurationHandlers,
                            projectPaths.getDefaultSourcePath());
                    assertSourcesEquals(
                            sources,
                            (FileSourcesImpl) outputFile.getSources());
                    // TODO: add source Transformers to config file & check here they are added
                }
            }

            {
                Output outputFile = outputFiles.get(1);
                assertEquals(
                        outputFile.getName(),
                        new QualifiedName("secondOutput"));

                {
                    GeneratorReference generatorConfiguration
                            = outputFile.getContentGenerator();
                    assertEquals(
                            new QualifiedName(
                                "org.apache.torque.gf.test.readConfiguration.anotherGenerator"),
                            generatorConfiguration.getName());
                }


                assertNull(outputFile.getFilename());
                assertFalse(outputFile.isSkipIfExists());

                {
                    VelocityGenerator filenameGenerator
                            = (VelocityGenerator) outputFile.getFilenameGenerator();
                    String templateContent
                            = filenameGenerator.getContent(controllerState);
                    // remove License from template by comparing only
                    // the last line
                    String templateContentLicenseRemoved
                            = StringUtils.substringAfterLast(templateContent, "\r\n");
                    assertEquals(
                            "test template output",
                            templateContentLicenseRemoved);
                    assertEquals(
                            new QualifiedName(
                                    "org.apache.torque.gf.configuration.filenameGenerator"),
                            filenameGenerator.getName());
                    assertTrue(filenameGenerator.isOptionsInContext());
                    assertTrue(filenameGenerator.isSourceAttributesInContext());
                    assertTrue(filenameGenerator.isVariablesInContext());
                }

                {
                    FileSourcesImpl sources = new FileSourcesImpl(
                            new PropertiesSourceType(),
                            "secondSourcePath",
                            "properties/entry",
                            null,
                            new ArrayList<TransformerDefinition>(),
                            configurationHandlers,
                            projectPaths.getDefaultSourcePath());
                    assertSourcesEquals(
                            sources,
                            (FileSourcesImpl) outputFile.getSources());
                }
            }
            {
                Output outputFile = outputFiles.get(2);
                assertEquals(
                        outputFile.getName(),
                        new QualifiedName("thirdOutput"));

                {
                    GeneratorReference generatorConfiguration
                            = outputFile.getContentGenerator();
                    assertEquals(
                            new QualifiedName(
                                "org.apache.torque.gf.test.readConfiguration.thirdGenerator"),
                            generatorConfiguration.getName());
                }


                assertEquals("outputFileName", outputFile.getFilename());
                assertNull(outputFile.getFilenameGenerator());
                assertFalse(outputFile.isSkipIfExists());

                {
                    FileSourcesImpl sources = new FileSourcesImpl(
                            null,
                            "third.source.path.properties",
                            null,
                            null,
                            new ArrayList<TransformerDefinition>(),
                            configurationHandlers,
                            projectPaths.getDefaultSourcePath());
                    assertSourcesEquals(
                            sources,
                            (FileSourcesImpl) outputFile.getSources());
                }
            }
        }

        {
            GeneratorConfiguration generatorConfiguration
                    = unitConfiguration.getGeneratorConfiguration();
            Map <QualifiedName, Generator> generatorMap
                    = generatorConfiguration.getGenerators();
            assertEquals(2, generatorMap.size());
            {
                JavaGenerator generator
                        = (JavaGenerator) generatorMap.get(new QualifiedName(
                                "org.apache.torque.gf.test.readConfiguration.javaGenerator"));
                assertEquals("Foo2", generator.getFoo());
                assertEquals("Bar2", generator.getBar());
                Map<String, MergepointMapping> mergepointMappings
                        = generator.getMergepointMappings();
                assertEquals(3, mergepointMappings.size());
                {
                    MergepointMapping mergepointMapping
                            = mergepointMappings.get("properties");
                    assertEquals(1, mergepointMapping.getActions().size());
                    MergepointAction action
                            = mergepointMapping.getActions().get(0);
                    assertEquals(
                            new TraverseAllAction(
                                    "entry",
                                    "org.apache.torque.gf.velocity.propertyCopy",
                                    true),
                            action);
                    assertEquals("inputElement", generator.getInputElementName());
                }
                {
                    // mergepoint from the separate mapping
                    MergepointMapping mergepointMapping
                            = mergepointMappings.get("mergepointName");
                    assertEquals(1, mergepointMapping.getActions().size());
                    MergepointAction action 
                            = mergepointMapping.getActions().get(0);
                    assertEquals(
                            new ApplyAction(
                                    null,
                                    "someGeneratorAction",
                                    false),
                            action);
                    assertEquals("inputElement", generator.getInputElementName());
                }
                {
                    // other mergepoint from the separate mapping
                    MergepointMapping mergepointMapping
                            = mergepointMappings.get("mergepointFromParent");
                    assertEquals(1, mergepointMapping.getActions().size());
                    MergepointAction action
                            = mergepointMapping.getActions().get(0);
                    assertEquals(
                            new ApplyAction(
                                    null,
                                    "newGeneratorAction",
                                    true),
                            action);
                    assertEquals("inputElement", generator.getInputElementName());
                }
            }

            {
                VelocityGenerator generator
                        = (VelocityGenerator) generatorMap.get(new QualifiedName(
                                "org.apache.torque.gf.test.readConfiguration.anotherGenerator"));
                String templateContent
                        = generator.getContent(controllerState);
                // remove License from template by comparing only
                // the last line
                String templateContentLicenseRemoved
                        = StringUtils.substringAfterLast(templateContent, "\r\n");
                assertEquals(
                        "test template output",
                        templateContentLicenseRemoved);
                assertEquals(0, generator.getMergepointMappings().size());
                assertNull(generator.getInputElementName());
                assertFalse(generator.isOptionsInContext());
                assertFalse(generator.isSourceAttributesInContext());
                assertFalse(generator.isVariablesInContext());
            }
        }
    }

    @Test
    public void testReadConfigFromClasspath() throws Exception
    {
        ProjectPaths projectPaths = new CustomProjectPaths(
                null,
                "org.apache.torque.gf.test.readfromclasspath",
                new File("src"),
                new File("target"),
                new File("generated-src"));
        UnitDescriptor unitDescriptor = new UnitDescriptor(
                UnitDescriptor.Packaging.CLASSPATH,
                projectPaths,
                new DefaultTorqueGfPaths());
        ConfigurationHandlers configurationHandlers
                = new ConfigurationHandlers();
        UnitConfigurationReader configurationReader
                = new UnitConfigurationReader();
        UnitConfiguration unitConfiguration
                = configurationReader.read(
                        unitDescriptor,
                        configurationHandlers);

        // check that we have read the correct configuration
        {
            Options options = unitConfiguration.getOptions();
            Option option  = options.getInHierarchy(
                    new QualifiedName("configuration"));
            assertNotNull("option configuration should be set", option);
            assertEquals("fromClasspath", option.getValue());
        }

        // check that the generators are read
        {
            GeneratorConfiguration generatorConfiguration
                    = unitConfiguration.getGeneratorConfiguration();
            Map <QualifiedName, Generator> generatorMap
                    = generatorConfiguration.getGenerators();
            assertEquals(2, generatorMap.size());
        }
    }

    @Test
    public void testOverrideOptions() throws Exception
    {
        ProjectPaths projectPaths = new Maven2DirectoryProjectPaths(
                new File("src/test/configuration"));
        Map<String, String> overrideOptions = new HashMap<String, String>();
        overrideOptions.put("optionWithoutNamespace", "overriddenValue");
        overrideOptions.put("newOption", "newValue");
        OptionsConfiguration optionConfiguration
                = new MapOptionsConfiguration(overrideOptions);

        UnitDescriptor unitDescriptor = new UnitDescriptor(
                UnitDescriptor.Packaging.DIRECTORY,
                projectPaths,
                new DefaultTorqueGfPaths(),
                null,
                optionConfiguration,
                null);
        ConfigurationHandlers configurationHandlers
                = new ConfigurationHandlers();
        UnitConfigurationReader configurationReader
                = new UnitConfigurationReader();
        UnitConfiguration unitConfiguration
                = configurationReader.read(
                        unitDescriptor,
                        configurationHandlers);
        Options options = unitConfiguration.getOptions();
        {
            Option option = options.getInHierarchy(
                    new QualifiedName("optionWithoutNamespace"));
            assertEquals("overriddenValue", option.getValue());
        }
        {
            Option option = options.getInHierarchy(
                    new QualifiedName("newOption"));
            assertEquals("newValue", option.getValue());
        }
    }


    @Test
    public void testInheritance() throws Exception
    {
        CustomProjectPaths projectPaths;
        UnitConfiguration unitConfiguration;
        ConfigurationHandlers configurationHandlers
                = new ConfigurationHandlers();
        {
            CustomProjectPaths parentProjectPaths
                    = new CustomProjectPaths(
                            new Maven2DirectoryProjectPaths(
                                    new File("src/test/configuration")));
            parentProjectPaths.setConfigurationDir(
                    new File("src/test/configuration/src/main/torque-gf-parent"));
            parentProjectPaths.setNewFileTargetDir(
                    new File("src/test/configuration/target/parentCustom"));
            parentProjectPaths.setModifiedFileTargetDir(
                    new File("src/test/configuration/src/main/parentCustom"));
            UnitDescriptor parentUnitDescriptor = new UnitDescriptor(
                    UnitDescriptor.Packaging.DIRECTORY,
                    parentProjectPaths,
                    new DefaultTorqueGfPaths(),
                    null,
                    null,
                    null);

            projectPaths = new CustomProjectPaths(
                    new Maven2DirectoryProjectPaths(
                            new File("src/test/configuration")));
            projectPaths.setNewFileTargetDir(
                    new File("src/test/configuration/target/custom"));
            projectPaths.setModifiedFileTargetDir(
                    new File("src/test/configuration/src/main/custom"));
            UnitDescriptor unitDescriptor = new UnitDescriptor(
                    UnitDescriptor.Packaging.DIRECTORY,
                    projectPaths,
                    new DefaultTorqueGfPaths(),
                    parentUnitDescriptor,
                    null,
                    null);
            UnitConfigurationReader configurationReader
                    = new UnitConfigurationReader();
            unitConfiguration
                    = configurationReader.read(
                            unitDescriptor,
                            configurationHandlers);
        }

        assertEquals(Loglevel.DEBUG, unitConfiguration.getLoglevel());

        {
            File newFileTargetDirectory
                    = unitConfiguration.getNewFileTargetDirectory();
            File expected = new File("src/test/configuration/target/custom");
            assertEquals(expected, newFileTargetDirectory);
        }
        {
            File modifiedFileTargetDirectory
                    = unitConfiguration.getModifiedFileTargetDirectory();
            File expected = new File("src/test/configuration/src/main/custom");
            assertEquals(expected, modifiedFileTargetDirectory);
        }

        {
            Options options = unitConfiguration.getOptions();
            Map<QualifiedName, Option> globalScope = options.getGlobalScope();
            assertEquals(6, globalScope.size());

            {
                QualifiedName optionWithoutNamespaceQName
                        = new QualifiedName("optionWithoutNamespace", "");
                Option optionWithoutNamespace
                        = globalScope.get(optionWithoutNamespaceQName);
                assertEquals(
                        new OptionImpl(
                                optionWithoutNamespaceQName,
                                "optionWithoutNamespaceValue"),
                        optionWithoutNamespace);
            }
            {
                QualifiedName optionWithNamespaceQName
                        = new QualifiedName(
                                "org.apache.torque.gf.optionWithNamespace");
                Option optionWithNamespace
                        = globalScope.get(optionWithNamespaceQName);
                assertEquals(
                        new OptionImpl(
                                optionWithNamespaceQName,
                                "optionWithNamespaceValue"),
                        optionWithNamespace);
            }
            {
                QualifiedName optionWithoutNamespaceQName
                        = new QualifiedName("parentOptionWithoutNamespace", "");
                Option optionWithoutNamespace
                        = globalScope.get(optionWithoutNamespaceQName);
                assertEquals(
                        new OptionImpl(
                                optionWithoutNamespaceQName,
                                "parentOptionWithoutNamespaceParentValue"),
                        optionWithoutNamespace);
            }
            {
                QualifiedName optionWithNamespaceQName
                        = new QualifiedName(
                                "org.apache.torque.gf.parentOptionWithNamespace");
                Option optionWithNamespace
                        = globalScope.get(optionWithNamespaceQName);
                assertEquals(
                        new OptionImpl(
                                optionWithNamespaceQName,
                                "parentOptionWithNamespaceParentValue"),
                        optionWithNamespace);
            }
        }
        {
            List<Output> outputFiles = unitConfiguration.getOutputFiles();
            assertEquals(4, outputFiles.size());

            {
                Output outputFile = outputFiles.get(0);
                assertEquals(
                        outputFile.getName(),
                        new QualifiedName("parentOutput"));
                {
                    GeneratorReference generatorConfiguration
                            = outputFile.getContentGenerator();
                    assertEquals(
                            new QualifiedName(
                                "org.apache.torque.gf.test.readConfiguration.testParentGenerator"),
                            generatorConfiguration.getName());
                }


                assertNull(outputFile.getFilename());

                {
                    JavaGenerator filenameGenerator
                            = (JavaGenerator) outputFile.getFilenameGenerator();
                    assertEquals("ParentFoo", filenameGenerator.getFoo());
                    assertEquals("ParentBar", filenameGenerator.getBar());
                    assertEquals(
                            new QualifiedName(
                                    "org.apache.torque.gf.configuration.filenameGenerator"),
                            filenameGenerator.getName());
                }

                {
                    FileSourcesImpl sources = new FileSourcesImpl(
                            new XmlSourceType(),
                            "parentSource",
                            null,
                            null,
                            new ArrayList<TransformerDefinition>(),
                            configurationHandlers,
                            projectPaths.getDefaultSourcePath());
                    assertSourcesEquals(
                            sources,
                            (FileSourcesImpl) outputFile.getSources());
                }
            }
            {
                Output outputFile = outputFiles.get(1);
                assertEquals(
                        outputFile.getName(),
                        new QualifiedName("org.apache.torque.gf.firstOutput"));

                {
                    GeneratorReference generatorConfiguration
                            = outputFile.getContentGenerator();
                    assertEquals(
                            new QualifiedName(
                                "org.apache.torque.gf.test.readConfiguration.javaGenerator"),
                            generatorConfiguration.getName());
                }


                assertNull(outputFile.getFilename());

                {
                    JavaGenerator filenameGenerator
                            = (JavaGenerator) outputFile.getFilenameGenerator();
                    assertEquals("Foo", filenameGenerator.getFoo());
                    assertEquals("Bar", filenameGenerator.getBar());
                    assertEquals(
                            new QualifiedName(
                                    "org.apache.torque.gf.configuration.filenameGenerator"),
                            filenameGenerator.getName());
                }

                {
                    FileSourcesImpl sources = new FileSourcesImpl(
                            new XmlSourceType(),
                            "source",
                            null,
                            null,
                            new ArrayList<TransformerDefinition>(),
                            configurationHandlers,
                            projectPaths.getDefaultSourcePath());
                    assertSourcesEquals(
                            sources,
                            (FileSourcesImpl) outputFile.getSources());
                    // TODO: add source Transformers to config file & check here they are added
                }
            }

            {
                Output outputFile = outputFiles.get(2);
                assertEquals(
                        outputFile.getName(),
                        new QualifiedName("secondOutput"));

                {
                    GeneratorReference generatorConfiguration
                            = outputFile.getContentGenerator();
                    assertEquals(
                            new QualifiedName(
                                "org.apache.torque.gf.test.readConfiguration.anotherGenerator"),
                            generatorConfiguration.getName());
                }


                assertNull(outputFile.getFilename());

                {
                    VelocityGenerator filenameGenerator
                            = (VelocityGenerator) outputFile.getFilenameGenerator();
                    String templateContent
                            = filenameGenerator.getContent(controllerState);
                    // remove License from template by comparing only
                    // the last line
                    String templateContentLicenseRemoved
                            = StringUtils.substringAfterLast(templateContent, "\r\n");
                    assertEquals(
                            "test template output",
                            templateContentLicenseRemoved);
                    assertEquals(
                            new QualifiedName(
                                    "org.apache.torque.gf.configuration.filenameGenerator"),
                            filenameGenerator.getName());
                }

                {
                    FileSourcesImpl sources = new FileSourcesImpl(
                            new PropertiesSourceType(),
                            "secondSourcePath",
                            "properties/entry",
                            null,
                            new ArrayList<TransformerDefinition>(),
                            configurationHandlers,
                            projectPaths.getDefaultSourcePath());
                    assertSourcesEquals(
                            sources,
                            (FileSourcesImpl) outputFile.getSources());
                }
            }
            {
                Output outputFile = outputFiles.get(3);
                assertEquals(
                        outputFile.getName(),
                        new QualifiedName("thirdOutput"));

                {
                    GeneratorReference generatorConfiguration
                            = outputFile.getContentGenerator();
                    assertEquals(
                            new QualifiedName(
                                "org.apache.torque.gf.test.readConfiguration.thirdGenerator"),
                            generatorConfiguration.getName());
                }

                assertEquals("outputFileName", outputFile.getFilename());
                assertNull(outputFile.getFilenameGenerator());
                {
                    FileSourcesImpl sources = new FileSourcesImpl(
                            null,
                            "third.source.path.properties",
                            null,
                            null,
                            new ArrayList<TransformerDefinition>(),
                            configurationHandlers,
                            projectPaths.getDefaultSourcePath());
                    assertSourcesEquals(
                            sources,
                            (FileSourcesImpl) outputFile.getSources());
                }
            }
        }

        {
            GeneratorConfiguration generatorConfiguration
                    = unitConfiguration.getGeneratorConfiguration();
            Map <QualifiedName, Generator> generatorMap
                    = generatorConfiguration.getGenerators();
            assertEquals(3, generatorMap.size());
            {
                JavaGenerator generator
                        = (JavaGenerator) generatorMap.get(new QualifiedName(
                                "org.apache.torque.gf.test.readConfiguration.javaGenerator"));
                assertEquals("Foo2", generator.getFoo());
                assertEquals("Bar2", generator.getBar());
                Map<String,MergepointMapping> mergepointMappings
                        = generator.getMergepointMappings();
                assertEquals(3, mergepointMappings.size());
                {
                    MergepointMapping mergepointMapping 
                            = mergepointMappings.get("properties");
                    assertEquals(1, mergepointMapping.getActions().size());
                    MergepointAction action 
                            = mergepointMapping.getActions().get(0);
                    assertEquals(
                            new TraverseAllAction(
                                    "entry",
                                    "org.apache.torque.gf.velocity.propertyCopy",
                                    true),
                            action);
                    assertEquals("inputElement", generator.getInputElementName());                    
                }
                {
                    // mergepoint from the separate mapping in child
                    MergepointMapping mergepointMapping 
                            = mergepointMappings.get("mergepointName");
                    assertEquals(1, mergepointMapping.getActions().size());
                    MergepointAction action 
                            = mergepointMapping.getActions().get(0);
                    assertEquals(
                            new ApplyAction(
                                    null,
                                    "someGeneratorAction",
                                    false),
                            action);
                    assertEquals("inputElement", generator.getInputElementName());
                }
                {
                    // other mergepoint from the separate mapping in child
                    MergepointMapping mergepointMapping 
                            = mergepointMappings.get("mergepointFromParent");
                    assertEquals(1, mergepointMapping.getActions().size());
                    MergepointAction action 
                            = mergepointMapping.getActions().get(0);
                    assertEquals(
                            new ApplyAction(
                                    null,
                                    "newGeneratorAction",
                                    true),
                            action);
                    assertEquals("inputElement", generator.getInputElementName());
                }
            }

            {
                VelocityGenerator generator
                        = (VelocityGenerator) generatorMap.get(new QualifiedName(
                                "org.apache.torque.gf.test.readConfiguration.anotherGenerator"));
                String templateContent
                        = generator.getContent(controllerState);
                // remove License from template by comparing only
                // the last line
                String templateContentLicenseRemoved
                        = StringUtils.substringAfterLast(templateContent, "\r\n");
                assertEquals(
                        "test template output",
                        templateContentLicenseRemoved);
                assertEquals(0, generator.getMergepointMappings().size());
            }
            {
                VelocityGenerator generator
                        = (VelocityGenerator) generatorMap.get(new QualifiedName(
                                "org.apache.torque.gf.test.readConfiguration.parentGenerator"));
                String templateContent
                        = generator.getContent(controllerState);
                // remove License from template by comparing only
                // the last line
                String templateContentLicenseRemoved
                = StringUtils.substringAfterLast(templateContent, "\r\n");
                assertEquals(
                        "parent test template output",
                        templateContentLicenseRemoved);
                assertEquals(0, generator.getMergepointMappings().size());
            }
        }
    }

    private static void assertSourcesEquals(
            FileSourcesImpl expected,
            FileSourcesImpl actual)
    {
        if (expected.getSourceFilter() != null)
        {
            assertEquals(
                    expected.getSourceFilter().getClass(),
                    actual.getSourceFilter().getClass());
        }
        else
        {
            assertNull(actual.getSourceFilter());
        }
        assertEquals(
                expected.getStartElementsPath(),
                actual.getStartElementsPath());
        assertEquals(
                expected.getTransformerDefinitions(),
                actual.getTransformerDefinitions());
        assertEquals(
                expected.getType(),
                actual.getType());
        assertEquals(
                expected.getPath(),
                actual.getPath());
        assertEquals(
                expected.getPaths(),
                actual.getPaths());
    }
}
