package org.apache.torque.gf.source;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import org.apache.torque.gf.configuration.ConfigurationException;
import org.apache.torque.gf.configuration.ConfigurationHandlers;
import org.apache.torque.gf.source.properties.PropertiesSourceType;
import org.apache.torque.gf.source.xml.XmlSourceType;
import org.junit.Test;

public class FileSourcesImplTest
{
    private ConfigurationHandlers configurationHandler 
        = new ConfigurationHandlers();

    @Test
    public void testGetDirsNoWildcards()
    {
        String path = "/tmp/schema.xml";
        int baseDirSeparatorPos = FileSourcesImpl.getBaseDirSeparatorPos(path);
        assertEquals(4, baseDirSeparatorPos);
        String baseDir = FileSourcesImpl.getBaseDir(path, baseDirSeparatorPos);
        assertEquals("/tmp", baseDir);
        String wildcardPattern
                = FileSourcesImpl.getWildcardPattern(path, baseDirSeparatorPos);
        assertEquals("schema.xml", wildcardPattern);
    }

    @Test
    public void testGetBaseDirOnlyFilename()
    {
        String path = "schema.xml";
        int baseDirSeparatorPos = FileSourcesImpl.getBaseDirSeparatorPos(path);
        assertEquals(-1, baseDirSeparatorPos);
        String baseDir = FileSourcesImpl.getBaseDir(path, baseDirSeparatorPos);
        assertEquals(".", baseDir);
        String wildcardPattern
                = FileSourcesImpl.getWildcardPattern(path, baseDirSeparatorPos);
        assertEquals("schema.xml", wildcardPattern);
    }

    @Test
    public void testGetBaseDirQuestionMark()
    {
        String path = "C:\\schema\\?\\schema.xml";
        int baseDirSeparatorPos = FileSourcesImpl.getBaseDirSeparatorPos(path);
        assertEquals(9, baseDirSeparatorPos);
        String baseDir = FileSourcesImpl.getBaseDir(path, baseDirSeparatorPos);
        assertEquals("C:\\schema", baseDir);
        String wildcardPattern
                = FileSourcesImpl.getWildcardPattern(path, baseDirSeparatorPos);
        assertEquals("?\\schema.xml", wildcardPattern);
    }

    @Test
    public void testGetBaseDirAsterisk()
    {
        String path = "C:\\schema\\*\\schema.xml";
        int baseDirSeparatorPos = FileSourcesImpl.getBaseDirSeparatorPos(path);
        assertEquals(9, baseDirSeparatorPos);
        String baseDir = FileSourcesImpl.getBaseDir(path, baseDirSeparatorPos);
        assertEquals("C:\\schema", baseDir);
        String wildcardPattern
                = FileSourcesImpl.getWildcardPattern(path, baseDirSeparatorPos);
        assertEquals("*\\schema.xml", wildcardPattern);
    }

    @Test
    public void testGetBaseDirMultipleWildcards()
    {
        String path = "/tmp/*/???/schema.xml";
        int baseDirSeparatorPos = FileSourcesImpl.getBaseDirSeparatorPos(path);
        assertEquals(4, baseDirSeparatorPos);
        String baseDir = FileSourcesImpl.getBaseDir(path, baseDirSeparatorPos);
        assertEquals("/tmp", baseDir);
        String wildcardPattern
                = FileSourcesImpl.getWildcardPattern(path, baseDirSeparatorPos);
        assertEquals("*/???/schema.xml", wildcardPattern);
    }

    @Test
    public void testGetBaseDirWildcardFirst()
    {
        String path = "*/???/schema.xml";
        int baseDirSeparatorPos = FileSourcesImpl.getBaseDirSeparatorPos(path);
        assertEquals(-1, baseDirSeparatorPos);
        String baseDir = FileSourcesImpl.getBaseDir(path, baseDirSeparatorPos);
        assertEquals(".", baseDir);
        String wildcardPattern
                = FileSourcesImpl.getWildcardPattern(path, baseDirSeparatorPos);
        assertEquals("*/???/schema.xml", wildcardPattern);
    }

    @Test
    public void testGetSourcesFixedNameBasedir() throws ConfigurationException
    {
        FileSourcesImpl fileSourcesImpl = new FileSourcesImpl(
                null,
                "1.properties",
                null,
                null,
                new ArrayList<TransformerDefinition>(),
                configurationHandler,
                new File("src/test/fileSourcesImpl"));
        Set<File> paths = new HashSet<File>();
        while (fileSourcesImpl.hasNext())
        {
            FileSource fileSource = (FileSource) fileSourcesImpl.next();
            paths.add(fileSource.getPath());
        }
        Set<File> expectedFiles = new HashSet<File>();
        expectedFiles.add(new File("src/test/fileSourcesImpl/./1.properties"));
        assertEquals(expectedFiles.size(), paths.size());
        assertEquals(expectedFiles, paths);
    }

    @Test
    public void testGetSourcesFixedNameSubdir() throws ConfigurationException
    {
        FileSourcesImpl fileSourcesImpl = new FileSourcesImpl(
                null,
                "subfolder/2.properties",
                null,
                null,
                new ArrayList<TransformerDefinition>(),
                configurationHandler,
                new File("src/test/fileSourcesImpl"));
        Set<File> paths = new HashSet<File>();
        while (fileSourcesImpl.hasNext())
        {
            FileSource fileSource = (FileSource) fileSourcesImpl.next();
            paths.add(fileSource.getPath());
        }
        Set<File> expectedFiles = new HashSet<File>();
        expectedFiles.add(
                new File("src/test/fileSourcesImpl/subfolder/2.properties"));
        assertEquals(expectedFiles.size(), paths.size());
        assertEquals(expectedFiles, paths);
    }

    @Test
    public void testGetSourcesMixedSlashBackslash() throws ConfigurationException
    {
        FileSourcesImpl fileSourcesImpl = new FileSourcesImpl(
                null,
                "subfolder/subsubfolder\\3.properties",
                null,
                null,
                new ArrayList<TransformerDefinition>(),
                configurationHandler,
                new File("src/test/fileSourcesImpl"));
        Set<File> paths = new HashSet<File>();
        while (fileSourcesImpl.hasNext())
        {
            FileSource fileSource = (FileSource) fileSourcesImpl.next();
            paths.add(fileSource.getPath());
        }
        Set<File> expectedFiles = new HashSet<File>();
        expectedFiles.add(
                new File("src/test/fileSourcesImpl/subfolder/subsubfolder/3.properties"));
        assertEquals(expectedFiles.size(), paths.size());
        assertEquals(expectedFiles, paths);
    }

    @Test
    public void testGetSourcesWildcardDir() throws ConfigurationException
    {
        FileSourcesImpl fileSourcesImpl = new FileSourcesImpl(
                null,
                "*//2.properties",
                null,
                null,
                new ArrayList<TransformerDefinition>(),
                configurationHandler,
                new File("src/test/fileSourcesImpl"));
        Set<File> paths = new HashSet<File>();
        while (fileSourcesImpl.hasNext())
        {
            FileSource fileSource = (FileSource) fileSourcesImpl.next();
            paths.add(fileSource.getPath());
        }
        Set<File> expectedFiles = new HashSet<File>();
        expectedFiles.add(
                new File("src/test/fileSourcesImpl/./subfolder/2.properties"));
        assertEquals(expectedFiles.size(), paths.size());
        assertEquals(expectedFiles, paths);
    }

    @Test
    public void testGetSourcesQuestionmarkFilename()
            throws ConfigurationException
    {
        FileSourcesImpl fileSourcesImpl = new FileSourcesImpl(
                null,
                "?.properties",
                null,
                null,
                new ArrayList<TransformerDefinition>(),
                configurationHandler,
                new File("src/test/fileSourcesImpl"));
        Set<File> paths = new HashSet<File>();
        while (fileSourcesImpl.hasNext())
        {
            FileSource fileSource = (FileSource) fileSourcesImpl.next();
            paths.add(fileSource.getPath());
        }
        Set<File> expectedFiles = new HashSet<File>();
        expectedFiles.add(
                new File("src/test/fileSourcesImpl/./1.properties"));
        assertEquals(expectedFiles.size(), paths.size());
        assertEquals(expectedFiles, paths);
    }

    @Test
    public void testGetSourcesWildcardFilename()
            throws ConfigurationException
    {
        FileSourcesImpl fileSourcesImpl = new FileSourcesImpl(
                null,
                "*.properties",
                null,
                null,
                new ArrayList<TransformerDefinition>(),
                configurationHandler,
                new File("src/test/fileSourcesImpl"));
        Set<File> paths = new HashSet<File>();
        while (fileSourcesImpl.hasNext())
        {
            FileSource fileSource = (FileSource) fileSourcesImpl.next();
            paths.add(fileSource.getPath());
        }
        Set<File> expectedFiles = new HashSet<File>();
        expectedFiles.add(
                new File("src/test/fileSourcesImpl/./1.properties"));
        expectedFiles.add(
                new File("src/test/fileSourcesImpl/./11.properties"));
        assertEquals(expectedFiles.size(), paths.size());
        assertEquals(expectedFiles, paths);
    }

    @Test
    public void testGetSourcesDoubleDotsStayingInSourceDir()
            throws ConfigurationException
    {
        FileSourcesImpl fileSourcesImpl = new FileSourcesImpl(
                null,
                "subfolder/../1.properties",
                null,
                null,
                new ArrayList<TransformerDefinition>(),
                configurationHandler,
                new File("src/test/fileSourcesImpl"));
        Set<File> paths = new HashSet<File>();
        while (fileSourcesImpl.hasNext())
        {
            FileSource fileSource = (FileSource) fileSourcesImpl.next();
            paths.add(fileSource.getPath());
        }
        Set<File> expectedFiles = new HashSet<File>();
        expectedFiles.add(
                new File("src/test/fileSourcesImpl/subfolder/../1.properties"));
        assertEquals(expectedFiles.size(), paths.size());
        assertEquals(expectedFiles, paths);
    }

    @Test
    public void testGetSourcesDoubleDotsLeavingSourceDir()
            throws ConfigurationException
    {
        FileSourcesImpl fileSourcesImpl = new FileSourcesImpl(
                null,
                "../1.properties",
                null,
                null,
                new ArrayList<TransformerDefinition>(),
                configurationHandler,
                new File("src/test/fileSourcesImpl/subfolder"));
        Set<File> paths = new HashSet<File>();
        while (fileSourcesImpl.hasNext())
        {
            FileSource fileSource = (FileSource) fileSourcesImpl.next();
            paths.add(fileSource.getPath());
        }
        Set<File> expectedFiles = new HashSet<File>();
        expectedFiles.add(
                new File("src/test/fileSourcesImpl/subfolder/../1.properties"));
        assertEquals(expectedFiles.size(), paths.size());
        assertEquals(expectedFiles, paths);
    }
}
