package org.apache.torque.gf.source;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertSame;
import static org.junit.Assert.fail;

import java.io.File;
import java.util.ArrayList;

import org.apache.torque.gf.configuration.paths.Maven2DirectoryProjectPaths;
import org.apache.torque.gf.configuration.paths.ProjectPaths;
import org.apache.torque.gf.source.properties.PropertiesSourceType;
import org.junit.Before;
import org.junit.Test;

public class RichSourceElementImplTest
{
    private RichSourceElementImpl rootSourceElement;

    @Before
    public void setUp()
    {
        rootSourceElement = new RichSourceElementImpl("parent");
        RichSourceElementImpl child1 = new RichSourceElementImpl("child1");
        rootSourceElement.addChild(child1);
        RichSourceElementImpl child2 = new RichSourceElementImpl("child2");
        rootSourceElement.addChild(child2);
        RichSourceElementImpl child3 = new RichSourceElementImpl("child3");
        rootSourceElement.addChild(child3);
    }


    @Test
    public void testAddChild()
    {
        SourceElement child1 = rootSourceElement.getChildren().get(0);
        SourceElement child2 = rootSourceElement.getChildren().get(1);
        SourceElement child3 = rootSourceElement.getChildren().get(2);
        RichSourceElementImpl child4 = new RichSourceElementImpl("child4");
        rootSourceElement.addChild(child4);

        assertEquals("parent should have 4 children",
                4,
                rootSourceElement.getChildren().size());
        assertSame(child1, rootSourceElement.getChildren().get(0));
        assertSame(child2, rootSourceElement.getChildren().get(1));
        assertSame(child3, rootSourceElement.getChildren().get(2));
        assertSame(child4, rootSourceElement.getChildren().get(3));
        assertSame("child2 has wrong parent",
                rootSourceElement,
                child4.getParent());
    }

    @Test
    public void testAddChildAtIndex()
    {
        SourceElement child1 = rootSourceElement.getChildren().get(0);
        SourceElement child2 = rootSourceElement.getChildren().get(1);
        SourceElement child3 = rootSourceElement.getChildren().get(2);
        RichSourceElementImpl child4 = new RichSourceElementImpl("child4");
        rootSourceElement.addChild(1, child4);
        assertSame(child1, rootSourceElement.getChildren().get(0));
        assertSame(child4, rootSourceElement.getChildren().get(1));
        assertSame(child2, rootSourceElement.getChildren().get(2));
        assertSame(child3, rootSourceElement.getChildren().get(3));
    }

    @Test
    public void testRemoveChild()
    {
        SourceElement child1 = rootSourceElement.getChildren().get(0);
        SourceElement child2 = rootSourceElement.getChildren().get(1);
        SourceElement child3 = rootSourceElement.getChildren().get(2);
        SourceElement result = rootSourceElement.removeChild(child2);
        assertSame(child2, result);
        assertEquals("parent should have 2 children",
                2,
                rootSourceElement.getChildren().size());
        assertSame(child1, rootSourceElement.getChildren().get(0));
        assertSame(child3, rootSourceElement.getChildren().get(1));
    }

    @Test
    public void testRemoveNonExistingChild()
    {
        SourceElement child1 = rootSourceElement.getChildren().get(0);
        SourceElement child2 = rootSourceElement.getChildren().get(1);
        SourceElement child3 = rootSourceElement.getChildren().get(2);
        RichSourceElementImpl child4 = new RichSourceElementImpl("child4");

        SourceElement result = rootSourceElement.removeChild(child4);
        assertNull(result);
        assertEquals("parent should have 3 children",
                3,
                rootSourceElement.getChildren().size());
        assertSame(child1, rootSourceElement.getChildren().get(0));
        assertSame(child2, rootSourceElement.getChildren().get(1));
        assertSame(child3, rootSourceElement.getChildren().get(2));
    }

    @Test
    public void testGetChildIndex()
    {
        SourceElement child2 = rootSourceElement.getChildren().get(1);

        assertEquals("child index should be 1",
                1,
                rootSourceElement.getChildIndex(child2));
    }

    @Test
    public void testGetNonExistingChildIndex()
    {
        RichSourceElementImpl child4 = new RichSourceElementImpl("child4");
        assertEquals("child index should be -1",
                -1,
                rootSourceElement.getChildIndex(child4));
    }

    @Test
    public void testAddParent()
    {
        RichSourceElementImpl child = new RichSourceElementImpl("chiild");
        RichSourceElementImpl parent1 = new RichSourceElementImpl("parent1");
        child.addParent(parent1);
        RichSourceElementImpl parent2 = new RichSourceElementImpl("parent2");
        child.addParent(parent2);

        assertEquals("child should have 2 parents",
                2,
                child.getParents().size());
        assertSame("child has wrong primary parent",
                parent1,
                child.getParent());
        assertSame("parent1 has wrong child",
                child,
                parent1.getChildren().get(0));
        assertSame("child has wrong second parent",
                parent2,
                child.getParents().get(1));
        assertSame("parent2 has wrong child",
                child,
                parent2.getChildren().get(0));
    }

    @Test
    public void testTreeEquals()
    {
        RichSourceElementImpl original = new RichSourceElementImpl("root");
        SourceElement toCompare
                = new RichSourceElementImpl("wrongName");
        if (original.graphEquals(toCompare))
        {
            fail("SourceElements with different names must not be equal");
        }

        toCompare = new RichSourceElementImpl("root");
        if (!original.graphEquals(toCompare))
        {
            fail("empty SourceElements with equal names must be equal");
        }

        original.setAttribute("attributeName", "attributeValue");
        if (original.graphEquals(toCompare))
        {
            fail("SourceElements with different attribute names"
                    + " must not be equal");
        }

        toCompare.setAttribute("attributeName", "anotherAttributeValue");
        if (original.graphEquals(toCompare))
        {
            fail("SourceElements with different attribute content"
                    + " must not be equal(1)");
        }

        toCompare.setAttribute("attributeName", null);
        if (original.graphEquals(toCompare))
        {
            fail("SourceElements with different attribute content"
                    + " must not be equal(2)");
        }

        toCompare.setAttribute("attributeName", "attributeValue");
        if (!original.graphEquals(toCompare))
        {
            fail("SourceElements with equal attribute content"
                    + " must be equal(1)");
        }

        original.setAttribute("attributeName", null);
        if (original.graphEquals(toCompare))
        {
            fail("SourceElements with different attribute content"
                    + " must not be equal(3)");
        }

        toCompare.setAttribute("attributeName", null);
        if (!original.graphEquals(toCompare))
        {
            fail("SourceElements with equal attribute content"
                    + " must be equal(2)");
        }

        original.addChild(new RichSourceElementImpl("child"));
        if (original.graphEquals(toCompare))
        {
            fail("SourceElements with different children"
                    + " must not be equal(1)");
        }

        toCompare.addChild(new RichSourceElementImpl("wrongChild"));
        if (original.graphEquals(toCompare))
        {
            fail("SourceElements with different children"
                    + " must not be equal(2)");
        }

        toCompare = new RichSourceElementImpl("root");
        toCompare.setAttribute("attributeName", null);
        toCompare.addChild(new RichSourceElementImpl("child"));
        if (!original.graphEquals(toCompare))
        {
            fail("SourceElements with equal children"
                    + " must be equal(1)");
        }

        original.addChild(original);
        if (original.graphEquals(toCompare))
        {
            fail("SourceElements with different children"
                    + " must not be equal(3)");
        }

        toCompare.addChild(toCompare);
        if (!original.graphEquals(toCompare))
        {
            fail("SourceElements with equal children"
                    + " must be equal(2)");
        }
    }
    
    @Test
    public void testSetAttribute()
    {
        SourceElement sourceElement = new RichSourceElementImpl("root");
        assertEquals(sourceElement.getAttributeNames().size(), 0);
        assertEquals(null, sourceElement.getAttribute("attributeName"));

        sourceElement.setAttribute("attributeName", "123");
        assertEquals(1, sourceElement.getAttributeNames().size());
        assertEquals("123", sourceElement.getAttribute("attributeName"));
    }

    @Test
    public void testSetAttributeToNull()
    {
        SourceElement sourceElement = new RichSourceElementImpl("root");
        sourceElement.setAttribute("attributeName", "123");
        assertEquals(1, sourceElement.getAttributeNames().size());

        sourceElement.setAttribute("attributeName", null);
        assertEquals(0, sourceElement.getAttributeNames().size());
    }

    @Test
    public void testToString() throws Exception
    {
        SourceElement sourceElement = new RichSourceElementImpl("root");
        sourceElement.setAttribute("attributeName", "123");
        sourceElement.addChild(new RichSourceElementImpl("child"));
        sourceElement.addChild(sourceElement);
        String expected
                = "(name=root,attributes=(attributeName=123),"
                    + "children=((name=child,attributes=(),children=()),"
                    + "<<loop detected>>))";
        assertEquals(expected, sourceElement.toString());
    }

    @Test
    public void testCopySourceElement() throws Exception
    {
        ProjectPaths projectPaths
                = new Maven2DirectoryProjectPaths(
                    new File("src/test/propertyToJava"));
        File propertiesFile
                = new File(
                    projectPaths.getDefaultSourcePath(),
                    "propertiesData.properties");
        FileSource fileSource
                = new FileSource(
                        new PropertiesSourceType(),
                        propertiesFile,
                        null,
                        null,
                        new ArrayList<TransformerDefinition>());
        RichSourceElementImpl rootElement
                = (RichSourceElementImpl) fileSource.getRootElement();

        SourceElement copiedRootElement = rootElement.copy();
        if (!rootElement.graphEquals(copiedRootElement))
        {
            fail("copied tree does not equal original tree");
        }
    }
}
